﻿local _,ns = ...
ns.Ants = ns:RegisterModule(nil,"Ants")

ns.Ants.minimap = {
	count = 1,
	x_shift = 0,
	y_shift = 0,
}
ns.Ants.worldmap = {
	count = 1,
	x_shift = 0,
	y_shift = 0,
}

ns.Ants.vars = {
	speed = 2.0,
	space = 20,
	show = false,
	minimapShow = false,
	mapVisible = false,
	minimap = {
		shown = false,
	},
	worldmap = {
		shown = false,
	},
}

local x_shift, y_shift = 0, 0
local ants, mapAnts, vars = ns.Ants.minimap, ns.Ants.worldmap, ns.Ants.vars
mapAnts.last_count, ants.last_count = 0, 0 -- store how many ants was used last time 

local Astrolabe = DongleStub("Astrolabe-1.0") 

local mm_icon = CreateFrame("Frame", nil, Minimap)
mm_icon:SetSize(1,1)
local wm_icon = CreateFrame("Frame", nil, WorldMapButton)
wm_icon:SetSize(1,1)

local active_point

function ns.Ants:IsActive()
	if active_point then
		return true
	else
		return false
	end
end

function ns.Ants:SetMFPoint(m, f, x, y)
	if type(m)~="number" or type(x)~="number" or type(y)~="number" then
		m:mdebug("WARNING: Attempt to set ants to invalid point. Call ignored.")
		return
	end
	
	Astrolabe:PlaceIconOnMinimap(mm_icon, m, f, x, y)
	Astrolabe:PlaceIconOnWorldMap(WorldMapButton, wm_icon, m, f, x, y)
	active_point = {m = m, f = f, x = x, y = y}
	--active_point = true; --for now, lets see if we need more
end

function ns.Ants:ClearPoint()
	active_point = nil;
	Astrolabe:RemoveIconFromMinimap(mm_icon)
	wm_icon:Hide()
	
	ants.x_shift = 0
	ants.y_shift = 0
	mapAnts.x_shift = 0
	mapAnts.y_shift = 0
end

local mapLayer = CreateFrame("Frame", "ManaCoreMapOverlay", WorldMapButton)
mapLayer:SetAllPoints()
mapLayer:Show()

--these are still here because of legacy and laziness
local waypoint,waypointMap
waypoint = mm_icon
waypointMap = wm_icon
local waypointPlayer = CreateFrame("Frame", nil, mapLayer)

local function getPlayerPosition()
	--[[local x,y = GetPlayerMapPosition("player")
	local left,bottom,width,height = WorldMapDetailFrame:GetRect()
	local pX = (x * width)
	local pY = (-y * height)]]--
	if WorldMapPlayer:IsVisible() then
		local pX,pY = select(4,WorldMapPlayer:GetPoint())
		return pX,pY
	end
end

local function distance(p1 ,p2) return sqrt(p1^2 + p2^2) end

function ants:Hide()
	if not self.hidden then 
	for i,v in ipairs(self) do v:Hide() end
		self.hidden = true
	end
end
mapAnts.Hide = ants.Hide

local function createAnt(map)
	local ant = CreateFrame("Frame", nil, map and mapLayer or Minimap)
	ant:SetHeight(8)
	ant:SetWidth(8)
	ant.icon = ant:CreateTexture(nil,"OVERLAY")
	ant.icon:SetTexture("Interface\\AddOns\\ManaCore\\Media\\Ant")
	ant.icon:SetPoint("CENTER", 0, 0)
	ant.icon:SetAlpha(0.5)
	ant.icon:SetHeight(6)
	ant.icon:SetWidth(6)
	return ant
end

local function updateAnts(point, relativeTo, relativePoint, tab, wayX, wayY, originX, originY, onMap)
	--print(point, relativeTo, relativePoint, tab, wayX, wayY, originX, originY, onMap)
	originX, originY = originX or 0, originY or 0
	
	local d = distance(wayX, wayY)							-- Total distance of the ant line according to rotation
	tab.rx, tab.ry  = (vars.space * wayX / d), (vars.space * wayY / d)	-- Spacing of ants with rotation figured in
	tab.x_shift = tab.x_shift + (vars.speed * wayX / d)				-- Pixel shift of ants with rotation figured in (X axis)
	tab.y_shift = (wayY * tab.x_shift) / wayX					-- Pixel shift of ants with rotation figured in (Y axis)
	tab.lx, tab.ly  = tab.x_shift, tab.y_shift							-- Offset of current iteration
	local count = 1	-- Current iteration
	local dd = 0		-- Actual distance from origin of current iteration
	
	while dd < d and d > vars.space do
		tab[count] = tab[count] or createAnt(onMap)
		tab[count]:SetPoint(point, relativeTo, relativePoint, tab.lx + originX, tab.ly + originY)
		tab[count]:Show()
		
		-- Set offsets for next iteration
		tab.lx = tab.lx + tab.rx
		tab.ly = tab.ly + tab.ry
		dd = distance(tab.lx,tab.ly)
		count = count + 1
	end
	if count > 0 then tab.hidden = false end
	for i = count, tab.last_count do if tab[i] then tab[i]:Hide() end end
	if distance(tab.x_shift, tab.y_shift) > vars.space then tab.x_shift = tab.x_shift - tab.rx end
	tab.last_count = count
end

local function updateMinimapAnts()
	if not vars.minimap.shown then return ants:Hide() end
	local point, relativeTo, relativePoint, wayX, wayY = waypoint:GetPoint()
	if not wayX then return ants:Hide() end
	updateAnts(point, relativeTo, relativePoint, ants, wayX, wayY)
end

local function updateMapAnts()
	local point, relativeTo, relativePoint, wx, wy = waypointMap:GetPoint()
	local px, py = getPlayerPosition()
	--print(wx, wy, px,py)
	if not(wx and px) then return mapAnts:Hide() end
	local wayX, wayY = wx - px, wy - py
	updateAnts(point, relativeTo, relativePoint, mapAnts, wayX, wayY, px, py, true)
end

local TTL = 0.05
local frame = CreateFrame("Frame",nil,WorldFrame)
MC_ANTS_FRAME = frame
frame.TSLU = 0
frame.OnUpdate = function(self, elapsed)
	self.TSLU = self.TSLU + elapsed 	
	while (self.TSLU >= TTL) do
		self.TSLU = self.TSLU - TTL		-- Ant movement should be consistent across framerate, so we use a while loop in case updates are delayed.
		if ns.Ants.enabled then
			vars.show = true
			-- Update worldmap ants only if they're enabled and there's a current waypoint
			if mapVisible and wm_icon:IsShown() and active_point then
				vars.worldmap.shown = true
				updateMapAnts()
			-- No reason to fire Hide() more than once, we're doing enough processing as is.
			elseif vars.worldmap.shown then
				vars.worldmap.shown = nil
				mapAnts:Hide()
			end
			
			-- Same deal with minimap ants.
			local dist,x,y = Astrolabe:GetDistanceToIcon(mm_icon)
			if active_point and dist then
				vars.minimap.shown = true
				updateMinimapAnts()
			elseif vars.minimap.shown then
				vars.minimap.shown = nil
				ants:Hide()
			end
		--[[legacy, never called
		elseif ns.Ants.enabled then
			vars.show = false
			ants:Hide()
			mapAnts:Hide()
		]]
		end
	end
end

ns.Ants.WORLD_MAP_UPDATE = function(self)
	mapVisible = WorldMapFrame:IsVisible()
	
	if not active_point then return end;
	
	local p = active_point
	local x,y = Astrolabe:PlaceIconOnWorldMap(WorldMapButton, wm_icon, p.m, p.f, p.x, p.y)
	
  if (x and y and (0 < x and x <= 1) and (0 < y and y <= 1)) then
      wm_icon:Show()
  else
      wm_icon:Hide()
  end
end

ns.Ants["MANACORE_CFG_READY"] = function(self)
	if ns.dbpc.enableants == nil then ns.dbpc.enableants = true; end
	self.enabled = ns.dbpc.enableants
	if self.enabled then
		frame.TSLU = 0
		frame:SetScript("OnUpdate",frame.OnUpdate)
	else
		ants:Hide()
		mapAnts:Hide()
		frame:SetScript("OnUpdate",nil)
	end
end
ns.Ants["MANACORE_CFG_CHANGED"] = function(self,key)
	if key ~= "enableants" then return end
	if ns.dbpc.enableants == nil then ns.dbpc.enableants = true; end
	self.enabled = ns.dbpc.enableants
	if self.enabled then -- Ants previously disabled from config
		frame.TSLU = 0
		frame:SetScript("OnUpdate",frame.OnUpdate)
	else
		ants:Hide()
		mapAnts:Hide()
		frame:SetScript("OnUpdate",nil)
	end
end
ns.Ants.MANACORE_VARS_CHANGED = function (self,k)
	if k ~= 'dungeon_mode' then return end
	self.enabled = not(ns.vars.dungeon_mode)
	if ns.vars.dungeon_mode then
		ants:Hide()
		mapAnts:Hide()
		frame:SetScript("OnUpdate",nil)
	else
		frame.TSLU = 0
		frame:SetScript("OnUpdate",frame.OnUpdate)
	end
end
ns:RegisterModuleEvent("Ants","WORLD_MAP_UPDATE")
