﻿local AddonName,ns = ... -- 'ns' is a table not referenced in _G, passed to each file of a loaded addon. Yes, this is handy.

local ICONSIZE, CHECKSIZE, GAP = 16, 16, 4
local FIXEDWIDTH = ICONSIZE + CHECKSIZE + GAP*4 - 4
local elapsed, oldsize, newsize

local Guide = Guide
local ww = WidgetWarlock
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local dataobj = ldb:GetDataObjectByName("Guide")
local m,i = ns:RegisterModule()

ns.SFComps = m

local function GetQuadrant(frame)
  local x,y = frame:GetCenter()
  if not x or not y then return "BOTTOMLEFT", "BOTTOM", "LEFT" end
  local hhalf = (x > UIParent:GetWidth()/2) and "RIGHT" or "LEFT"
  local vhalf = (y > UIParent:GetHeight()/2) and "TOP" or "BOTTOM"
  return vhalf..hhalf, vhalf, hhalf
end

local f = CreateFrame("Button", "GuideStatusFrame", UIParent)
Guide.statusframe = f
f:SetHeight(40)
f:SetFrameStrata("LOW")
f:EnableMouse(true)
f:RegisterForClicks("anyUp")

local checkTooltip = function(self) 
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("BOTTOMRIGHT", f, "TOPLEFT")
    GameTooltip:SetText(self.txt, nil, nil, nil, nil, true)
    GameTooltip:Show()
end

m.status_right = CreateFrame("Button", "GuideStatusFrameRight", Guide.statusframe)
m.status_right:SetWidth(128)
m.status_right:SetHeight(64)
m.status_right:SetPoint("LEFT", "GuideStatusFrame", "RIGHT", 0, -12)
m.status_right:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\single_right")
m.status_right:RegisterForDrag("LeftButton")
m.status_right:SetScript("OnDragStart", function()
  Guide.statusframe:OnDragStart()
end)
m.status_right:SetScript("OnDragStop", function()
  Guide.statusframe:OnDragStop()
end)
ns.status_right = m.status_right

--For use with xp rate when re-introduced
m.status_left = CreateFrame("Button", "GuideStatusFrameLeft", Guide.statusframe)
m.status_left:SetWidth(128)
m.status_left:SetHeight(64)
m.status_left:SetPoint("RIGHT", "GuideStatusFrame", "LEFT", 0, -12)
m.status_left:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\single_left")
m.status_left:RegisterForDrag("LeftButton")
m.status_left:SetScript("OnDragStart", function()
	Guide.statusframe:OnDragStart()
end)
m.status_left:SetScript("OnDragStop", function()
	Guide.statusframe:OnDragStop()
end)
ns.status_left = m.status_left
--]]

m.label = ww.SummonFontString(m.status_left, nil, "GameFontNormal", nil, "RIGHT", m.status_left, "RIGHT", -10, 9)
m.label:SetFont("Fonts\\ARIALN.TTF", 12, "OUTLINE")
m.label:SetWidth(100)
m.label:SetTextColor(1,1,1)
m.label:SetJustifyH("RIGHT")
ns.info_label = m.label

m.status_left_off = CreateFrame("Button", "GuideStatusFrameLeftOff", Guide.statusframe)
m.status_left_off:SetWidth(64)
m.status_left_off:SetHeight(64)
m.status_left_off:SetPoint("RIGHT", "GuideStatusFrame", "LEFT", 0, -12)
m.status_left_off:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\single_left_close")
m.status_left_off:SetScript("OnDragStart", function()
	Guide.statusframe:OnDragStart()
end)
m.status_left_off:SetScript("OnDragStop", function()
	Guide.statusframe:OnDragStop()
end)
ns.status_left_off = m.status_left_off

m.misc_btn_off = CreateFrame("Button", nil, m.status_left_off)
m.misc_btn_off:SetWidth(32)
m.misc_btn_off:SetHeight(32)
m.misc_btn_off:SetPoint("RIGHT", m.status_left_off, "RIGHT", -3, 9)
m.misc_btn_off:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\misc_off")
m.misc_btn_off:SetHighlightTexture("Interface\\AddOns\\ManaCore\\Media\\misc_off")
m.misc_btn_off:SetScript("OnClick", function(frame, button) 
  ns.dbpc.show_xp = true
  ns.status_left:Show()
  ns.status_left_off:Hide()
end)
ns.misc_btn_off = m.misc_btn_off

m.misc_btn = CreateFrame("Button", nil, ns.status_left)
m.misc_btn:SetWidth(32)
m.misc_btn:SetHeight(32)
m.misc_btn:SetPoint("LEFT", ns.status_left, "LEFT", 32, 9)
m.misc_btn:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\misc")
m.misc_btn:SetHighlightTexture("Interface\\AddOns\\ManaCore\\Media\\misc")
m.misc_btn:RegisterForClicks("LeftButtonDown", "RightButtonDown")
m.misc_btn:SetScript("OnClick", function(frame, button) 
  if button == 'LeftButton' then
    ns.dbpc.show_xp = false
    ns.status_left:Hide()
    ns.status_left_off:Show()
  else -- right button resets xp
    ns.xp_tracker:reset()
  end
end)
ns.misc_btn = m.misc_btn

f:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\single_back")
f:GetNormalTexture():SetTexCoord(0, 0, 0, 0.625)

f:SetBackdropColor(0.09, 0.09, 0.19, 0.5)
f:SetBackdropBorderColor(0.5, 0.5, 0.5, 0.5)

local check = ww.SummonCheckBox(CHECKSIZE, f, "LEFT", GAP-4, -3)
local icon = ww.SummonTexture(f, "ARTWORK", ICONSIZE, ICONSIZE, nil, "LEFT", check, "RIGHT", GAP, 0)
local text = ww.SummonFontString(f, "OVERLAY", "GameFontNormal", nil, "RIGHT", -GAP-4, -3)
text:SetPoint("LEFT", icon, "RIGHT", GAP-4, 0)
check.txt = "Click this box to mark the current step as completed."
check:SetScript('OnEnter',checkTooltip)
check:SetScript('OnLeave',function () GameTooltip:Hide() end)

local f2 = CreateFrame("Frame", nil, f)
local f2anchor = "RIGHT"
f2:SetHeight(32)
f2:SetWidth(100)
local text2 = ww.SummonFontString(f2, "OVERLAY", "GameFontNormalSmall", nil, "RIGHT", -GAP-4, 0)
local icon2 = ww.SummonTexture(f2, "ARTWORK", ICONSIZE, ICONSIZE, nil, "RIGHT", text2, "LEFT", -GAP+4, 0)
local check2 = ww.SummonCheckBox(CHECKSIZE, f2, "RIGHT", icon2, "LEFT", -GAP+4, 0)
check2:SetChecked(true)
check2:EnableMouse(false)
f2:Hide()
--check:Hide()


f2:SetScript("OnUpdate", function(self, el)
  elapsed = elapsed + el
  if elapsed > 1 then
    self:Hide()
    icon:SetAlpha(1)
    text:SetAlpha(1)
    f:SetWidth(newsize)
  else
    self:SetPoint(f2anchor, f, f2anchor, 0, elapsed*40)
    self:SetAlpha(1 - elapsed)
    text:SetAlpha(elapsed)
    icon:SetAlpha(elapsed)
    f:SetWidth(oldsize + (newsize-oldsize)*elapsed)
  end
end)


function Guide:PositionStatusFrame()
  f:ClearAllPoints()
  ns:PositionStatusFrame()
  if self.db.profile.statusframepoint and ns.dbpc.statusframe_detached then
    f:SetPoint(self.db.profile.statusframepoint, self.db.profile.statusframex, self.db.profile.statusframey)
  end

  if self.db.char.showstatusframe then f:Show() else f:Hide() end
end


ldb.RegisterCallback(Guide, "LibDataBroker_AttributeChanged_Guide_text", function(event, name, attr, value, dataobj)
  oldsize = f:GetWidth()
  icon:SetAlpha(0)
  text:SetAlpha(0)
  elapsed = 0
  f2:SetWidth(f:GetWidth())
  f2anchor = select(3, GetQuadrant(f))
  f2:ClearAllPoints()
  f2:SetPoint(f2anchor, f, f2anchor, 0, 0)
  f2:SetAlpha(1)
  text2:SetText(text:GetText())

  text:SetText(value)
  check:SetChecked(false)
  check:SetButtonState("NORMAL")
  if Guide.db.char.currentguide == "No Guide" then check:Disable() else check:Enable() end
  if i == 1 then f:SetWidth(FIXEDWIDTH + text:GetStringWidth()) end
  newsize = FIXEDWIDTH + text:GetStringWidth()
  newsize = newsize > 125 and newsize or 125

  f2:Show()
end)


ldb.RegisterCallback(Guide, "LibDataBroker_AttributeChanged_Guide_icon", function(event, name, attr, value, dataobj)
  local oldtexture = icon:GetTexture()
  icon2:SetTexture(oldtexture)
  if oldtexture and oldtexture:match("Interface\\Icons") then icon2:SetTexCoord(4/48, 44/48, 4/48, 44/48) else icon2:SetTexCoord(0,1,0,1) end

  icon:SetTexture(value)
  if value and value:match("Interface\\Icons") then icon:SetTexCoord(4/48, 44/48, 4/48, 44/48) else icon:SetTexCoord(0,1,0,1) end
end)


f:SetScript("OnClick", dataobj.OnClick)
f:SetScript("OnLeave", dataobj.OnLeave)
f:SetScript("OnEnter", dataobj.OnEnter)
check:SetScript("OnClick", function(self, btn) Guide:SetTurnedIn() end)

function Guide.GetUIParentAnchor(frame)
  local w, h, x, y = UIParent:GetWidth(), UIParent:GetHeight(), frame:GetCenter()
  local hhalf, vhalf = (x > w/2) and "RIGHT" or "LEFT", (y > h/2) and "TOP" or "BOTTOM"
  local dx = hhalf == "RIGHT" and math.floor(frame:GetRight() + 0.5) - w or math.floor(frame:GetLeft() + 0.5)
  local dy = vhalf == "TOP" and math.floor(frame:GetTop() + 0.5) - h or math.floor(frame:GetBottom() + 0.5)

  return vhalf..hhalf, dx, dy
end

f.OnDragStop = function(frame)
  ns:FireEvent('GUIDE_FRAME_DRAG_STOP')
  frame:StopMovingOrSizing()
  if ManaCoreRing:IsMouseOver() or ManaCoreButton:IsMouseOver() then
    ns.dbpc.statusframe_detached = false
    ns:PositionStatusFrame()
    return
  end
  Guide.db.profile.statusframepoint, Guide.db.profile.statusframex, Guide.db.profile.statusframey = Guide.GetUIParentAnchor(frame)
  Guide:Debug(1, "Status frame moved", Guide.db.profile.statusframepoint, Guide.db.profile.statusframex, Guide.db.profile.statusframey)
  frame:ClearAllPoints()
  frame:SetPoint(Guide.db.profile.statusframepoint, Guide.db.profile.statusframex, Guide.db.profile.statusframey)
  ns.dbpc.statusframe_detached = true
  --dataobj.OnEnter(frame)
end

f.OnDragStart = function(frame)
  --if not(frame:IsMovable()) or ns.dbpc.lock_frame == true then return end
  if Guide.objectiveframe:IsVisible() then HideUIPanel(Guide.objectiveframe) end
  dataobj.OnLeave(frame)
  frame:StartMoving()
  ns:FireEvent('GUIDE_FRAME_DRAG_START')
end

f:RegisterForDrag("LeftButton")
--f:SetMovable(true)
f:SetClampedToScreen(true)

local function createButton(texture, parent, normal, size)
  local f = CreateFrame("Button", nil, parent or Guide.statusframe)
  f:SetWidth(size or 32) f:SetHeight(size or 32)
  f:SetNormalTexture(texture)
  f:SetHighlightTexture(normal and texture or texture..'_hover')
  f:SetScript("OnClick", function() f:onClick() end)
  return f
end

local function addTooltip(widget, text, ...)
  widget:EnableMouse(true)
  widget:SetScript("OnEnter", function() 
    GameTooltip:SetOwner(widget, "ANCHOR_NONE")
    GameTooltip:SetPoint("TopLeft", widget, "BottomLeft")
    GameTooltip:SetText(text, nil, nil, nil, nil, true)
  end)
  widget:SetScript("OnLeave", function() GameTooltip:Hide() end)
end

local maximize = createButton("Interface\\AddOns\\ManaCore\\Media\\maximize", ns.status_right)
local minimize = createButton("Interface\\AddOns\\ManaCore\\Media\\minimize", ns.status_right)
Guide.minimize = minimize
Guide.maximize = maximize

maximize:SetPoint("TOPRIGHT", ns.status_right, "TOPRIGHT", -55, 0)
function maximize:onClick()
  ShowUIPanel(Guide.objectiveframe)
end

minimize:SetPoint("TOPRIGHT", ns.status_right, "TOPRIGHT", -55, 0)
minimize:Hide()
function minimize:onClick()
  HideUIPanel(Guide.objectiveframe)
end

-- Force resize in case of font changes
m.MANACORE_CFG_READY = function(self)
  local oldnewsize = newsize
  newsize = FIXEDWIDTH + text:GetWidth()
  newsize = newsize > 125 and newsize or 125
  if oldnewsize ~= newsize and not f2:IsVisible() then
    oldsize = newsize
    f:SetWidth(newsize)
  end
  f:SetMovable(not(ns.dbpc.lock_frame))
  f:SetScript("OnDragStart", f.OnDragStart)
  f:SetScript("OnDragStop", f.OnDragStop)
  f:RegisterForDrag(not(ns.dbpc.lock_frame) and 'LeftButton' or nil)
	
	--addTooltip(config, "|cFF6666FFOpen Booster configuration")
  --addTooltip(muleshop, "|cFF6666FFOpen Mule Shopper")
	addTooltip(m.misc_btn_off, "Click to show XP per hour.")
	addTooltip(m.misc_btn, "Left-Click to hide XP per hour.\nRight-Click to reset counter.")
	
	if ns.dbpc.show_xp == true then		
		m.status_left:Show()
		m.status_left_off:Hide()
	else
		ns.status_left:Hide()
		ns.status_left_off:Show()
	end
end 

m.MANACORE_CFG_CHANGED = function (self,var,old,new)
    if var == 'lock_frame' then
        f:SetMovable(not(ns.dbpc.lock_frame))
        f:RegisterForDrag(not(ns.dbpc.lock_frame) and 'LeftButton' or nil)
    end
end
