﻿local _,ns = ...
local m,i = ns:RegisterModule(nil,'ManaArrow')

ManaArrow = m;
Astrolabe = DongleStub("Astrolabe-1.0")

--default db values
ns.dbpc.arrow = {}
ns.dbpc.arrow.enable = true
ns.dbpc.arrow.lock = false

--utility funcs/values
local function ColorGradient(perc, ...)
	local num = select("#", ...)
	local hexes = type(select(1, ...)) == "string"

	if perc == 1 then
		return select(num-2, ...), select(num-1, ...), select(num, ...)
	end

	num = num / 3

	local segment, relperc = math.modf(perc*(num-1))
	local r1, g1, b1, r2, g2, b2
	r1, g1, b1 = select((segment*3)+1, ...), select((segment*3)+2, ...), select((segment*3)+3, ...)
	r2, g2, b2 = select((segment*3)+4, ...), select((segment*3)+5, ...), select((segment*3)+6, ...)

	if not r2 or not g2 or not b2 then
		return r1, g1, b1
	else
		return r1 + (r2-r1)*relperc,
		g1 + (g2-g1)*relperc,
		b1 + (b2-b1)*relperc
	end
end

local twopi = math.pi * 2

---arrow frames---
local wayframe = CreateFrame("Button", "ManaArrowFrame", UIParent)
wayframe:EnableMouse(true)
wayframe:SetMovable(true)
wayframe:SetHeight(70)
wayframe:SetWidth(70)
wayframe:SetPoint("CENTER", 0, 0)
--wayframe:Hide()

local titleframe = CreateFrame("Frame", nil, wayframe)
wayframe.title = titleframe:CreateFontString("OVERLAY", nil, "GameFontHighlightSmall")
wayframe.status = titleframe:CreateFontString("OVERLAY", nil, "GameFontNormalSmall")
wayframe.tta = titleframe:CreateFontString("OVERLAY", nil, "GameFontNormalSmall")
wayframe.title:SetPoint("TOP", wayframe, "BOTTOM", 0, 0)
wayframe.status:SetPoint("TOP", wayframe.title, "BOTTOM", 0, 0)
wayframe.tta:SetPoint("TOP", wayframe.status, "BOTTOM", 0, 0)

local function OnDragStart(self, button)
	if not ns.dbpc.arrow.lock then
		self:StartMoving()
	end
end

local function OnDragStop(self, button)
	self:StopMovingOrSizing()
end

local function OnEvent(self, event, ...)
	if event == "ZONE_CHANGED_NEW_AREA" and ns.dbpc.arrow.enable then
		self:Show()
	end
end

wayframe:SetScript("OnDragStart", OnDragStart)
wayframe:SetScript("OnDragStop", OnDragStop)
wayframe:RegisterForDrag("LeftButton")
wayframe:RegisterEvent("ZONE_CHANGED_NEW_AREA")
wayframe:SetScript("OnEvent", OnEvent)

wayframe.arrow = wayframe:CreateTexture(nil, "OVERLAY")
wayframe.arrow:SetTexture([[Interface\AddOns\ManaCore\Media\wparrow]])
wayframe.arrow:SetPoint("CENTER")
wayframe.arrow:SetWidth(85)
wayframe.arrow:SetHeight(85)

wayframe.arrowbg = wayframe:CreateTexture(nil, "BACKGROUND")
wayframe.arrowbg:SetTexture([[Interface\AddOns\ManaCore\Media\wparrowbg]])
wayframe.arrowbg:SetPoint("CENTER")
wayframe.arrowbg:SetWidth(85)
wayframe.arrowbg:SetHeight(85)

wayframe.arrowdot = wayframe:CreateTexture(nil, "OVERLAY")
wayframe.arrowdot:SetTexture([[Interface\AddOns\ManaCore\Media\wparrowdot]])
wayframe.arrowdot:SetPoint("CENTER")
wayframe.arrowdot:SetWidth(85)
wayframe.arrowdot:SetHeight(85)

---map icon frames---
local mm_icon = CreateFrame("Frame", nil, Minimap)
mm_icon:SetSize(10,10)

mm_icon.dot = mm_icon:CreateTexture(nil, "OVERLAY")
mm_icon.dot:SetPoint("CENTER")
mm_icon.dot:SetWidth(20)
mm_icon.dot:SetHeight(20)
mm_icon.dot:SetTexture([[Interface\AddOns\ManaCore\Media\SilverBlueDot]])

mm_icon.arrow = mm_icon:CreateTexture(nil, "OVERLAY")
mm_icon.arrow:SetPoint("CENTER")
mm_icon.arrow:SetWidth(40)
mm_icon.arrow:SetHeight(40)
mm_icon.arrow:SetTexture([[Interface\AddOns\ManaCore\Media\MinimapArrow-Blue]])

local square_half = math.sqrt(0.5)
local rad_135 = math.rad(135)

local function mm_rotateArrow(self)
  local angle = Astrolabe:GetDirectionToIcon(self)
	if not angle then return end;
  angle = angle + rad_135

  if GetCVar("rotateMinimap") == "1" then
      local cring = GetPlayerFacing()
      angle = angle - cring
  end

  local sin,cos = math.sin(angle) * square_half, math.cos(angle) * square_half
  self.arrow:SetTexCoord(0.5-sin, 0.5+cos, 0.5+cos, 0.5+sin, 0.5-cos, 0.5-sin, 0.5+sin, 0.5-cos)
end

local mm_update_throttle = 0
local function mm_onUpdate(self, elapsed)
	mm_update_throttle = mm_update_throttle + elapsed
	
	if mm_update_throttle < 0.1 then return end
	
	mm_update_throttle = 0
	
	local edge = Astrolabe:IsIconOnEdge(self)
	
	if edge then
		self.dot:Hide()
		self.arrow:Show()
		
		mm_rotateArrow(self)
	else
		self.dot:Show()
		self.arrow:Hide()
	end
end
mm_icon:SetScript("OnUpdate", mm_onUpdate)

local wm_icon = CreateFrame("Frame", nil, WorldMapButton)
wm_icon:SetSize(10,10)

wm_icon.dot = wm_icon:CreateTexture(nil, "OVERLAY")
wm_icon.dot:SetPoint("CENTER")
wm_icon.dot:SetWidth(20)
wm_icon.dot:SetHeight(20)
wm_icon.dot:SetTexture([[Interface\AddOns\ManaCore\Media\SilverBlueDot]])
--wm_icontex:SetTexCoord(0, 0.125, 0, .25)

--Globals used by ants to draw its stuff
ManaArrow.WorldMapIcon = wm_icon
ManaArrow.MinimapIcon = mm_icon

---waypoint code---
local arrive_distance = 5 --fixed distance for now
local active_point, showDownArrow

function m:IsActive()
	if active_point then
		return true
	else
		return false
	end
end

function m:SetMFPoint(m, f, x, y, desc)
	if type(m)~="number" or type(x)~="number" or type(y)~="number" then
		m:mdebug("WARNING: Attempt to set arrow to invalid point. Call ignored.")
		return
	end
	
	Astrolabe:PlaceIconOnMinimap(mm_icon, m, f, x, y)
	Astrolabe:PlaceIconOnWorldMap(WorldMapButton, wm_icon, m, f, x, y)
	active_point = {m = m, f = f, x = x, y = y, desc = desc}
	--active_point = true; --for now, lets see if we need more
	wayframe.title:SetText(desc)
	wayframe:Show()
end

function m:ClearPoint()
	active_point = nil;
	Astrolabe:RemoveIconFromMinimap(mm_icon)
	wm_icon:Hide()
	wayframe:Hide()
end
	
	
local status = wayframe.status
local tta = wayframe.tta
local arrow = wayframe.arrow
local arrowbg = wayframe.arrowbg
local arrowdot = wayframe.arrowdot
local pulse_throttle = 0
local count = 0
local last_distance = 0
local tta_throttle = 0
local speed = 0
local speed_count = 0

--[[Using 2d method, this is depreciated
function m:SetAngle(angle)
	if showDownArrow then
		arrow:SetHeight(56)
		arrow:SetWidth(42)
		arrow:SetTexture("Interface\\AddOns\\TomTom\\Images\\Arrow")
		showDownArrow = false
	end

	local cell = floor(angle / twopi * 108 + 0.5) % 108
	local column = cell % 9
	local row = floor(cell / 9)

	local xstart = (column * 56) / 512
	local ystart = (row * 42) / 512
	local xend = ((column + 1) * 56) / 512
	local yend = ((row + 1) * 42) / 512
	arrow:SetTexCoord(xstart,xend,ystart,yend)
end
]]

local function OnUpdate(self, elapsed)
	if not active_point then
		self:Hide()
		return
	end

	local dist,x,y = Astrolabe:GetDistanceToIcon(mm_icon)

	-- The only time we cannot calculate the distance is when the waypoint
	-- is on another continent, or we are in an instance
	if not dist then
		self:Hide()
		return
	end

	status:SetText(format("%d yards", dist))

	if dist <= arrive_distance then
		arrow:Hide()
		arrowbg:SetTexCoord(0, 1, 0, 1)
		arrowdot:SetTexCoord(0, 1, 0, 1)
		
		pulse_throttle = pulse_throttle + elapsed
		
		--pulse at 60 FPS
		while pulse_throttle > .016 do
			count = count + 2
			pulse_throttle = pulse_throttle - .016
		end
		
		if count >= 100 then
			count = count % 100
		end
		
		local pulse = 1-abs(count-50)/100
		
		arrowdot:SetVertexColor(pulse,pulse,pulse)
	else
		arrow:Show()
		
		local angle = Astrolabe:GetDirectionToIcon(mm_icon)
		local player = GetPlayerFacing()
	
		angle = angle - player
		
		local arad = angle + rad_135
		
		local sin,cos = math.sin(arad) * square_half, math.cos(arad) * square_half
		arrow:SetTexCoord(0.5-sin, 0.5+cos, 0.5+cos, 0.5+sin, 0.5-cos, 0.5-sin, 0.5+sin, 0.5-cos)
		arrowbg:SetTexCoord(0.5-sin, 0.5+cos, 0.5+cos, 0.5+sin, 0.5-cos, 0.5-sin, 0.5+sin, 0.5-cos)
		arrowdot:SetTexCoord(0.5-sin, 0.5+cos, 0.5+cos, 0.5+sin, 0.5-cos, 0.5-sin, 0.5+sin, 0.5-cos)
		
		local perc = math.abs((math.pi - math.abs(angle)) / math.pi)
		
		local r,g,b = ColorGradient(perc, 1, 0, 0, 1, 1, 0, 0, 1, 0)
		arrowdot:SetVertexColor(r,g,b)
	end
	-- Calculate the TTA every second  (%01d:%02d)

	tta_throttle = tta_throttle + elapsed

	if tta_throttle >= 1.0 then
		-- Calculate the speed in yards per sec at which we're moving
		local current_speed = (last_distance - dist) / tta_throttle

		if last_distance == 0 then
			current_speed = 0
		end

		if speed_count < 2 then
			speed = (speed + current_speed) / 2
			speed_count = speed_count + 1
		else
			speed_count = 0
			speed = current_speed
		end

		if speed > 0 then
			tta:SetText("***")
			--local eta = math.abs(dist / speed)
			--tta:SetFormattedText("%01d:%02d", eta / 60, eta % 60)
		else
			tta:SetText("***")
		end

		last_distance = dist
		tta_throttle = 0
	end
end
wayframe:SetScript("OnUpdate", OnUpdate)

function m:WORLD_MAP_UPDATE(self)
	if not active_point then return end;
	
	local p = active_point
	local x,y = Astrolabe:PlaceIconOnWorldMap(WorldMapButton, wm_icon, p.m, p.f, p.x, p.y)
	
  if (x and y and (0 < x and x <= 1) and (0 < y and y <= 1)) then
      wm_icon:Show()
  else
      wm_icon:Hide()
  end
end
m:RegisterEvent("WORLD_MAP_UPDATE")