﻿local AddonName, ns = ...;
local m, i = ns:RegisterModule(nil, "MapMarker");
MapMarker = m;
--TODO: see if we can get mapmarker down to local

--[[NOTE: OUT OF DATE

Here are 3 examples for use:

/dump MapMarker.ShowRoute ( "VashjirKelpForest" , "Herbs" )
/dump MapMarker.ShowRoute ( "VashjirKelpForest" , "Herbs & Mining" )
/dump MapMarker.ShowRoute ( "VashjirKelpForest" , "Mining" )


/dump MapMarker.HideRoute ( "VashjirKelpForest" , "Herbs" )
/dump MapMarker.HideRoute ( "VashjirKelpForest" , "Herbs & Mining" )
/dump MapMarker.HideRoute ( "VashjirKelpForest" , "Mining" )


/dump MapMarker.HideAllRoutes()
]]
local NODE_TEXTURE = "Interface\\AddOns\\ManaCore\\Media\\Arrow_Down";
local NODE_HEIGHT = 16;
local NODE_WIDTH = 16;

local LINE_TEXTURE = "Interface\\AddOns\\ManaCore\\Media\\line-outline";
local LINE_WIDTH = 25;
local LINE_COLOR = {
	["Skin"] = { -- sandyBrown
		["r"] = .96,
		["g"] = .64,
		["b"] = .38,
	},
	["Herb"] = { -- springGreen1
		["r"] = 0,
		["g"] = 1,
		["b"] = .49,
	},
	["Mine"] = { -- firebrick
		["r"] = .7,
		["g"] = .13,
		["b"] = .13,
	},
	["Cloth"] = { -- white
		["r"] = 1,
		["g"] = 1,
		["b"] = 1,
	},
};



-- Returns a unique number every time it is called.
local uniquealizer = 0;
function MapMarker.unique()
	uniquealizer = uniquealizer + 1;
	return uniquealizer;
end


--[[
	minimum_update_interval is the most-often a frame will update. 
	maximum_update_interval is the less-often a frame will update.
	
	To use, make a local variable "current_update_interval" and set it appropriately, depending on how often you want the frame to update.
	Look at minimap.lua for an example: When there's nothing to be displayed, the minimap slows down its update speed, to lessen wasted resources.
]]--
MapMarker.minimum_update_interval = .03;
MapMarker.maximum_update_interval = 1;


-- The preferred method of showing and hiding routes. Encapsulates all minimap and worldmap action.
function MapMarker.SetRoute(zoneid, route)
	MapMarker.drawWorldmapLines(zoneid, route);
	MapMarker.drawMinimapLines(zoneid, route);
end

function MapMarker.DisableRoute()
	 MapMarker.hideWorldmapRoute();
	 MapMarker.hideMinimapRoute();
end

-- returns the color the route's line is, or mixes the colors if it has multiple types
-- color_lookups is a table of numbered indices with string values equal to the first indices in the LINE_COLOR table.
-- if an index in color_lookups does not equal something from the LINE_COLOR table, it will be ignored and white will be used instead
function MapMarker.getColor(color_lookups)
	if (#(color_lookups) == 1) then return LINE_COLOR[color_lookups[1]] or {1,1,1,}; end

	local toReturn = {
		["r"] = 0,
		["g"] = 0,
		["b"] = 0,
	};
	for i=1, #(color_lookups) do
		local colortbl = LINE_COLOR[color_lookups[i]] or {1,1,1,};
		toReturn["r"] = toReturn["r"] + (colortbl["r"]/#(color_lookups));
		toReturn["g"] = toReturn["g"] + (colortbl["g"]/#(color_lookups));
		toReturn["b"] = toReturn["b"] + (colortbl["b"]/#(color_lookups));
	end

	return toReturn;
end

-- takes an encoded coordinate and returns the x,y that it is supposed to be
function MapMarker.decodeCoord(coord)
	local x = floor(coord/ 10000) / 10000;
	local y = (coord % 10000) / 10000;
	return x,y;
end

-- takes an x,y and returns the encoded coordinate for storing
function MapMarker.encodeCoord(x, y)
	return floor(x * 10000 + 0.5) * 10000 + floor(y * 10000 + 0.5)
end

--[[
	Draws a line on the parent frame
		parent - Frame - the frame upon which to draw the line
		fresh_lines - Table - contains available lines
		used_lines - Table - the table to insert used lines
		sx, sy - Number - the starting x,y coordinates of the line
		ex, ey - Number - the ending x,y coordinates of the line
		color - *Optional* Table - contains the color mapping information. Has 3 indices: r, g, b - each has a value between 0 and 1, indicating the respective amount of that color to include. If color is not included, the line drawn will default to white
]]
function MapMarker.drawLine(parent, fresh_lines, used_lines, sx, sy, ex, ey, color)
	local fh, fw = parent:GetHeight(), parent:GetWidth();
	sx, sy, ex, ey = sx*fw, sy*fh, ex*fw, ey*fh;

	local line = tremove(fresh_lines) or parent:CreateTexture(nil, "ARTWORK");
	line:SetTexture(LINE_TEXTURE);
	tinsert(used_lines, line);

	line:SetDrawLayer("ARTWORK");
	if (color) then line:SetVertexColor(color["r"], color["g"], color["b"], 1);
	else line:SetVertexColor(1,1,1,1);
	end

	-- Determine dimensions and center point of line
	local dx,dy = ex - sx, ey - sy;
	local cx,cy = (sx + ex) / 2, (sy + ey) / 2;

	-- Calculate actual length of line
	local l = ((dx * dx) + (dy * dy)) ^ 0.5;

	-- Quick escape if it's zero length
	if l == 0 then
		line:ClearAllPoints();
		line:SetTexCoord(0,0,0,0,0,0,0,0);
		line:SetPoint("BOTTOMLEFT", parent, "BOTTOMLEFT", cx, cy);
		line:SetPoint("TOPRIGHT",   parent, "BOTTOMLEFT", cx, cy);
		return line;
	end

	-- Normalize direction if necessary
	if (dx < 0) then dx,dy = -dx,-dy; end

	-- Sin and Cosine of rotation, and combination (for later)
	local s,c = -dy / l, dx / l;
	local sc = s * c;

	-- Calculate bounding box size and texture coordinates
	local Bwid, Bhgt, BLx, BLy, TLx, TLy, TRx, TRy, BRx, BRy;
	if (dy >= 0) then
		Bwid = ((l * c) - (LINE_WIDTH * s)) * TAXIROUTE_LINEFACTOR_2;
		Bhgt = ((LINE_WIDTH * c) - (l * s)) * TAXIROUTE_LINEFACTOR_2;
		BLx, BLy, BRy = (LINE_WIDTH / l) * sc, s * s, (l / LINE_WIDTH) * sc;
		BRx, TLx, TLy, TRx = 1 - BLy, BLy, 1 - BRy, 1 - BLx;
		TRy = BRx;
	else
		Bwid = ((l * c) + (LINE_WIDTH * s)) * TAXIROUTE_LINEFACTOR_2;
		Bhgt = ((LINE_WIDTH * c) + (l * s)) * TAXIROUTE_LINEFACTOR_2;
		BLx, BLy, BRx = s * s, -(l / LINE_WIDTH) * sc, 1 + (LINE_WIDTH / l) * sc;
		BRy, TLx, TLy, TRy = BLx, 1 - BRx, 1 - BLx, 1 - BLy;
		TRx = TLy;
	end
	
	--TexCoord Out Of Range error fix
	TLx = max(min(TLx,10000), -10000)
	TLy = max(min(TLy,10000), -10000)
	BLx = max(min(BLx,10000), -10000)
	BLy = max(min(BLy,10000), -10000)
	TRx = max(min(TRx,10000), -10000)
	TRy = max(min(TRy,10000), -10000)
	BRx = max(min(BRx,10000), -10000)
	BRy = max(min(BRy,10000), -10000)

	-- Set texture coordinates and anchors
	line:ClearAllPoints();
	line:SetTexCoord( TLx, TLy, BLx, BLy, TRx, TRy, BRx, BRy )
	line:SetPoint("BOTTOMLEFT", parent, "BOTTOMLEFT", cx - Bwid, cy - Bhgt);
	line:SetPoint("TOPRIGHT",   parent, "BOTTOMLEFT", cx + Bwid, cy + Bhgt);
	line:Show();
	return line;
end

-- Releases all the parent's child lines to the pool of available (fresh) lines, so they can be reused (saves memory)
function MapMarker.undrawLine(parent, fresh_lines, used_lines)
	local line;
	for i=#(used_lines),1,-1 do
		line = used_lines[i];
		if (line:GetParent() == parent) then
			line:Hide();
			tinsert(fresh_lines, tremove(used_lines, i));
		end
	end
end

--[[
	Draws a node on the parent frame
		parent - Frame - the frame upon which to draw the node
		fresh_nodes - Table - contains available nodes
		used_nodes - Table - the table to insert used nodes
		x, y - Number - the x,y coordinates of the node
]]
function MapMarker.drawNode(parent, fresh_nodes, used_nodes, x, y)
	if (x < 0) then x = x * -1; end
	if (y > 0) then y = y * -1; end

	local node = tremove(fresh_nodes);
	if (not node) then
		local id = MapMarker.unique();
		node = CreateFrame("Frame", "MapMarkerAddon_Node_" .. id, parent);
		node:SetHeight(NODE_HEIGHT);
		node:SetWidth(NODE_WIDTH);
		node:SetPoint("CENTER", parent, "TOPLEFT", x*parent:GetWidth(), y*parent:GetHeight());
		node:EnableMouse(false);

			node.art = node:CreateTexture(nil, "OVERLAY");
			node.art:SetParent(node);
			node.art:SetTexture(NODE_TEXTURE);
			node.art:SetPoint("TOPRIGHT", node, 0, 0);
			node.art:SetPoint("BOTTOMLEFT", node, 0, 0);
	else
		node:SetPoint("CENTER", parent, "TOPLEFT", x*parent:GetWidth(), y*parent:GetHeight());
	end

	tinsert(used_nodes, node);
	node:Show();

	return node;
end

-- Releases all the parent's child nodes to the pool of available (fresh) nodes, so they can be reused (saves memory)
function MapMarker.undrawNode(parent, fresh_nodes, used_nodes)
	local node;
	for i=#(used_nodes),1,-1 do
		node = used_nodes[i];
		if (node:GetParent() == parent) then
			node:Hide();
			tinsert(fresh_nodes, tremove(used_nodes, i));
		end
	end
end

-- returns the input ordered pair as a polar coordinate
function MapMarker.cartesianToPolar(x, y)
	return ((x*x) + (y*y))^.5, math.atan2(y,x); -- atan2 DOES take (y,x) NOT (x,y) as would be expected.
end

-- returns the input polar coord as an x,y ordered pair
function MapMarker.polarToCartesian(r, theta)
	return r * math.cos(theta), r * math.sin(theta);
end

-- alternate names for zones due to phasing
MapMarker.altMapInfo = {
	["TwilightHighlands_terrain1"] = "TwilightHighlands",
	["Hyjal_terrain1"] = "Hyjal",
	["Uldum_terrain1"] = "Uldum",
	["BattleforGilneas"] = "GilneasCity",
	["Gilneas_terrain1"] = "Gilneas",
	["Gilneas_terrain2"] = "Gilneas",
	["TheLostIsles_terrain1"] = "TheLostIsles",
	["TheLostIsles_terrain2"] = "TheLostIsles",
};



