﻿if (IsAddOnLoaded("Booster")) or IsAddOnLoaded("Edge") then 

	local _G = getfenv(0)

	-- Addon
	local _, ns = ...
	ns.TalentHelper = ns:RegisterModule(nil,"TalentHelper")
	local core = ns.TalentHelper

	local DISABLE_TEXT = "|cffffa0a0"..DISABLE
	local TalentTooltipText = "This is where you select what talent spec suggestions to show in the talent window. You can choose any talent spec listed to see that spec suggestions, or you can choose Disable to turn off talent helper completely."

	-- Template Data, filled in by class files
	core.templates = {}

	-- Work vars
	local activeTemplate
	local activeCategory
	local activeTemplateName
	local activeCategoryName
	local talentTab = {}
	local talentIndex = {}
	local glyphTmp = { {}, {}, {}, }
	local talentTmp = {}
	local recentLevelMade = 0

	-- Dialog
	-- Booster dialog currenly not used
	StaticPopupDialogs["BoosterSpentTalentPoints"] = {
	  text     = "You have unspent talent points. Would you like them to be assigned automatically?",
	  OnAccept = function() ns.TalentHelper:AutoLearnAllTalents() end,
	  button1  = "Yes", button2      = "No",
	  timeout  = 0,     hideOnEscape = true
	}

	StaticPopupDialogs["TalentHelperSpendAllInvalid"] = {
		text     = "You have points in talents that aren't included in your chosen template. Auto-fill can only be used when following the template. Please either continue to manually add points at your leisure or reset your talent build to use auto-fill.",
		button1  = "Ok",
	  timeout  = 0,     hideOnEscape = true
	}

	StaticPopupDialogs["TalentHelperConfirmSpendAll"] = {
		text     = "Are you sure you want to spend all your talent points? This cannot be undone without a paid talent reset!",
	  OnAccept = function() ns.TalentHelper:AutoLearnAllTalents() end,
	  button1  = "Yes", button2      = "No",
	  timeout  = 0,     hideOnEscape = true
	}

	--------------------------------------------------------------------------------------------------------
	--                                          Helper Functions                                          --
	--------------------------------------------------------------------------------------------------------


	-- TabUpdate
	local function TabUpdate()
		local tab = PanelTemplates_GetSelectedTab(PlayerTalentFrame)
		-- We don't have any templates for some reason, so we cannot show the user any details, just abandon here
		-- Test this out with the OR included. 
		if (not next(core.templates)) then --or (not activeTemplate) then
			return
		-- Specialization updates
		elseif (tab == 1) then
			if (core.TalentInitialize) then core:TalentInitialize() end
			core:TalentUpdate()
		-- Talent Update
		elseif (tab == 2) then
			if (core.TalentInitialize) then
				core:TalentInitialize()
			end
			core:TalentUpdate()
		-- Glyph Update
		elseif (tab == 3) then
			if (core.GlyphInitialize) then
				core:GlyphInitialize()
			end
			core:GlyphUpdate()
		end
	end

	-- Creates a new Glow Texture for the talent tree button selection
	local function CreateTalentButtonGlowTexture(talentBtn)
		local glow = talentBtn:CreateTexture(nil,"ARTWORK")
		glow:SetTexture("Interface\\Buttons\\UI-Panel-Button-Glow")
		glow:SetBlendMode("ADD")
		glow:SetSize(188,22)
		glow:SetTexCoord(0,0.75,0,0.609375)
		glow:SetPoint("TOPLEFT",-19,7)
		glow:SetPoint("BOTTOMRIGHT",19,-7)
		talentBtn.boosterGlow = glow
		return glow
	end

	-- Caches Talents for Future use
	local function CacheTalents()
		core:mdebug("Talents Cached!")
		local tab = 1
		for tal = 1, GetNumTalents(tab) do
			local name = GetTalentInfo(tal)
			if (name) then
				talentTab[name] = tab
				talentIndex[name] = tal
			end
		end
	end

	--Determines if auto-spec is currently valid
	local function IsAutoSpecInvalid()
		for i = 1, #activeTemplate.TALENTS do
			--DEFAULT_CHAT_FRAME:AddMessage(" isAutoSpecInvalid: index: "..tostring(talentIndex[activeTemplate.TALENTS[i]]))
			if (talentIndex[activeTemplate.TALENTS[i]]) then if (tostring(activeTemplate.TALENTS[i])) ~= (tostring(GetTalentInfo(talentIndex[activeTemplate.TALENTS[i]]))) then return true end end
		end
		return false
	end

	-- Determines the next talent to pick
	local function GetNextTalent(curTalent)
		if not(activeTemplate) then return end
		for index, name in ipairs(activeTemplate.TALENTS) do
			if name == curTalent then return activeTemplate.TALENTS[index+1] end
		end
	end

	function core:AutoLearnAllTalents()
		if IsAutoSpecInvalid() then
			StaticPopup_Show("TalentHelperSpendAllInvalid")
			return
		end
		
		local freePoints = GetNumUnspentTalents()
		local activeSpec = GetActiveSpecGroup()
		local nextTalent = activeTemplate.TALENTS[1]
		local totalPoints = 6
		
		-- figure out how many points you'd have if you had spent no points
		local lvl = UnitLevel("player")
		if lvl < 30 then 
			totalPoints = 1 
		elseif lvl < 45 then 
			totalPoints = 2
		elseif lvl < 60 then
			totalPoints = 3
		elseif lvl < 75 then
			totalPoints = 4
		elseif lvl < 90 then
			totalPoints = 5
		elseif lvl == 90 then
			totalPoints = 6
		end
		if freePoints <  totalPoints then 
			StaticPopup_Show("TalentHelperSpendAllInvalid") 
			return
		end
		-- Set Primary Tree
		if  not GetSpecialization() then
			SetSpecialization(activeTemplate.SPEC)
		end
		
		-- Learn
		--we for loop this for safety, don't want to infinite loop for a bad spec
		for i = 1, freePoints do
			if not nextTalent then break end
			LearnTalent(talentIndex[nextTalent])
			nextTalent = GetNextTalent(nextTalent)
		end
		
	end


	-- Check free points and auto learn next talent
	-- NOTE: Currently disabled, needs review
	function core:CheckFreePoints()
		if true then return end --This just seems like a dangerous idea, let them use the interface for now
		local freePoints = GetNumUnspentTalents()
		if (freePoints > 0) and (activeTemplate) then
			-- Ask user if they wish to spend all talent points automatically
			StaticPopup_Show("BoosterSpentTalentPoints")
		end
	end

	--------------------------------------------------------------------------------------------------------
	--                                           Public Scripts                                           --
	--------------------------------------------------------------------------------------------------------
	-- Check after everything else is working right. 
	function core:SetActiveSpec(title, category)
		local source = category and core.templates[category] or core.templates
		if source[title] then
			activeTemplate = source[title]
			activeTemplateName = title
			activeCategory = category and source or nil
			activeCategoryName = category
			if core.dropDown then --if the Talent UI has been loaded
				core.dropDown.text:SetText(title)
				TabUpdate()
			end
			ns.dbpc.talentTemplateName = title
			ns.dbpc.activeCategory = category
		end
	end

	function core:AddSpec(title, category, data)
		local tbl
		if category then
			tbl = core.templates[category]
			if not tbl then 
				tbl = {}
				core.templates[category] = tbl
				tbl.iscategory = true
			end
		else
			tbl = core.templates
		end
		tbl[title] = data
	end

	--------------------------------------------------------------------------------------------------------
	--                                           Frame Scripts                                            --
	--------------------------------------------------------------------------------------------------------

	ns:RegisterModuleEvent("TalentHelper", "PLAYER_LOGIN", "PLAYER_TALENT_UPDATE","PREVIEW_TALENT_POINTS_CHANGED","ADDON_LOADED","ACTIVE_TALENT_GROUP_CHANGED")

	-- Variables Loaded
	-- Delete if not being used. 
	function core:MANACORE_CFG_READY() end


	function core:PLAYER_LOGIN()

		-- Translate spellID into spellName
		for name, template in next, self.templates do
			if template.iscategory then
				for cname, ctemplate in next, template do
					if cname ~= "iscategory" then
						for index, spellID in ipairs(ctemplate.TALENTS) do
							local spellName = GetSpellInfo(spellID)
							ctemplate.TALENTS[index] = spellName
						end
					end
				end
			else
				for index, spellID in ipairs(template.TALENTS) do
					local spellName = GetSpellInfo(spellID)
					template.TALENTS[index] = spellName
				end
			end
		end
		
		-- Init Template
		local templateName = (ns and ns.dbpc and ns.dbpc.talentTemplateName) or "Booster Leveling" -- Default to "Leveling" if set template doesn't exist
		local categoryName = ns and ns.dbpc and ns.dbpc.activeCategory
		local source = categoryName and self.templates[categoryName] or self.templates
		if (templateName ~= 0) then
			activeTemplate = source[templateName]
			activeTemplateName = (activeTemplate and templateName) or DISABLE_TEXT
			activeCategory = self.templates[categoryName] or nil
			activeCategoryName = self.templates[categoryName] and categoryName or nil
			ns.dbpc.templateName = (activeTemplate and templateName) or 0 --set saved to a valid template or disabled
		else
			activeTemplateName = DISABLE_TEXT
		end
	end

	-- Addon Loaded
	function core:ADDON_LOADED(addon)
		if (addon == "Blizzard_TalentUI") then hooksecurefunc("PlayerTalentFrame_Refresh",TabUpdate) end
	end

	-- Fired when the player changes between dual talent specs, and possibly when learning or unlearning talents (?).
	function core:PLAYER_TALENT_UPDATE(...)
		if (not next(talentTab)) then CacheTalents() end
		self:CheckFreePoints()	
	end

	-- Preview Talents Changed
	-- May not be needed anymore delete if not needed. 
	--[[
	function core:PREVIEW_TALENT_POINTS_CHANGED()
		DEFAULT_CHAT_FRAME:AddMessage("PREVIEW EVENT FIRES")
		self:TalentUpdate()
	end
	]]--

	-- Fired when the player (with Dual Talent Specialization enabled) switches talent builds
	function core:ACTIVE_TALENT_GROUP_CHANGED(...)
		if (StaticPopup_Visible("BoosterSpentTalentPoints")) then
			StaticPopup_Hide("BoosterSpentTalentPoints")
		end
	end

	--------------------------------------------------------------------------------------------------------
	--                                         Template Drop Down                                         --
	--------------------------------------------------------------------------------------------------------

	local info = {}
	local dropdown_category
	local dropdown_in_category

	local function DropDownSelect(frame,arg1,arg2,checked)
		if (ns and ns.dbpc) then
			ns.dbpc.talentTemplateName = frame.value
		end
		if (frame.value == 0) then
			core.dropDown.text:SetText(DISABLE_TEXT)
			activeTemplate = nil
			activeCategory = nil
			activeTemplateName = DISABLE_TEXT
			activeCategoryName = nil
			core:TalentDisableObjects()
			core:GlyphDisableObjects()
			ns.dbpc.talentTemplateName = 0
			ns.dbpc.activeCategory = false
		else
			core.dropDown.text:SetText(frame.value)
			activeTemplate = dropdown_in_category and core.templates[dropdown_category][frame.value] or core.templates[frame.value]
			activeTemplateName = frame.value
			activeCategory = dropdown_in_category and core.templates[dropdown_category] or nil
			activeCategoryName = dropdown_in_category and dropdown_category or nil
			TabUpdate()
			ns.dbpc.talentTemplateName = frame.value
			ns.dbpc.activeCategory = dropdown_in_category and dropdown_category or false
		end
	end

	--lazy way of detecting category click, other way would be to
	--monitor the dropdown level being closed, but this was quicker
	--TODO: Should've used arg1, update someday for this
	local function DropDownCategroySelect(frame, arg1, arg2, checked)
		dropdown_in_category = true;
		DropDownSelect(frame, arg1, arg2, checked)
		dropdown_in_category = false;
		CloseDropDownMenus()
	end

	local function DropDownInitialize(frame,level,menu)
		if level == 2 then
			dropdown_category = UIDROPDOWNMENU_MENU_VALUE
			local source = core.templates[UIDROPDOWNMENU_MENU_VALUE]
			for name, tbl in next, source do
				if name ~= "iscategory" then
					wipe(info)
					info.func = DropDownCategroySelect
					info.text = name
					info.value = name
					info.checked = (name == activeTemplateName)
					UIDropDownMenu_AddButton(info, level)
				end
			end
		elseif level == 1 then
			wipe(info)
			info.func = DropDownSelect
			info.text = DISABLE_TEXT
			info.value = 0
			info.checked = (not activeTemplate)
			UIDropDownMenu_AddButton(info, level)
			for name, tbl in next, core.templates do
				wipe(info)
				if not tbl.iscategory then
					info.func = DropDownSelect
					info.text = name
					info.value = name
					info.checked = (core.templates[name] == activeTemplate)
					UIDropDownMenu_AddButton(info, level)
				else
					info.hasArrow = true
					info.checked = (activeCategoryName == name)
					info.notClickable = true
					info.keepShownOnClick = true
					info.text = name
					info.value = name
					UIDropDownMenu_AddButton(info, level)
				end
			end
		end
	end

	--------------------------------------------------------------------------------------------------------
	--                                            Talent Code                                             --
	--------------------------------------------------------------------------------------------------------

	local function addTooltip(...)
		for i = 1,select('#',...) do
			local widget = select(i,...)
			if widget.EnableMouse then widget:EnableMouse(true) end
			widget:SetScript("OnEnter", function() 
				GameTooltip:SetOwner(widget, "ANCHOR_NONE")
				GameTooltip:SetPoint("TopLeft", widget, "BottomLeft")
				GameTooltip:SetText(TalentTooltipText, nil, nil, nil, nil, true)
			end)
			widget:SetScript("OnLeave", function() GameTooltip:Hide() end)
		end
	end

	local function autoSpecOnClick() StaticPopup_Show("TalentHelperConfirmSpendAll") end

	local function autoSpecOnUpdate(self) if (not activeTemplate) or (GetActiveSpecGroup() ~=  PlayerTalentFrame.talentGroup) then self:Disable() else self:Enable() end end

	-- Initialize Talent Stuff
	function core:TalentInitialize()
	core:TalentDisableObjects()
		-- DropDown
		self.dropDown = CreateFrame("Frame","BoosterTalentDropDown",PlayerTalentFrame,"UIDropDownMenuTemplate")
		self.dropDown:SetPoint("TOPRIGHT", PlayerTalentFrame , "BOTTOMRIGHT", -110,0)
		BoosterTalentDropDownButton:SetHitRectInsets(-100,0,0,0)
		self.dropDown.initialize = DropDownInitialize
		self.dropDown.button = BoosterTalentDropDownButton
		self.dropDown.text = BoosterTalentDropDownText
		self.dropDown.text:SetWidth(100) --forces trunication

		-- Add tooltip to drop down
		addTooltip(self.dropDown,self.dropDown.button)	
		self.dropDown.text:SetText(activeTemplateName)
		
		-- HOOK: PlayerSpecTab_OnClick
		hooksecurefunc("PlayerSpecTab_OnClick",TabUpdate)
		
		-- Create Individual Talent Advisers
		for tal = 1, 18 do
			local name, iconTexture, tier, column, selected, available = GetTalentInfo(tal)
			local frame = _G["PlayerTalentFrameTalentsTalentRow"..tier.."Talent"..column]
			local advise = CreateFrame("Frame",nil,frame)
			advise:SetWidth(20)
			advise:SetHeight(20)
			advise:SetPoint("LEFT",13,0)
			advise:SetBackdrop({ bgFile = "Interface\\ChatFrame\\ChatFrameBackground", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 12, insets = { left = 3, right = 3, top = 3, bottom = 3 } })
			advise:SetBackdropColor(0.1,0.1,0.1,0.8)
			advise:SetBackdropBorderColor(1,1,0,0.8)
			advise:Hide()
			advise.text = advise:CreateFontString(nil,"ARTWORK","GameFontHighlightLarge")
			advise.text:SetPoint("CENTER", 1, -1)
			frame.boosterAdvise = advise
		end
		
		--Spec button
		self.autoSpecButton = CreateFrame("Button", "TalentHelperAutoButton", PlayerTalentFrame, "UIPanelButtonTemplate")
		self.autoSpecButton:SetSize(100, 20)
		self.autoSpecButton:SetPoint("RIGHT", self.dropDown, "LEFT", 18, 2)
		self.autoSpecButton:Show()
		self.autoSpecButton:SetText("Apply Points")
		self.autoSpecButton:SetScript("OnClick", autoSpecOnClick)
		self.autoSpecButton:SetScript("OnUpdate", autoSpecOnUpdate)
		
		-- Cleanup, this func isnt needed anymore
		self.TalentInitialize = nil
	end

	-- Needed to make the disable function work properly
	local curSpecTemp

	-- Talent Update
	function core:TalentUpdate()
		if not activeTemplate then return end
		curSpecTemp = activeTemplate.SPEC
		-- Summary Page
		if (PlayerTalentFrameSpecialization:IsVisible()) then
			for a = 1, GetNumSpecializations() do
				local btn = _G["PlayerTalentFrameSpecializationSpecButton"..a]
				if btn then
					if btn.boosterGlow then
						btn.boosterGlow:Hide()
					end
				end
			end
			local mainTree = activeTemplate.SPEC
			local btn = _G["PlayerTalentFrameSpecializationSpecButton"..mainTree]
			if (not btn.boosterGlow) then CreateTalentButtonGlowTexture(btn) end
			btn.boosterGlow:Show()
		-- Talent Tree Page
		else
			for a = 1, 18 do
				local name, _, t, c, s, av  = GetTalentInfo(a)
				local frame = _G["PlayerTalentFrameTalentsTalentRow"..t.."Talent"..c]
				if frame then
					if frame.boosterGlow then
						frame.boosterGlow:Hide()
					end
				end
			end
			for index, name in ipairs(activeTemplate.TALENTS) do
			end
			local shownSpec = PlayerTalentFrame.talentGroup
			local nextTalent = activeTemplate.TALENTS[1]
			local id, name, description, icon, background, role = GetSpecializationInfo(shownSpec)
			local totalPoints = 6
			for tal = 1, 18 do
				local name, iconTexture, tier, column, selected, available = GetTalentInfo(tal)
				local frame = _G["PlayerTalentFrameTalentsTalentRow"..tier.."Talent"..column]
				local advise = frame.boosterAdvise
				
				advise:Show()
				advise.text:SetText("+")
				-- Highlight the first talent with green code needs to be added!
				if (name == nextTalent) then
					advise.text:SetTextColor(0,1,0)
					advise:SetAlpha(1)
					if (not frame.boosterGlow) then CreateTalentButtonGlowTexture(frame) end
					frame.boosterGlow:Show()
					nextTalent = GetNextTalent(nextTalent)
				-- Color unavailable talents grey
				elseif (not available) then
					advise.text:SetTextColor(.7,.7,.7)
					advise:SetAlpha(1)
				else
					advise.text:SetTextColor(1, 0 ,0)
					advise:SetAlpha(1)
				end
			end
		end
	end

	-- Hides all objects
	function core:TalentDisableObjects()
		-- Summary Page
		local mainTree = curSpecTemp
		if (not mainTree) then return end
		local btn = _G["PlayerTalentFrameSpecializationSpecButton"..mainTree]
		if (btn.boosterGlow) then	btn.boosterGlow:Hide() end
		curSpecTemp = nil
		-- Talent Tree Page
		for tal = 1, 18 do
			local name, iconTexture, tier, column, selected, available = GetTalentInfo(tal)
			local frame = _G["PlayerTalentFrameTalentsTalentRow"..tier.."Talent"..column]
			local advise = frame.boosterAdvise
			if (frame.boosterGlow) then frame.boosterGlow:Hide() end
			if (advise) then
				advise:Hide()
			end
		end
	end

	--------------------------------------------------------------------------------------------------------
	--                                             Glyph Code                                             --
	--------------------------------------------------------------------------------------------------------
	-- Tooltip Hook
	local function ShowRecommendedGlyphTooltip(self)
		if (self.boosterAdvisedGlyphID) then
			local tip = core.tooltip
			tip:SetOwner(self,"ANCHOR_NONE")
			tip:SetPoint("TOPLEFT",self,"TOPRIGHT")
			tip:SetSpellByID(self.boosterAdvisedGlyphID)
			tip:AddLine("Manaview Advised Glyph",0,1,0)
			tip:AddLine("If you don't know this glyph, \nget it from the Auction House or an Inscriptionist.", 1,0,0)
			tip:Show()
		end
	end

	-- Initialize Glyph Stuff
	function core:GlyphInitialize()

		-- Create Tooltip
		self.tooltip = CreateFrame("GameTooltip","BoosterGlyphTip",UIParent,"GameTooltipTemplate")
		self.tooltip:SetOwner(UIParent,"ANCHOR_NONE")
		self.tooltip:SetScale(0.9)
		
		-- TipTac Support?
		if (TipTac) then
			TipTac:AddModifiedTip(self.tooltip)
		end
		
		-- Create Glyph Helper Text
		for i = 1, NUM_GLYPH_SLOTS do
		
			local glyph = _G["GlyphFrameGlyph"..i]
			
			-- Hook Tooltip
			glyph:HookScript("OnEnter",ShowRecommendedGlyphTooltip)
			glyph:HookScript("OnLeave",function(self) core.tooltip:Hide(); end)
			
			-- Add Advise Text
			local text = glyph:CreateFontString(nil,"OVERLAY")
			text:SetFont(GameFontNormal:GetFont(),10,"OUTLINE")
			text:SetPoint("CENTER",0,-11)
			glyph.boosterAdvised = text
		end
		
		-- Cleanup, this func isnt needed anymore
		self.GlyphInitialize = nil
	end

	 local function tremovebyval(tab, val)
	   for k,v in pairs(tab) do
		 if(v==val) then
		   table.remove(tab, k);
		   return true;
		 end
	   end
	   return false;
	 end

	 local function isActiveTemplateGlyph(ID)
		for m = 1, 3 do
			for z = 1, 2 do
				if activeTemplate[z][m] == ID then return true end
			end
		end
		return false
	 end


	--We should show reccomendations in the proper order!
	-- on the glyph tab, the major glyphs go: top=2, bottom left=4, bottom right=6.
	-- on the glyph tab, the mino glyphs go: top right=1, top left=3, bottom=5.

	function core:GlyphUpdate()

		-- Do nothing if there is no active template
		if not activeTemplate then return end
		
		-- Keep a variable of he currently chosen spec. 
		local shownSpec = PlayerTalentFrame.talentGroup
		
		-- Copy our glyph advice to a temp table
		for type = 1, #activeTemplate do
			wipe(glyphTmp[type])
			for index = 1, #activeTemplate[type] do
				tinsert(glyphTmp[type],activeTemplate[type][index])
			end
		end
		
		--Remove glyphs from the temp table that we find in place
		for i = 1, NUM_GLYPH_SLOTS do
			local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i,shownSpec)
			-- Is this a recommended glyph?
			if enabled then
				if glyphSpell then
					tremovebyval(glyphTmp[glyphType], glyphSpell)
				end
			end
		end
		
		-- Scan #2
		for i = 1, NUM_GLYPH_SLOTS do
			local glyph = _G["GlyphFrameGlyph"..i]
			local ring = glyph.ring
			local advised = glyph.boosterAdvised
			local enabled, glyphType, glyphTooltipIndex, glyphSpell, icon = GetGlyphSocketInfo(i,shownSpec)
			local isThere = isActiveTemplateGlyph(glyphSpell)
			-- only check if enabled slot
			if (enabled) then
				if (isThere) then
					core:mdebug("glyph OK", glyphSpell, i)
					glyph.boosterAdvisedGlyphID = nil
					ring:SetVertexColor(0,1,0)
					advised:SetText("")
				end
				if (not isThere) or (not tostring(glyphSpell)) then
					core:mdebug("glyph BAD", glyphSpell, i)
					local id = tremove(glyphTmp[glyphType], 1)
					glyph.boosterAdvisedGlyphID = id
					ring:SetVertexColor(1,0,0)
					for x = 1, GetNumGlyphs() do
						local name, glyphType, isKnown, icon, castSpell = GetGlyphInfo(x)
						if (castSpell == id) then
							advised:SetText(name)
						end
					end
				end
			else
				glyph.boosterAdvisedGlyphID = nil
				ring:SetVertexColor(1,1,1)
				advised:SetText("")
			end
		
		end
	end


	-- Hides all objects
	function core:GlyphDisableObjects()
		for i = 1, NUM_GLYPH_SLOTS do
			local glyph = _G["GlyphFrameGlyph"..i]
			if (glyph) then
				local ring = glyph.ring
				local advised = glyph.boosterAdvised
				glyph.boosterAdvisedGlyphID = nil
				ring:SetVertexColor(1,1,1)
				if (advised) then
					advised:SetText("")
				end
			end
		end
	end
end