﻿local _,ns = ...

local lh = LightHeaded
local tg = Guide
local fill_completed, fill_full, start_point, select_guide, quered = false, false, false, false, false

function ns:AddGuide(name, next_guide, side, guide)
  local steps, index, count = {}, 1, 0
  for step in guide:gmatch("[^\r\n]+") do
    local class = step:match("|C|(.-)|")
    local race = step:match("|R|(.-)|")
    if (not class or class == ns.Class) and (not race or race == ns.Race) then
      count = count + 1
    end
    steps[index] = string.gsub(step,'|N|','|N|(Step '..count..') ')
    index = index +  1
  end
  local new_guide = table.concat(steps,"\n")
  tg:RegisterGuide(name, next_guide, side, function() return new_guide; end)
end

local function AddButtons()
  local tekbutton = LibStub("tekKonfig-Button")
  local fr = Guide.configpanel
  local report = tekbutton.new(fr, "BottomRight", fr, "BottomRight", -10, 11)
  report:SetWidth(110)
  report:SetText("Report Error")
  report.tiptext = "Report any problems with the addon or routes. Only works when using the updater."
  report:SetScript("OnClick", function(self)
    ns.error_report:Show()
  end)

  local bt = tekbutton.new(fr, "right", report, "left")
  bt:SetWidth(140)
  bt:SetText("Reset all settings!")
  bt.tiptext = "Reset all settings of Guide"
  bt:SetScript("OnClick", function(self)
    tg.reset = true
    BOOSTER_GuideAlphaDB = nil
    BOOSTER_GuideQuestHistoryDB = nil
    tg:Initialize()
    ReloadUI()
  end)
  local fill_btn = tekbutton.new(fr, "right", bt, "left", 0, 0)
  fill_btn:SetWidth(140)
  fill_btn:SetText("Auto select guide")
  fill_btn.tiptext = "Automaticly find and select a guide appropriate for your character"
  fill_btn:SetScript("OnClick", ns.SelectGuide)
end

local function findQid(index)
	local action = tg.actions[index]
	local tags = tg.tags[index]
	local qid = tags:match("|QID|(%d+)|")
	if qid then return tonumber(qid) end
	if (action == 'KILL' or action == 'NOTE') and tags:match("|Q|") then
		local quest = tags:match("|Q|(.-)|")
		while index > 0 do
			if tg.quests[index] == quest then 
				local qid = tg.tags[index]:match("|QID|(%d+)|")
				return tonumber(qid)
			end
			index = index - 1
		end
	end
end

local function getQuestChain(qid) 
	local chain = select(13, lh:GetQuestInfo(qid))
	if chain == "" or chain == nil then return end
	local ids = {}
	for step, id in chain:gmatch("([^\029]+)\029([^\029]+)") do
		tinsert(ids, id)
	end
	return ids
end

local function getCompleted()
	local completed = {}
	for x, q in ipairs(tg.quests) do
		for c, y in pairs(tg.turnedin) do
			if q == c and y then completed[x] = true; break end
		end
	end
	return completed
end

local function checkOtherSteps()
	tg:DebugF(1, "CheckOtherSteps(): start_point = %d", start_point)
	local completed, last = getCompleted(), 0
	if start_point then last = start_point - 1 end
	for x, y in pairs(completed) do if x > last then last = x end end
	tg:DebugF(1, "CheckOtherSteps(): last = %d", last)
	--[[
	local uncompleted = {}
	local completed_quests = {}
  
	local quest_chains = {}
	for x = last + 1, #tg.actions do -- find quests that are in quest chains of rest of guide
		if tg.actions[x] == 'ACCEPT' then 
			local chain = getQuestChain(tonumber(tg.tags[x]:match("|QID|(%d+)|")))
			if chain then 
				for x,y in ipairs(chain) do quest_chains[y] = true end 
			end
		end
	end

	for x = 1, last do -- we are looking for first uncompleted quest need for any chain later
		qid = tg.tags[x]:match("|QID|(%d+)|")
		if tg.actions[x] == 'ACCEPT' and not completed[x] and quest_chains[qid] then
			max = x
			break
		end
	end
	--]]
  
	local high = nil
	for x = (start_point or 1), last do -- we are looking for first uncompleted quest after the start point
		if tg.actions[x] == 'ACCEPT' and not completed[x] then
			high = x
			break
		end
	end
	tg:DebugF(1, "CheckOtherSteps(): high = %d; high or last = %d", last, high or last)

	for x = 1, high or last do
		if fill_full == "soft" then
			if tg.tags[x]:match("|LVL|") or
				tg.actions[x] == 'RUN' or
				tg.actions[x] == 'HEARTH' or 
				tg.actions[x] == 'SETHEARTH' or 
				tg.actions[x] == 'FLY' or 
				tg.actions[x] == 'GETFLIGHTPOINT' or 
				tg.actions[x] == 'BOAT' or 
				tg.tags[x]:match("|TRAIN|") or
				tg.tags[x]:match("|U|6948|") then
				tg:SetTurnedIn(x, true) 
			end
		else
			logi = select(2, tg:GetObjectiveStatus(x))
			--don't mark quests in log complete, accepted and completed actions are handled by display logic in ObjectivesFrame.lua
			--DO mark quests in the log complete that are before the start point
			if not logi or x < (start_point or 0) then
			tg:SetTurnedIn(x, true)
			end
		end
	end
  
  start_point = false;

	--for x = 1, last do
		--if tg.actions[x] == 'TURNIN' then
			--if completed[x] then tinsert(completed_quests,x) else tinsert(uncompleted, x) end
		--end
	--end
	--print(100*#uncompleted/(#uncompleted+#completed_quests))
	--return quest_chains
end

local function fillCompleted()
	local completed = {}
	GetQuestsCompleted(completed)
	for i, t in ipairs(tg.tags) do
		qid = findQid(i)
		if qid and completed[qid] then tg:SetTurnedIn(i, true) end
	end
	if fill_full then 
		checkOtherSteps()
		fill_full = nil
	end
	ns:CheckLevelSteps()
end

local function QueryQuestsCompleted()
	quered = true
	if select_guide then ns:SelectGuide() end
	if fill_completed then fillCompleted() end
end

function ns:FillCompleted(full)
	fill_full = full
	if quered then
		fillCompleted()
	else
		fill_completed = true
		QueryQuestsCompleted()
	end
end

--Checks to see if the player has leveled beyond their current guide max
--Level must be passed because when the level up event is sent UnitLevel("player") returns the wrong level
ns.LevelUpGuideCheck = function (self,level)
	if not lh then return end;
	local high, qlevel = 0, 0;
	for i = 1, #tg.actions do
		qid = tg:GetObjectiveTag("QID", i);
		if qid then
			_, _,qlevel = lh:GetQuestInfo(qid)
			if tonumber(level) and tonumber(qlevel) then
			    high = max(high, qlevel)
			end
		end
	end

	if not(tonumber(high)) then return end
	
	if tonumber(level) > tonumber(high) + GetQuestGreenRange() then
		StaticPopup_Show ("BoosterOverGuide")
	end
end

local m,i = ns:RegisterModule(nil,'FillCompleted','TRAINER_SHOW','PLAYER_LEVEL_UP','QUEST_QUERY_COMPLETE','NEUTRAL_FACTION_SELECT_RESULT')

ns.CheckLevelSteps = function (self,level)
	level = level or UnitLevel("player")
	for i, t in ipairs(tg.tags) do
		lvl = tonumber(t:match("|LVL|(%d+)|"))
		if lvl and lvl <= tonumber(level) then tg:SetTurnedIn(i, true) end
	end
end

m.NEUTRAL_FACTION_SELECT_RESULT = function(self)
	local englishFaction, localizedFaction = UnitFactionGroup('player')
	DEFAULT_CHAT_FRAME:AddMessage("Picked Faction")
	if englishFaction ~= "Neutral" then
		local guideToLoad = "Booster Pandaren Choose "..englishFaction.." (12)" 
		if IsAddOnLoaded("Booster "..englishFaction) then Guide:LoadGuide(guideToLoad) end
	end
end

m.QUEST_QUERY_COMPLETE = function (self)
	quered = true
	if select_guide then ns:SelectGuide() end
	if fill_completed then fillCompleted() end
end

m.PLAYER_LEVEL_UP = function (self,level)
	ns:CheckLevelSteps(level)
	ns:LevelUpGuideCheck(level)
end

m.TRAINER_SHOW = function (self)
	if tg.tags[tg.current]:match("|TRAIN|") then
		tg:SetTurnedIn(tg.current, true) 
	end
end

ns.WaypointReached = function (self,active_point)
    ns:FireEvent('TOMTOM_WAYPOINT_REACHED',active_point)
end
