﻿-- some locals to speedup access to them
local ww = WidgetWarlock
local tg = Guide
local lh = LightHeaded
local status_frame = tg.statusframe
local _,ns = ...
local sfm,sfmi = ns:RegisterModule()

function getChain(qid) -- return chain info about given quest (with links to outhers)
	if not lh then return end
	local chain = select(13, lh:GetQuestInfo(qid))
  if chain == "" or chain == nil then return end
	local text = "|cffffffffQuest Chain:|r"
	for step, id in chain:gmatch("([^\029]+)\029([^\029]+)") do
		local name = lh:GetQuestName(id)
		local link = "|cFF0066FF|Hlhref:quest:"..id.."|h["..name.."]|h|r"
		if qid == tonumber(id) then
			text = text .. "\n>> "..step..".|r " .. name
		else
			text = text .. "\n     |cffffd100 "..step..".|r " .. link
		end
	end
  return text
end

-- funtion that setup frame and show it if needed
local showFrame = nil

local function createButton(text, width, parent, ...)
    local btn = CreateFrame("button", nil, parent)--, "UIPanelButtonTemplate")
    btn:SetWidth(width) btn:SetHeight(64)
    if select(1, ...) then btn:SetPoint(...) end
    btn:SetText(text)
    btn:SetScript("OnClick", function(self)
        btn:onClick()
    end)
    return btn
end

local frame = CreateFrame("Frame", "HexiLightheadedInfo", UIParent) -- our main frame
frame:SetFrameStrata("DIALOG")
frame:SetWidth(410) 
frame:SetPoint("TOPRIGHT", status_frame, "BOTTOMRIGHT")
frame.bg = frame:CreateTexture(nil,'BACKGROUND')
frame.bg:SetAllPoints(frame)
frame.bg:SetTexture(0,0,0,0.5)
frame.bg.borders = {}
local b = frame.bg.borders
for i = 1,4 do
    b[i] = frame:CreateTexture(nil,'BORDER')
    b[i]:SetTexture(1,1,1,0.3)
    b[i]:SetBlendMode('ADD')
end
b[1]:SetPoint('BOTTOMLEFT',frame.bg,'TOPLEFT',-2,0)
b[1]:SetPoint('BOTTOMRIGHT',frame.bg,'TOPRIGHT',2,0)
b[1]:SetHeight(1)
b[2]:SetPoint('TOPLEFT',frame.bg,'BOTTOMLEFT',-2,0)
b[2]:SetPoint('TOPRIGHT',frame.bg,'BOTTOMRIGHT',2,0)
b[2]:SetHeight(1)
b[3]:SetPoint('TOPLEFT',b[1],'BOTTOMLEFT')
b[3]:SetPoint('BOTTOM',b[2],'TOP')
b[3]:SetWidth(1)
b[4]:SetPoint('TOPRIGHT',b[1],'BOTTOMRIGHT')
b[4]:SetPoint('BOTTOM',b[2],'TOP')
b[4]:SetWidth(1)


local comments_btn = createButton("", 256, frame)

local showTooltip = function(self)
    GameTooltip:Hide()
    status_frame.txt = "Left-click: Toggle the quest log.\n\nRight-click: Toggle the step list.\n\nShift-click: Mark the current step as completed."
    GameTooltip:SetOwner(status_frame, "ANCHOR_NONE")
    GameTooltip:SetPoint("BOTTOMRIGHT", status_frame, "TOPLEFT")
    GameTooltip:SetText(status_frame.txt, nil, nil, nil, nil, true)
    GameTooltip:Show()
end

status_frame:SetScript("OnLeave", function() GameTooltip:Hide() end)
status_frame:SetScript("OnEnter", function(...)
     showFrame()
     showTooltip()
     if lh and ns.dbpc.show_comments == true then comments_btn:Click() end
end)

local counter, refresh = 0, 0
frame:SetScript("OnUpdate", function(x, last)
	refresh = refresh + last                    -- don't do unnesesary checking to offen
	if refresh < 0.1 then return end    -- and save cpu cycles. 10 fps is enought for us
    if tg.objectiveframe:IsVisible() then frame:Hide() end
	if (MouseIsOver(status_frame) or MouseIsOver(frame)) then
		counter = 0
	else
		counter = last + counter
		if counter > 0.5 then
			frame:Hide()
			counter = 0
		end
	end
end)

local note_text = ww.SummonFontString(frame, nil, "GameFontNormal", nil, "TOPLEFT", frame, "TOPLEFT", 10, -15)
note_text:SetWidth(400)
note_text:SetJustifyH("left")

local lh_text = CreateFrame("SimpleHTML", nil, frame)
lh_text:SetPoint("TOPLEFT", note_text, "BOTTOMLEFT", 0, -15)
lh_text:SetWidth(395)
lh_text:SetHeight(1)
lh_text:SetFontObject("GameFontNormal")
lh_text:SetScript("OnHyperlinkClick", function(...) 
    QuestLogFrame:Show()
    lh:OnHyperlinkClick(...) 
    frame:Hide()
end)

local comment_text = CreateFrame("SimpleHTML", nil, frame)
comment_text:SetPoint("TOPLEFT", lh_text, "BOTTOMLEFT", 0, -15)
comment_text:SetWidth(395)
comment_text:SetHeight(1)
comment_text:SetFontObject("GameFontNormal")
comment_text:SetScript("OnHyperlinkClick", function(...) lh:OnHyperlinkClick(...) end)
comment_text:SetScript("OnHyperlinkEnter", function(...) lh:OnHyperlinkEnter(...) end)
comment_text:SetScript("OnHyperlinkLeave", function() LightHeadedFrameSub.tooltip:Hide() end)

comments_btn:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\btn_show")
comments_btn:SetHighlightTexture("Interface\\AddOns\\ManaCore\\Media\\btn_show")
comments_btn:SetPoint("BOTTOM", frame, "BOTTOM", 50, -15)
comments_btn:SetScale(0.8, 0.8)

local next_btn = createButton(">>", 256, frame, "TOPRIGHT", comment_text, "TOPRIGHT", 95, 15)
next_btn:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\btn_next")
next_btn:SetDisabledTexture("Interface\\AddOns\\ManaCore\\Media\\btn_next_disabled")
next_btn:SetHighlightTexture("Interface\\AddOns\\ManaCore\\Media\\btn_next")
next_btn:SetScale(0.8, 0.8)

local prev_btn = createButton("<<", 256, frame, "TOPLEFT", comment_text, "TOPLEFT", 0, 15)
prev_btn:SetNormalTexture("Interface\\AddOns\\ManaCore\\Media\\btn_previous")
prev_btn:SetDisabledTexture("Interface\\AddOns\\ManaCore\\Media\\btn_previous_disabled")
prev_btn:SetHighlightTexture("Interface\\AddOns\\ManaCore\\Media\\btn_previous")
prev_btn:SetScale(0.8, 0.8)

local last, last_qid, last_note = nil, nil, ''
local show_comments, last_comment, comments_count = false, nil, 0

local function show_comment(index)
	if not lh then return end
    next_btn:Enable()
    prev_btn:Enable()
    if index == 1 then
        prev_btn:Disable()
    elseif index == comments_count then
        next_btn:Disable()
    end
    body = lh:GetCommentText(last_qid, index)
    comment_text:Show()
    comment_text:SetText("|cFFFFFFFF|h                                            Comment "..index..":|r\n\n"..body)
    frame:SetHeight(lh_text:GetHeight() + lh_text:GetRegions():GetHeight() + comment_text:GetRegions():GetHeight() + 90)
    last_comment = index
end

comments_btn.onClick = function()
    comments_btn:Hide()
    if not last_qid then return end
    if comments_count > 1 then
        next_btn:Show()
        prev_btn:Show()
    else
        next_btn:Hide()
        prev_btn:Hide()
    end
    show_comment(1)
    showFrame()
end

next_btn.onClick = function()
    show_comment(last_comment + 1)
    showFrame()
end

prev_btn.onClick = function()
    show_comment(last_comment - 1)
    showFrame()
end


function findQid()
    local action = tg.actions[tg.current]
    local qid = last:match("|QID|(%d+)|")
    if qid then return qid end
    if (action == 'KILL' or action == 'NOTE') and last:match("|Q|") then
        local x = tg.current
        local quest = last:match("|Q|(.-)|")
        while x > 0 do
            if tg.quests[x] == quest then 
                local qid = tg.tags[x]:match("|QID|(%d+)|")
                return tonumber(qid)
            end
            x = x - 1
        end
    end
end

function ns:HideComments()
    last = ""
end

showFrame = function()
    local tags = tg.tags[tg.current]
    if last ~= tags then
        last = tags
        last_note = tg:GetObjectiveTag("N", tg.current)
        note_text:SetText(last_note)
        next_btn:Hide()
        prev_btn:Hide()
        comment_text:SetText("")
        lh_text:SetText("")
        comments_btn:Hide()

        last_qid = findQid()
        if last_qid then
            last_qid = tonumber(last_qid)
            local chain = getChain(last_qid)
            lh_text:SetText(chain)
            comment_text:SetPoint("TOPLEFT", lh_text, "TOPLEFT", 5, -15 - lh_text:GetRegions():GetHeight())
            comments_count = lh and lh:GetNumQuestComments(last_qid) or 0
            comments_btn:Hide()
            if comments_count > 0 then
                --comments_btn:SetPoint("TOP", lh_text, "TOP", 0, -10 - lh_text:GetRegions():GetHeight())
                comments_btn:Show()
                frame:SetHeight(lh_text:GetRegions():GetHeight()+note_text:GetHeight()+70)
                if ns.dbpc.show_comments == true then comments_btn:Click() end
            else
                frame:SetHeight(lh_text:GetRegions():GetHeight()+note_text:GetHeight()+50)
            end
        else
            frame:SetHeight(note_text:GetHeight()+32)
        end
    end
    frame:Position()
end

ns.lh_frame = frame
function frame:Position()
    --if last_qid or last_note ~= '' then
    --[[
        local x = status_frame:GetTop() < GetScreenHeight() / 2 
        local y = status_frame:GetLeft() < GetScreenWidth() / 2
        if x and y then
            --x, y =    frame:GetWidth() - status_frame:GetWidth() + 50, frame:GetHeight() + status_frame:GetHeight()
            x, y = 90, frame:GetHeight() + status_frame:GetHeight()
        elseif x and not y then -- bottomright corner
            x, y = 90, frame:GetHeight() + status_frame:GetHeight()
        elseif not x and not y then -- topright corner
            x, y = 90, -status_frame:GetHeight() + 6
        elseif not x and y then
            x, y = 90, -status_frame:GetHeight() + 6
            --x, y = frame:GetWidth() - status_frame:GetWidth() + 50, 0
        end
        frame:SetPoint("TOPRIGHT", status_frame, "BOTTOMRIGHT", x, y)
        frame:Show() ]]
        
    
        --local quadrant,opposite = ns.GetQuadrant(status_frame)
        frame:SetPoint('TOP', status_frame, 'BOTTOM')
        frame:Show()
end

local old_on_show = tg.objectiveframe:GetScript("OnShow")
tg.objectiveframe:SetScript("OnShow", function()
    frame:Hide()
    if old_on_show then old_on_show() end
end)

local old_on_hide = tg.objectiveframe:GetScript("OnHide")
tg.objectiveframe:SetScript("OnHide", function()
    if MouseIsOver(status_frame) then showFrame() end
    if old_on_hide then old_on_hide() end
end)

local old_self_turned_in = tg.SetTurnedIn
function tg.SetTurnedIn(...)
    old_self_turned_in(...)
    if frame:IsVisible() then showFrame() end -- update frame
end
