﻿-- ManaCore basic module API. Further documentation to follow.
local usedebug-- = true
local AddonName,ns = ...
ManaCore = ns

local racesEnglish = {
	["BloodElf"] = "Blood Elf",
	["NightElf"] = "Night Elf",
}
local raceEnglish = function ()
	local _,race = UnitRace('player')
	return racesEnglish[race] or race
end

local classEnglish = function ()
	local _,class = UnitClass('player')
	if class == "DEATHKNIGHT" then
		return "Death Knight"
	else
		class = class:sub(1,1)..class:sub(2):lower()
		return class
	end
end

ns.Class = classEnglish()
ns.Race = raceEnglish()

local DebugFrame = CreateFrame('ScrollingMessageFrame','ManaCoreDebug',UIParent)
DebugFrame.scroll = CreateFrame('StatusBar',nil,DebugFrame)
DebugFrame:SetMaxLines(4096) -- If we really need that much debug info in the course of a session, something is *very* wrong.
local function printhelp(...) if select('#',...)>0 then return tostring((select(1,...))), printhelp(select(2,...)) end end
local debug = function (...)
	DebugFrame:AddMessage('ManaCore: '..date('%X')..': '..strjoin(' | ',printhelp(...)))
	DebugFrame.scroll:SetMinMaxValues(0,DebugFrame:GetCurrentLine())
end
local temp = {}
local ptable = function (t)
	table.wipe(temp)
	if type(t) == 'table' then
		for k,v in pairs(t) do
			temp[#temp+1] = tostring(k)..' = '..tostring(v)
		end
		return unpack(temp)
	else return tostring(t) end
end

-- Utility funciton to dump a string's contents into table form, separated by pattern or unique character
local split = function (str,pat)
	local t = {}
	local fpat = "(.-)"..pat
	local last_end = 1
	local s, e, cap = str:find(fpat, 1)
	while s do
		if s ~= 1 or cap ~= "" then
			t[cap] = true
		end
		last_end = e+1
		s, e, cap = str:find(fpat, last_end)
	end
	if last_end <= #str then
		cap = str:sub(last_end)
		t[cap] = true
	end
	return t
end

local FireEvent = function(self,event,...)
	for i,module in pairs(self.RegisteredModules) do
		local f = self.ModuleEvents[event] and self.ModuleEvents[event][i] and module[event]
		if f then
			if event ~= 'WORLD_MAP_UPDATE' then self:debug(i,event,...) end -- Fuck you, worldmap
			f(module,...)
		end
	end
	if self.GlobalEvents[event] then
		self:debug('Global Event',event,...)
		for core,cns in pairs(ns.RegisteredCores) do
			if core~=self.CoreName then cns:FireEvent(event,...) end
		end
	end
	if event ~= 'WORLD_MAP_UPDATE' then self:debug('FireEvent',event,...) end
end

local addEvent = function (self,m,event)
	if type(event) ~= 'string' then return debug('attempt to register invalid event "'..tostring(event)..'" for module '..m) end
	self.EventFrame:RegisterEvent(event)
	self.ModuleEvents[event] = self.ModuleEvents[event] or {}
	self.ModuleEvents[event][m] = self.ModuleEvents[event][m] or false
	rawset(self.RegisteredModules[m].Events,event,true)
	
	-- use a temporary false flag to make sure we're not incrementing the event index on an existing event
	if self.ModuleEvents[event][m] == false then
		self.RegisteredEvents[event] = self.RegisteredEvents[event] or 0
		self.RegisteredEvents[event] = self.RegisteredEvents[event] + 1
		self:debug(m,event,'event registered')
	end
	self.ModuleEvents[event][m] = true
end

local remEvent = function (self,m,event)
	if not(type(m)=='string' and type(event)=='string') then return end
	if not(self.RegisteredEvents[event] and self.ModuleEvents[event] and self.ModuleEvents[event][m]) then return end
	self.RegisteredEvents[event] = self.RegisteredEvents[event] - 1
	self.ModuleEvents[event][m] = nil
	if self.RegisteredEvents[event] <= 0 then
		self.RegisteredEvents[event] = nil
		self.EventFrame:UnregisterEvent(event)
	end
	self:debug(m,event,'event unregistered')
end

local ChildFunctions = {
	-- Make it a little easier to register module events on the fly by providing module:[Un]RegisterEvent funcs
	RegisterEvent = function (self,...)
		self.parent:RegisterModuleEvent(self.ModuleName,...)
	end,
	
	UnregisterEvent = function (self,...)
		self.parent:UnregisterModuleEvent(self.ModuleName,...)
	end,
	
	mdebug = function (self,...)
		return self.parent:debug(self.ModuleName,...)
	end
}

local InitDB = function (self,t)
	if getmetatable(t)=='locked' then return end
	local core = ns.RegisteredCores[t._PARENT]
	local base
	if t._LOOKUP and not(_G[t._LOOKUP]) then
		_G[t._LOOKUP] = {}
		base = _G[t._LOOKUP]
	elseif t._LOOKUP then
		base = _G[t._LOOKUP]
	else
		base = t._DIRECT
	end
	setmetatable(t,{
		-- Set __newindex to trigger events and also prevent modifying the proxy table
		__newindex = function (self,k,v)
			local old = base[k]
			rawset(base,k,v)
			if old ~= v then
				core:FireEvent(t._EVENT,k,old,v)
			end
			core:debug('metamethod',t._EVENT,k,old,v)
		end,
		
		-- Return values directly from savedvars
		__index = function (self,k)
			return base[k]
		end,
		
		__metatable = 'locked'	-- Prevent future modification of metatable
	})
	rawset(t,'_INITIALIZED',true)
	if t._DEFAULT then
		for k,v in pairs(t._DEFAULT) do
			if base[k] == nil then
				rawset(base,k,v)
				core:debug('InitDB setting default',k,v)
			end
		end
	end
	self:debug('variables ready',t._READY)
	self:FireEvent(t._READY)
	ns:FireEvent('CORE_VARIABLES_READY',t._PARENT,t._PROXY,t._READY,t._EVENT)
	return t
end

local CoreFunctions = {
	-- FIXME: Revamp
	debug = function (self,...)
		local from = self.ModuleName and self.parent or self
		DebugFrame:AddMessage(date('%X')..'| '..from.CoreName..': '..strjoin(' | ',printhelp(...)))
		DebugFrame.scroll:SetMinMaxValues(0,DebugFrame:GetCurrentLine())
	end,
	
	-- Trigger an internal event with args - core:TriggerEvent('TEST_EVENT','arg1','arg2')
	FireEvent = function(self,event,...)
		return self.EventHandler(self.EventFrame,event,...)
	end,
	
	-- General event handler.
	EventHandler = function (self,event,...)
		if not(self.parent.RegisteredEvents[event]) then return end
		self.LastEvent = event or self.LastEvent
		FireEvent(self.parent,event,...)
	end,

	-- Metamethods to automate savedvariable-loaded/changed internal events using info from RegisterCoreModule. DO NOT CALL MANUALLY.
	InitDB = function (self,single)
		if single then
			return InitDB(self,single)
		end

		for _,t in pairs(self.DBProxies) do
			InitDB(self,t)
		end
	end,
	
	-- Return a module's namespace by name/index
	GetModule = function (self,name)
		if type(name)~='string' then return end
		return self.RegisteredModules[name]
	end,
	
	-- Register an event or events to a module by name/index
	RegisterModuleEvent = function (self,m,...)
		for i = 1,select('#',...) do
			local e = select(i,...)
			local t = type(e)
			if t == 'string' then
				addEvent(self,m,select(i,...))
				local n = select(i+1,...)
				if type(n) == 'function' then
					self.RegisteredModules[m][e] = n
				end
			end
		end
	end,
	
	-- Unregister an event or events from a module by name/index
	UnregisterModuleEvent = function (self,m,...)
		for i = 1,select('#',...) do
			remEvent(self,m,select(i,...))
		end
	end,
		
	--[[ core:RegisterModule(namespace,name,desc) - Registers a module, allowing the use of module-specific internal events and providing a frameless event handler
		Args (all optional):
			namespace: Specifies the table ManaCore looks in when firing or passing along events. If nil or invalid, a table will be created.
			name: Specifies the name of the module if desired for easier lookups using ManaCore:GetModule(name).
			desc: Description of the module if not set in namespace.desc (planned for ingame loaded module list).
		
		If a name is not specified, the module will use a numeric index instead.
		Registering an existing name/index is not allowed.
		
		Returns namespace and name or index and registers 
	]]
	RegisterModule = function (self,namespace,name,...)
		self.MIndex = self.MIndex or 0
		
		local m = namespace or {}
		if type(m) ~= 'table' then m = {}; end
		if (type(name) ~= 'string') then
			self.MIndex = self.MIndex + 1
			name = self.MIndex
		end
		
		if self.RegisteredModules[name] then return end
		if not(getmetatable(m)) then
			self.RegisteredModules[name] = setmetatable(m,{__index = ChildFunctions})
		else
			self.RegisteredModules[name] = m
			m.mdebug = ChildFunctions.mdebug
			for k,v in pairs(ChildFunctions) do
				m[k] = m[k] or v
			end
		end
		m.parent = self
		m.ModuleName = name
		
		-- Optionally, just use module.RegisteredEvents[event] = true to register an event.
		m.Events = setmetatable({},{__newindex = function(self,k,v)
			if tonumber(k) and type(v) == 'string' then
				k = v
				v = true
			end
			if type(v) == 'function' then
				m[k] = v
			end
			rawset(self,k,v)
			m:RegisterEvent(k)
		end})
		
		for k,v in pairs(self.DBEvents) do
			m.Events[k] = v
		end
		for i = 1,select('#',...) do
			local event = select(i,...)
			m.Events[event] = true
		end
		
		self:debug(name,'module registered')
		return m,name
	end,
	
	-- Remove module completely. Likely not to be used, here for completeness.
	UnregisterModule = function (self,name)
		if not(name) or type(name) ~= 'string' then return end
		if not(self.RegisteredModules[name]) then return self:debug('attempt to unregister nonexisting module '..name) end
		self.RegisteredModules[name] = nil
	end,
}

--[[ ManaCore:RegisterCoreModule( {namespace}, 'CoreModuleName', ... )
	namespace = addon-specific namespace passed via ... at file load
	CoreModuleName = actual name of addon passed via ... at file load
	
	'...' serves two purposes depending on type of each entry:
		string: Adds an event to register automatically to each module.
		table: Adds a savedvariable reference for metatable application. Format: { [lookup = 'string' or direct = table], proxy = 'string', event = 'string'}
			lookup: String to pass to _G to look up savedvariables during proxy access and ADDON_LOADED('CoreModuleName')
			proxy: Name of the proxy subtable (ie, Booster uses Booster.db as the savedvar proxy, so proxy == 'db')
			event: Event name to fire when namespace[proxy] is modified.
			ready: Event name to fire when namespace[proxy] is available for use.
			direct: Table or name of table to use as for event-accessible session variables. If lookup and direct are omitted, direct is assumed and set as {}
			localonly: When set, don't apply as a global event (see below)
		Events passed through table format or by string will be registered to modules automatically.

		Events created via table format are automatically registered as global (API-wide) events, allowing child modules from any module to watch for variable changes.

	ManaCore does register its own core module, look here first for an example.
]]

ns.RegisterCoreModule = function (self,namespace,name,...)
	if type(name) ~= 'string' then return debug'Core module registration FAILED, no name provided!' end
	if type(namespace) ~= 'table' then return debug'Core module registration FAILED, no namespace provided!' end
	-- Set the core functions as an index metatable, makes life easier and allows selective override without breaking things.
	local m
	if getmetatable(namespace) then
		m = namespace
		for k,v in pairs(CoreFunctions) do
			m[k] = m[k] or v
		end
	else
		m = setmetatable(namespace,{__index = CoreFunctions})
	end
	
	m.CoreName = name
	m.DBEvents = {}
	m.DBProxies = {}
	
	m.EventHandler = EventHandler
	m.EventFrame = CreateFrame('Frame',nil,UIParent)
	m.EventFrame:SetScript('OnEvent',m.EventHandler)
	m.EventFrame.parent = m
	
	m.RegisteredModules = {}
	m.ModuleEvents = {}
	m.GlobalEvents = m.GlobalEvents or {}
	m.RegisteredEvents = {}

	ns.RegisteredCores = ns.RegisteredCores or {}
	ns.RegisteredCores[name] = m
	if not(_G[name]) then _G[name] = m; end
	
	for i = 1,select('#',...) do
		local v = select(i,...)
		local t = type(v)
		if t == 'string' then
			m.DBEvents[v] = true
			self:debug(name,v,'registered as DB event')
		elseif t == 'table' then
			if (v.proxy and v.event and v.ready) then
				if not(v.lookup or v.direct) then
					v.direct = {}
				elseif type(v.direct) == 'string' then
					m[v.direct] = (type(m[v.direct]) == 'table') and m[v.direct] or {}
					v.direct = m[v.direct]
				end
				if not(v.localonly) then
					m.GlobalEvents[v.event] = true
					m.GlobalEvents[v.ready] = true
				end
				m[v.proxy] = {_LOOKUP = v.lookup, _DIRECT = v.direct, _EVENT = v.event, _READY = v.ready, _PARENT = name, _PROXY = v.proxy, _LOCALONLY = v.localonly, _DEFAULT = v.default}
				m.DBEvents[v.event] = m[v.proxy]
				m.DBEvents[v.ready] = m[v.proxy]
				table.insert(m.DBProxies,m[v.proxy])
				self:debug('proxy table '..name..'.'..tostring(v.lookup or v.direct)..' registered, '..(v.direct and ('available immediately') or ('will be ready at ADDON_LOADED('..name..')')))
			else self:debug('invalid table passed to RegisterCoreModule supplemental arg#'..i,ptable(v)); end
		else
			self:debug('invalid supplemental arg#'..i..' passed to RegisterCoreModule, skipping')
		end
	end

	ns:FireEvent('CORE_REGISTERED',name)

	for i,t in pairs(m.DBProxies) do
		if t._DIRECT then
			m:InitDB(t)
		end
	end

	return m
end

ns.GetCore = function (self,core)
	local core,self = core,self
	if not(core) and type(self) == 'string' then
		core = self
		self = ns
	end
	return self.RegisteredCores[core]
end

-- Register the ManaCore core module
ns.GlobalEvents = {
	CORE_REGISTERED = true,
	CORE_VARIABLES_READY = true,
}
ns:RegisterCoreModule(ns,AddonName,{proxy='db', lookup='ManaCoreCFG', event='MANACORE_CFG_CHANGED_GLOBAL', ready = 'MANACORE_CFG_READY_GLOBAL'},{proxy='dbpc', lookup='ManaCoreCFGPC', event='MANACORE_CFG_CHANGED', ready = 'MANACORE_CFG_READY'},{proxy='vars', direct='Variables', event='MANACORE_VARS_CHANGED', ready='MANACORE_VARS_READY'})
local API,i = ns:RegisterModule(nil,'API','ADDON_LOADED')

local PositionDebugFrame = function ()
	DebugFrame:SetWidth(ns.db.DebugFrameWidth or 500)
	DebugFrame:SetHeight(ns.db.DebugFrameHeight or 200)
	local t = ns.db.DebugFramePosition or {}
	local point, relativeTo, relativePoint, xOffset, yOffset = unpack(t)
	DebugFrame:SetPoint(point or 'CENTER', relativeTo or UIParent, relativePoint or 'CENTER', xOffset, yOffset)
	
	if ns.db.DebugFrameShow == true then
		DebugFrame:Show()
	else
		DebugFrame:Hide()
	end
end

do -- Quickie child module to load savedvariables for each core at ADDON_LOADED (should handle LOD cores without issue)
	API.LODCores = {}
	API.LODCoresLoaded = {}
	API.ADDON_LOADED = function (self,addon)
		local m = ns.RegisteredCores[addon]
		if m then
			m:InitDB()
			if self.LODCores[addon] then
				self.LODCoresLoaded[addon] = table.remove(self.LODCores[addon])
			end
		end
	end

	API.LoadReadyModules = function (self)
		local event = ns.EventFrame.LastEvent
		for core,events in pairs(self.LODCores) do
			if events[event] and (IsAddOnLoadOnDemand(core) and not(IsAddOnLoaded(core))) then
				self:mdebug(core..' load triggered',LoadAddOn(core)) -- LoadAddOn pcalls the addon and then triggers ADDON_LOADED, which should do the rest.
			end
		end
	end

	API.GetCharInfo = function (self)
		local realm = GetRealmName()
		local name = UnitName('player')
		ns.db.perRealm = ns.db.perRealm or {}
		ns.db.perRealm[realm] = ns.db.perRealm[realm] or {}
		ns.dbs = ns.db.perRealm[realm]

		ns.dbs[name] = ns.dbs[name] or {}
		ns.dbc = ns.dbs[name]
		ns.dbs[name].class = select(2,UnitClass('player'))
		ns.dbs[name].race = select(2,UnitRace('player'))

		return ns.dbc
	end
	
	API.MANACORE_CFG_READY_GLOBAL = function (self)
		API:LoadReadyModules() -- Just in case.
		API:GetCharInfo()
		PositionDebugFrame()
		SLASH_MANACOREDEBUG1 = '/mcdebug'
		SlashCmdList['MANACOREDEBUG'] = function (self) ns.db.DebugFrameShow = not(ns.db.DebugFrameShow) end
	end

	API.MANACORE_CFG_CHANGED_GLOBAL = function (self,var)
		if var == 'DebugFrameShow' then
			PositionDebugFrame()
		end
	end
end

do -- Get available addons and toss their LOD events into API.LODCores for later processing via a single-purpose slave module
	for i = 1,GetNumAddOns() do
		local data = GetAddOnMetadata(i,'X-ManaCore')
		if data then
			API.LODCores[GetAddOnInfo(i)] = split(data,'%s+') -- format: { EVENT_NAME = true, EVENT_NAME_2 = true, ...}. Allows easy indexing without screwing with table index
		end
	end
	local m,i = ns:RegisterModule(nil,'APISlave') -- Register a slave to handle events so we don't overload anything on accident
	for core,events in pairs(API.LODCores) do
		for event,_ in pairs(events) do
			m[event] = function () return API.LoadReadyModules(API) end -- Run it through a wrapper so we can still call LoadReadyModules without changing anything
			m.Events[event] = true
		end
	end
end

do -- DebugFrame settings and such
	DebugFrame:Hide()
	DebugFrame:SetFading(false)
	DebugFrame:SetFontObject('SystemFont_Shadow_Small')
	DebugFrame:SetJustifyH('LEFT')
	DebugFrame:EnableMouse(true)
	DebugFrame:EnableMouseWheel(true)
	DebugFrame:SetScript('OnMouseWheel',function (self,delta)
		local scroll = delta
		if IsModifierKeyDown() then
			scroll = delta * 10
		end
		local cur = self:GetCurrentScroll()
		if cur + scroll > self:GetCurrentLine() then
			self:ScrollToTop()
		elseif cur + scroll < 0 then
			self:ScrollToBottom()
		else
			self:SetScrollOffset(cur+scroll)
		end
		self.scroll:SetValue(self:GetCurrentScroll())
	end)

	DebugFrame:SetResizable(true)
	DebugFrame:SetMovable(true)
	DebugFrame:RegisterForDrag('LeftButton')
	DebugFrame:SetScript('OnDragStart', function (self) self:StartMoving() end)
	DebugFrame:SetScript('OnDragStop', function (self)
		local point, relativeTo, relativePoint, xOffset, yOffset = DebugFrame:GetPoint()
		ns.db.DebugFramePosition = {point, relativeTo, relativePoint, xOffset, yOffset}
		self:StopMovingOrSizing()
	end)

	DebugFrame.bg = DebugFrame:CreateTexture(nil,'BACKGROUND')
	DebugFrame.bg:SetTexture(0,0,0,0.5)
	DebugFrame.bg:SetAllPoints(DebugFrame)

	DebugFrame.title = DebugFrame:CreateFontString(nil,'BACKGROUND')
	DebugFrame.title:SetFontObject'GameFontNormal'
	DebugFrame.title:SetText('ManaCore Debug')
	DebugFrame.title:SetPoint('BOTTOMLEFT',DebugFrame,'TOPLEFT')

	DebugFrame.scroll.tex = DebugFrame.scroll:CreateTexture(nil,'ARTWORK')
	DebugFrame.scroll.tex:SetTexture(1,1,1,0.5)
	DebugFrame.scroll:SetStatusBarTexture(DebugFrame.scroll.tex)
	DebugFrame.scroll:SetPoint('TOPRIGHT',DebugFrame)
	DebugFrame.scroll:SetPoint('BOTTOM')
	DebugFrame.scroll:SetWidth(5)
	DebugFrame.scroll:SetOrientation('VERTICAL')

	DebugFrame.resize = CreateFrame('Button',nil,DebugFrame)
	DebugFrame.resize:SetWidth(10)
	DebugFrame.resize:SetHeight(5)
	DebugFrame.resize:SetPoint('BOTTOMRIGHT',DebugFrame,'BOTTOMRIGHT')
	DebugFrame.resize.tex = DebugFrame.resize:CreateTexture(nil,'ARTWORK')
	DebugFrame.resize.tex:SetTexture(1,1,1)
	DebugFrame.resize.tex:SetVertexColor(1,1,1,0.15)
	DebugFrame.resize.tex:SetAllPoints(DebugFrame.resize)
	DebugFrame.resize:EnableMouse(true)
	DebugFrame.resize:RegisterForDrag('LeftButton')
	DebugFrame.resize:SetScript('OnEnter', function (self) self.tex:SetVertexColor(1,1,1,0.75) end)
	DebugFrame.resize:SetScript('OnLeave', function (self) self.tex:SetVertexColor(1,1,1,0.15) end)
	DebugFrame.resize:SetScript('OnDragStart', function (self)
		DebugFrame:StartSizing()
	end)
	DebugFrame.resize:SetScript('OnDragStop', function (self)
		DebugFrame:StopMovingOrSizing()
		ns.db.DebugFrameWidth, ns.db.DebugFrameHeight = DebugFrame:GetSize()
	end)
end
