﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"RingUI")
local slave = ns:RegisterModule()
local tg = Guide

m.pending = {}
m.Buttons = {}

m.Button = CreateFrame('Button','ManaCoreButton',UIParent)
m.Button.tex_off = m.Button:CreateTexture(nil,'ARTWORK',5)
m.Button.tex_on = m.Button:CreateTexture(nil,'ARTWORK',5)

m.RingContainer = CreateFrame('Frame','ManaCoreRingContainer',m.Button)
m.Ring = CreateFrame('Frame','ManaCoreRing',m.RingContainer)
m.Ring:SetFrameStrata('LOW')
m.Ring.tex = m.Ring:CreateTexture(nil,'BACKGROUND')

m.Button:SetMovable(true)
m.Button:RegisterForDrag('LeftButton')
m.Button:RegisterForClicks('AnyUp')
m.Button:SetClampedToScreen(true)

m.Button.tex_off:SetTexture('Interface\\AddOns\\ManaCore\\Media\\MC_logo_off')
m.Button.tex_on:SetTexture('Interface\\AddOns\\ManaCore\\Media\\MC_logo_on')
m.Button.tex_on:Hide()

local boosterFlyout = CreateFrame('Frame','BoosterRingFlyout',m.RingContainer)
boosterFlyout:SetHeight(35)
boosterFlyout:SetWidth(150)
boosterFlyout:SetPoint('BOTTOM',m.Ring,'TOP',0,0)
boosterFlyout:Hide()

local tycoonFlyout = CreateFrame('Frame','TycoonRingFlyout',m.RingContainer)
tycoonFlyout:SetHeight(35)
tycoonFlyout:SetWidth(150)
tycoonFlyout:SetPoint('BOTTOM',m.Ring,'TOP',0,0)
tycoonFlyout:Hide()

local positionButton = function (self)    
    if string.match(ns.dbpc.ringLayout,'Ring') and type(self.arc)=='number' then
        local arc = math.abs(self.arc)
        --get angle in rad
        local angle = math.rad(arc % 360)
        --get offset
        local sina = math.sin(angle)
        local cosa = math.cos(angle)
        local x = cosa*m.Ring:GetWidth()/3
        local y = sina*m.Ring:GetHeight()/3
        self:ClearAllPoints()
        self:SetPoint('CENTER', m.Ring, 'CENTER', ns.dbpc.ringLayout == 'Graphic Ring' and (x + 2) or x, y)
    elseif self.baranchor then
        self:ClearAllPoints()
        self:SetPoint(unpack(self.baranchor))
    end
end 

local sbbg = CreateFrame('Frame',nil,m.RingContainer)
local positionAllButtons = function ()
    sbbg:Hide()
    for i,button in pairs(m.Buttons) do positionButton(button) end
end

ns.Layouts = {'Graphic Ring','Minimal Ring','Graphic Bar','Simple Bar','Minimal Bar'}
local ringLayoutFuncs
do
    ringLayoutFuncs = {
        ['Graphic Ring'] = function ()
            m.Button:SetSize(32,32)
            m.Ring:SetSize(128,128)
            m.Button.tex_on:SetSize(64,128)
            m.Button.tex_off:SetSize(64,128)
            
            m.Button.tex_on:ClearAllPoints()
            m.Button.tex_on:SetPoint('BOTTOM',m.Button,'BOTTOM',0,-4)
            m.Button.tex_off:SetAllPoints(m.Button.tex_on)

            m.Ring:ClearAllPoints()
            m.Ring:SetPoint('CENTER',m.Button,'CENTER',-3,0)
            m.Ring.tex:ClearAllPoints()
            m.Ring.tex:SetAllPoints(m.Ring)
            m.Ring.tex:SetTexture('Interface\\AddOns\\ManaCore\\Media\\MC_ring')
            m.Ring.tex:SetAlpha(0.9)

            boosterFlyout:SetPoint('BOTTOM',m.Ring,'TOP',0,0)
            
            positionAllButtons()
        end,

        ['Minimal Ring'] = function () -- Ring layout with no ring graphic
            m.Button:SetSize(22,22)
            m.Ring:SetSize(96,96)
            m.Button.tex_on:SetSize(38,76)
            m.Button.tex_off:SetSize(38,76)
            
            m.Button.tex_on:ClearAllPoints()
            m.Button.tex_on:SetPoint('BOTTOM',m.Button,'BOTTOM',0,0)
            m.Button.tex_off:SetAllPoints(m.Button.tex_on)

            m.Ring:ClearAllPoints()
            m.Ring:SetPoint('CENTER',m.Button,'CENTER',-1,8)
            m.Ring.tex:ClearAllPoints()
            m.Ring.tex:SetTexture()

            boosterFlyout:SetPoint('BOTTOM',m.Ring,'TOP',0,-25)
            
            positionAllButtons()
        end,

        ['Graphic Bar'] = function () -- Bar layout with bg graphic
            m.Button:SetSize(22,22)
            m.Ring:SetSize(160,20)
            m.Button.tex_on:SetSize(38,76)
            m.Button.tex_off:SetSize(38,76)
            
            m.Button.tex_on:ClearAllPoints()
            m.Button.tex_on:SetPoint('BOTTOM',m.Button,'BOTTOM',0,0)
            m.Button.tex_off:SetAllPoints(m.Button.tex_on)

            m.Ring:ClearAllPoints()
            m.Ring:SetPoint('BOTTOM',m.Button,'BOTTOM',0,0)
            m.Ring.tex:SetAllPoints(m.Ring)
            m.Ring.tex:SetTexture('Interface\\AddOns\\ManaCore\\Media\\MC_attachment_bar')
            m.Ring.tex:SetAlpha(0.8)

            boosterFlyout:SetPoint('BOTTOM',m.Ring,'TOP',0,25)

            positionAllButtons()
        end,

        ['Simple Bar'] = function () -- Bar layout with simple Lua graphics (matches steplist)
            ringLayoutFuncs['Graphic Bar']()
            if not(sbbg.bg) then
                sbbg:SetFrameStrata('LOW')
                sbbg.bg = sbbg:CreateTexture(nil,'BACKGROUND')
                sbbg.bg:SetAllPoints(sbbg)
                sbbg.bg:SetTexture(0,0,0,0.5)
                sbbg.bg.borders = {}
                local b = sbbg.bg.borders
                for i = 1,4 do
                    b[i] = sbbg:CreateTexture(nil,'BORDER')
                    b[i]:SetTexture(1,1,1,0.3)
                    b[i]:SetBlendMode('ADD')
                end
                b[1]:SetPoint('BOTTOMLEFT',sbbg.bg,'TOPLEFT',-2,0)
                b[1]:SetPoint('BOTTOMRIGHT',sbbg.bg,'TOPRIGHT',2,0)
                b[1]:SetHeight(1)
                b[2]:SetPoint('TOPLEFT',sbbg.bg,'BOTTOMLEFT',-2,0)
                b[2]:SetPoint('TOPRIGHT',sbbg.bg,'BOTTOMRIGHT',2,0)
                b[2]:SetHeight(1)
                b[3]:SetPoint('TOPLEFT',b[1],'BOTTOMLEFT')
                b[3]:SetPoint('BOTTOM',b[2],'TOP')
                b[3]:SetWidth(1)
                b[4]:SetPoint('TOPRIGHT',b[1],'BOTTOMRIGHT')
                b[4]:SetPoint('BOTTOM',b[2],'TOP')
                b[4]:SetWidth(1)
            end

            m.Ring:SetSize(128,23)
            m.Ring.tex:SetTexture()
            sbbg:SetAllPoints(m.Ring)
            sbbg:Show()
        end,

        ['Minimal Bar'] = function () -- Bar layout with no graphics
            ringLayoutFuncs['Graphic Bar']()
            m.Ring.tex:SetTexture()
        end,
    }
end

local readyByDB = {
    Booster = {
        db = 'BOOSTER_CFG_READY',
        vars = 'BOOSTER_VARS_READY',
    },
    Tycoon = {
        dbpc = 'TYCOON_CFG_READY',
        vars = 'TYCOON_VARS_READY',
        db = 'TYCOON_CFG_READY_GLOBAL',
    },
    ManaCore = {
        dbpc = 'MANACORE_CFG_READY',
        vars = 'MANACORE_VARS_READY',
        db = 'MANACORE_CFG_READY_GLOBAL',
    }
}
local changedByDB = {
    Booster = {
        db = 'BOOSTER_CFG_CHANGED',
        vars = 'BOOSTER_VARS_CHANGED',
    },
    Tycoon = {
        dbpc = 'TYCOON_CFG_CHANGED',
        vars = 'TYCOON_VARS_CHANGED',
        db = 'TYCOON_CFG_CHANGED_GLOBAL',
    },
    ManaCore = {
        dbpc = 'MANACORE_CFG_CHANGED',
        vars = 'MANACORE_VARS_CHANGED',
        db = 'MANACORE_CFG_CHANGED_GLOBAL',
    }
}
-- Utility table, shortcuts to the opposing diagonal of a quadrant
local diags = {
    ['TOPLEFT'] = 'BOTTOMRIGHT',
    ['TOPRIGHT'] = 'BOTTOMLEFT',
    ['BOTTOMLEFT'] = 'TOPRIGHT',
    ['BOTTOMRIGHT'] = 'TOPLEFT',
}

ns.PositionStatusFrame = function (self)
    if ns.dbpc.statusframe_detached then return end
    tg.statusframe:ClearAllPoints()
    tg.statusframe:SetPoint('TOP',ns.vars.ringState and string.match(ns.dbpc.ringLayout,'Ring') and ManaCoreRing or ManaCoreButton,'BOTTOM',0,ns.vars.ringState and string.match(ns.dbpc.ringLayout,'Ring') and 15 or 10)
end

-- Returns the current quadrant and its opposing diagonal, makes sure tooltips don't hit the screen edge
local GetQuadrant = function (self)
    local x,y = self:GetCenter()
    local w,h = UIParent:GetSize()
    local xs = x/w > 0.5 and 'RIGHT' or 'LEFT'
    local ys = y/h > 0.5 and 'TOP' or 'BOTTOM'
    return ys..xs, diags[ys..xs]
end
ns.GetQuadrant = GetQuadrant

local StartMoving = function (self)
    if ns.dbpc.lock_ring then return end
    m.Button:HideTooltip()
    m.Button:StartMoving()
end

local StopMoving = function (self)
    if ns.dbpc.lock_ring then return end
    local point, relativeTo, relativePoint, xOffset, yOffset = m.Button:GetPoint()
	ns.dbpc.FramePosition = {point, relativeTo, relativePoint, xOffset, yOffset}
	m.Button:StopMovingOrSizing()
    if m.Button:IsMouseOver() then m.Button:ShowTooltip() end
end

m.Button:SetScript('OnDragStart',StartMoving)
m.Button:SetScript('OnDragStop',StopMoving)
m.Button:SetScript('OnEnter',function (self)
    self.tex_off:Hide()
    self.tex_on:Show()
    self:ShowTooltip()
end)
m.Button:SetScript('OnLeave',function (self)
    self.tex_off:Show()
    self.tex_on:Hide()
    self:HideTooltip()
end)

m.Button:SetScript('OnClick', function (self,button)
    if button == 'RightButton' then
        for i = 1,#ns.Layouts do
            if ns.dbpc.ringLayout == ns.Layouts[i] then
                --print(ns.Layouts[i+1] and ns.Layouts[i+1] or ns.Layouts[1])
                ns.vars.ringLayout = ns.Layouts[i+1] and ns.Layouts[i+1] or ns.Layouts[1]
                break
            end
        end
    elseif IsControlKeyDown() and button == 'LeftButton' then
			ns.dbpc.lock_ring = not(ns.dbpc.lock_ring)
		else
        ns.vars.ringState = not(ns.vars.ringState)
    end
end)

m.Button.ShowTooltip = function (self)
    local toggle = ns.dbpc.ringState == true and 'Fade the hub out, leaving only the center button.\n\n' or 'Show the hub.\n\n'
		local locktoggle = ns.dbpc.lock_ring == true and 'Unlock the hub, allowing it to be dragged.\n\n' or 'Lock the hub, preventing it from being moved\n\n'
    local text = 'Drag: Move the ManaCore hub.\n\nLeft-click: '..toggle..'Ctrl-Left-click:'..locktoggle..'Right-click: Change the hub\'s layout.\nCurrent layout: |cFFFFFFFF'..ns.dbpc.ringLayout..'|r'
    local vis = (ns.vars.dungeon_mode == true or ns.dbpc.statusframe_detached == true)
    local anchor = vis and (ns.dbpc.ringState == true and m.Ring or m.Button) or Guide.statusframe

    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint('TOP',anchor,'BOTTOM',0,-5)
    GameTooltip:SetText(text, nil, nil, nil, nil, true)
    GameTooltip:Show()
end

m.Button.HideTooltip = function (self)
    GameTooltip:Hide()
end


local PositionManaCoreButton = function ()
	local t = ns.dbpc.FramePosition or {}
	local point, relativeTo, relativePoint, xOffset, yOffset = unpack(t)
	m.Button:SetPoint(point or 'CENTER', UIParent, relativePoint or 'CENTER', xOffset, yOffset)
end

local step = 0
local RingOnUpdate,SetRingState,SetRingLayout,OnLayoutChange
do  -- So we can call RingOnUpdate within itself
    RingOnUpdate = function (self,elapsed)
        self.TSLU = self.TSLU + elapsed
        self.alpha = nil
        while self.TSLU >= 0.05 do
            self.alpha = self:GetAlpha() + step
            self.TSLU = self.TSLU - 0.05
            self.alpha = self.alpha < 0 and 0 or self.alpha > 1 and 1 or self.alpha -- clamp alpha to 0-1
            self:SetAlpha(self.alpha)
        end
        if self.alpha == 1 then
            m:mdebug('SetRingState: OnUpdate complete, clearing')
            self:SetScript('OnUpdate',nil)
        elseif self.alpha == 0 then
            m:mdebug('SetRingState: OnUpdate complete, clearing and hiding')
            self:SetScript('OnUpdate',nil)
            ns:PositionStatusFrame()
            self:Hide()
            if self.newlayout then
                ringLayoutFuncs[ns.dbpc.ringLayout]()
                self.newlayout = nil
                ns.vars.ringState = true
            end
        end
    end

    SetRingState = function (state,init)
        if init then
            if state then m.RingContainer:Show() else m.RingContainer:Hide() end
            return
        end
        
        if state ~= m.RingContainer.state then
            m:mdebug('SetRingState',state,m.RingContainer.state)
            m.RingContainer:Show()
            step = state and 0.1 or -0.1
            m.RingContainer.state = state
            m.RingContainer.TSLU = 0
            ns:PositionStatusFrame()
            m.RingContainer:SetScript('OnUpdate', RingOnUpdate)
        end
    end

    SetRingLayout = function (layout)
        m.RingContainer.newlayout = layout
        if ns.vars.ringState == true then   -- Trigger a hide, the onupdate does the rest
            ns.vars.ringState = false
        else
            m.RingContainer.state = true
            SetRingState(false)
        end
    end
end

local getCoreInfo = function (self)
    if IsAddOnLoaded(self.addon) then
        self.tex_on:Show()
        self.tex_off:Hide()
        self.tex_on:SetAlpha(1)
    elseif GetAddOnMetadata(self.addon,'Title') then
        self.tex_off:Show()
        self.tex_on:Hide()
        self.tex_off:SetAlpha(1)
    else
        self.tex_on:Hide()
        self.tex_off:Show()
        self.tex_off:SetAlpha(0.5)
    end
end

local createButton = function (button,onTex,offTex,texSize,arc,customBG,customBGA)
	button.arc = arc
	--if type(arc)=='number' then positionButton(button) else button:SetPoint(unpack(arc)) end
	button:SetSize(22,22)
	button:RegisterForClicks('AnyDown','AnyUp')
	
	button.bg = button:CreateTexture(nil,'BORDER',-4)
	button.bg:SetTexture(customBG or 'Interface\\AddOns\\ManaCore\\Media\\MC_button_back')
	button.bg:SetSize(22,22)
	button.bg:SetPoint('CENTER',button,'CENTER')
	button.bg:SetAlpha(customBGA or 0.6)
	
	button.tex_on = button:CreateTexture(nil,'ARTWORK')
	button.tex_on:SetTexture(onTex)
	button.tex_on:SetPoint('CENTER',button,'CENTER')
	button.tex_on:SetSize(texSize or 22, texSize or 22)
	button.tex_on:SetVertexColor(1,1,1,1)
	button.tex_on:Hide()
	
	button.tex_off = button:CreateTexture(nil,'ARTWORK')
	button.tex_off:SetTexture(offTex or onTex..'_desat')
	button.tex_off:SetPoint('CENTER',button,'CENTER')
	button.tex_off:SetSize(texSize or 22, texSize or 22)
	button.tex_off:SetVertexColor(0.8,0.8,0.8,0.8)
	button.tex_off:Hide()
end

local onEnter = function (self)
	self.active_tex:SetVertexColor(1,1,1,1)
	if self.ShowTooltip then self:ShowTooltip() end
end

local onLeave = function (self)
	if not self.lock_alpha then
		self.active_tex:SetVertexColor(0.8, 0.8, 0.8, 0.8)
	end
	GameTooltip:Hide()
end

local nl = '\n\n'
local ShowOptionTooltip = function(self)
	self.tipTextAll = self.tipHeaderPrefix..self.tipHeader..nl..self.tipBodyPrefix..self.tipBody..nl
	if self.proxy then
			self.tipTextState = ('|cFFFFFFFFCurrently: '..(self.proxy[self.var] == true and (self.tipTextOnPrefix..self.tipTextOn) or (self.tipTextOffPrefix..self.tipTextOff)))
	end
	GameTooltip:SetOwner(self, "ANCHOR_NONE")
	
	local a,b = GetQuadrant(self)
	GameTooltip:SetPoint(a,self,b)
	GameTooltip:SetText(self.tipTextAll..(self.disabled and (self.tipTextDisabledPrefix..self.tipTextDisabled) or self.tipTextState), nil, nil, nil, nil, true)
	GameTooltip:Show()
end

local setOptionTooltip = function (self)
	self.tipHeader = self.tipHeader or ""
	self.tipBody = self.tipBody or ""
	self.tipTextDisabled = self.tipTextDisabled or ""
	self.tipTextOn = self.tipTextOn or 'On'
	self.tipTextOff = self.tipTextOff or 'Off'
	
	self.tipHeaderPrefix = string.match(self.tipHeader,'^|') and "" or '|cFF6666FF'
	self.tipBodyPrefix = string.match(self.tipBody,'^|') and "" or '|cFFFFFFFF'
	self.tipTextDisabledPrefix = string.match(self.tipTextDisabled,'^|') and "" or '|cFFFF0000'
	self.tipTextOnPrefix = string.match(self.tipTextOn,'^|') and "" or '|cFF00FF00'
	self.tipTextOffPrefix = string.match(self.tipTextOff,'^|') and "" or '|cFFFF0000'
	
	self:EnableMouse(true)
	self.ShowTooltip = ShowOptionTooltip
end
m.setOptionTooltip = setOptionTooltip

local ShowProductTooltip = function(self)
    self.tipTextAll = self.tipHeaderPrefix..self.tipHeader..nl..self.tipBodyPrefix..self.tipBody..nl
    GameTooltip:SetOwner(self, "ANCHOR_NONE")

    local a,b = GetQuadrant(self)
    GameTooltip:SetPoint(a,self,b)
    GameTooltip:SetText(self.tipTextAll..(self.disabled and (self.tipTextDisabledPrefix..self.tipTextDisabled) or ""), nil, nil, nil, nil, true)
    GameTooltip:Show()
end

local setProductTooltip = function (self)
    self.tipHeader = self.tipHeader or ""
    self.tipBody = self.tipBody or ""
    self.tipTextDisabled = self.tipTextDisabled or ""
    
    self.tipHeaderPrefix = string.match(self.tipHeader,'^|') and "" or '|cFF6666FF'
    self.tipBodyPrefix = string.match(self.tipBody,'^|') and "" or '|cFFFFFFFF'
    self.tipTextDisabledPrefix = string.match(self.tipTextDisabled,'^|') and "" or '|cFFFF0000'
    
    self:EnableMouse(true)
    self.ShowTooltip = ShowProductTooltip
end
m.setProductTooltip = setProductTooltip


local onVarChanged = function (self,var)
    local f = m[var]
    if f then
        --print('oVC',var)
        f.active_tex = f.active_tex or f.tex_off
        f.active_tex:SetAlpha(1)
        f.active_tex:Hide()
        if f.proxy then
            f.active_tex = f.proxy[var] and f.tex_on or f.tex_off
        end
        f.active_tex:Show()
        f.active_tex:SetAlpha(0.8)

        if f:IsMouseOver() then
            GameTooltip:Hide()
            f:onEnter()
        end
    end
end
m.onVarChanged = onVarChanged

local initVar = function (var, core, db)
    local f = m[var]
    local c = ns:GetCore(core)
    f.m = c:RegisterModule() -- Register a child module under the intended core to catch events with minimal hassle
    f.m:RegisterEvent(readyByDB[core][db],changedByDB[core][db])
    
    f.m[ changedByDB[core][db] ] = function (self,etc) return onVarChanged(m,etc) end -- Catch events, pass them to onVarChanged. oVC checks the args for us, no harm in not handling here
    f.m[ readyByDB[core][db] ] = function (self) -- Same here, catch ready events and use oVC to init.
        f.disabled = nil
        f.proxy = c[db]
        onVarChanged(m,var)
        f.m[ readyByDB[core][db] ] = nil
    end
    
    if c[db]._INITIALIZED then
        f.m[readyByDB[core][db]]()
    end
end

local onClick = function (f,button,down)
    if down then
        f.bg:SetVertexColor(0,0,0)
        f:SetScript('OnUpdate', function (self,elapsed)
            local imo = self:IsMouseOver()
            if not(self.darkened) and imo then
                self.darkened = true
                self.bg:SetVertexColor(0,0,0)
            elseif self.darkened and not(imo) then
                self.darkened = nil
                self.bg:SetVertexColor(1,1,1)
            end
        end)
    else
        f:SetScript('OnUpdate',nil)
        f.bg:SetVertexColor(1,1,1)
        f.lastclick = button
        if f.OnClick then f:OnClick() end
    end
end

local varOnClick = function (self)
    self.proxy[self.var] = not(self.proxy[self.var])
end

local createToggle = function (var, core, db, size, arc, baranchor, customclick, onTex, offTex, tipHeader, tipBody, tipTextOn, tipTextOff, tipTextDisabled, customBG, customBGA, customParent)
    local f = CreateFrame("Button", nil, customParent or m.RingContainer)
    m[var] = f

    f.baranchor = baranchor or arc
    createButton(f,onTex,offTex,size,arc,customBG,customBGA)

    f.var = var
    f.onTex = onTex
    f.offTex = offTex
    
    f:SetWidth(size or 22)
    f:SetHeight(size or 22)

    f.active_tex = f.tex_off
    f.active_tex:SetAlpha(0.5)
    f.active_tex:Show()
    
    f.disabled = true
    do
        local c = ns:GetCore(core)
        if c then
            initVar(var,core,db)
        else
            m.pending[core] = m.pending[core] or {}
            table.insert(m.pending[core], function () return initVar(var,core,db) end)
        end
    end

    f.OnClick = customclick and customclick or varOnClick

    f.onEnter = onEnter
    f.onLeave = onLeave
    f:SetScript('OnEnter',onEnter)
    f:SetScript('OnLeave',onLeave)
    f:SetScript('OnClick',onClick)
    
    f.tipHeader = tipHeader
    f.tipBody = tipBody
    f.tipTextOn = tipTextOn
    f.tipTextOff = tipTextOff
    f.tipTextDisabled = tipTextDisabled or (core..' must be loaded for this option to become available.')
    setOptionTooltip(f)

    table.insert(m.Buttons,f)
    return f
end
m.createToggle = createToggle

m.EdgeButton = CreateFrame('Button','ManaCoreEdgeButton',m.RingContainer)
do
	createButton(m.EdgeButton,'Interface\\AddOns\\ManaCore\\Media\\MC_logo_edge','Interface\\AddOns\\ManaCore\\Media\\MC_logo_edge_desat',36,245)
	local f = m.EdgeButton
	f.addon = 'Edge'
	f.OnClick = function (self)
		ns.vars.active_product = "Edge"
		if (EdgeAdvisorPanel and EdgeAdvisorPanel:IsShown()) then
			HideUIPanel(EdgeAdvisorPanel)
		elseif EdgeAdvisorPanel then
			ShowUIPanel(EdgeAdvisorPanel)
		end
	end
	f.tipHeader = 'Edge Advisor'
	f.tipBody = "Opens the Edge Advisor Panel."
	f.var = 'show_flyout_edge'
	f.baranchor = {'RIGHT',m.Button,'LEFT', -3,0}
	table.insert(m.Buttons,f)
end

--var, core, db, size, arc, customclick, onTex, offTex, tipHeader, tipBody, tipTextOn, tipTextOff, tipTextDisabled, customBG, customBGA, customParent
--[[
createToggle('lock_ring', 'ManaCore', 'dbpc', nil, 200, {'RIGHT',m.EdgeButton,'LEFT',-3,0}, nil,
	'Interface\\AddOns\\ManaCore\\Media\\locked', 'Interface\\AddOns\\ManaCore\\Media\\unlocked',
	'Ring Lock','Lock the ManaCore ring to its current position.',
    'Locked','Unlocked')
--]]
m.ImpulseButton = CreateFrame('Button','ManaCoreImpulseButton',m.RingContainer)
do
	createButton(m.ImpulseButton,'Interface\\AddOns\\ManaCore\\Media\\MC_logo_impulse','Interface\\AddOns\\ManaCore\\Media\\MC_logo_impulse_desat',36,200)
	local f = m.ImpulseButton
	f.addon = 'Impulse'
	f.OnClick = function (self)
		ns.vars.active_product = "Impulse"
		if (ImpulsePanel and ImpulsePanel:IsShown()) then
			HideUIPanel(ImpulsePanel)
		elseif ImpulsePanel then
			ShowUIPanel(ImpulsePanel)
		end
	end
	f.tipHeader = 'Impulse'
	f.tipBody = "Opens the Impulse keybinds and macros panel."
	f.var = 'show_flyout_impulse'
	f.baranchor = {'RIGHT',m.EdgeButton,'LEFT',-3,0}
	table.insert(m.Buttons,f)
end
	
m.BoosterButton = CreateFrame('Button','ManaCoreBoosterButton',m.RingContainer)
do
	createButton(m.BoosterButton,'Interface\\AddOns\\ManaCore\\Media\\MC_logo_booster','Interface\\AddOns\\ManaCore\\Media\\MC_logo_booster_desat',16,340)
	local f = m.BoosterButton
	f.addon = 'Booster'
	f.OnClick = function (self)
		--activate booster if it is not active
		--NOTE: right now this is handled by Tycoon ending dungeon mode
		--setting on Booster. This should be changed to be handled properly
		--when the booster interface restoration occurs.
		if ns.vars.active_product ~= "Booster" then
			ns.vars.active_product = "Booster"
			Guide:PositionStatusFrame() --Frame isn't appearing on first selection without this
		else
			--show flyout if booster is active
			ns.vars.show_flyout_booster = not(ns.vars.show_flyout_booster)
		end
	end
	f.tipHeader = 'Activate Booster'
	f.tipBody = 'Activates the Booster leveling guide. Click again to show extra Booster options.'
	f.var = 'show_flyout_booster'
	f.baranchor = {'LEFT',m.Button,'RIGHT',3,0}
	table.insert(m.Buttons,f)
end

m.TycoonButton = CreateFrame('Button','ManaCoreTycoonButton',m.RingContainer)
do
	createButton(m.TycoonButton,'Interface\\AddOns\\ManaCore\\Media\\MC_logo_ty','Interface\\AddOns\\ManaCore\\Media\\MC_logo_ty_desat',20,295)
	local f = m.TycoonButton
	f.addon = 'Tycoon'
	f.OnClick = function (self)
		if ns.vars.active_product ~= "Tycoon" then
			ns.vars.active_product = "Tycoon"
		else
			if (TycoonPanel and TycoonPanel:IsShown()) then
				HideUIPanel(TycoonPanel)
			elseif TycoonPanel then
				ShowUIPanel(TycoonPanel)
			end
		end
	end
	f.tipHeader = 'Activate Tycoon'
	f.tipBody = "Activates Tycoon, the Mailbox Bustin' Gold Addon. Click again to show the Tycoon Panel."
	f.var = 'show_flyout_tycoon'
	f.baranchor = {'LEFT',m.BoosterButton,'RIGHT',3,0}
	table.insert(m.Buttons,f)
end
	
--[[
createToggle('placeholder_2', 'ManaCore', 'vars', 22, 245, nil,
    'Interface\\AddOns\\ManaCore\\Media\\muleshopper_open','Interface\\AddOns\\ManaCore\\Media\\muleshopper_open_off',
    'Mule Shopper','Toggles the Mule Shopper window.\n\nMule Shopper allows you to look at the gear of your other characters without needing to log out.',
    'Shown','Hidden')
]]

--var, core, db, size, arc, customclick, onTex, offTex, tipHeader, tipBody, tipTextOn, tipTextOff, tipTextDisabled, customBG, customBGA, customParent
--[[createToggle('show_objectiveframe', 'ManaCore', 'vars', nil, {'BOTTOM',boosterFlyout,'BOTTOM'}, nil, nil,
    'Interface\\AddOns\\ManaCore\\Media\\MC_maximize','Interface\\AddOns\\ManaCore\\Media\\MC_maximize_desat',
    'Step List','Toggles the step list, allowing you to see what\'s coming up or manually complete/undo steps.',
    'Shown','Hidden',nil,nil,nil,boosterFlyout)
--]]
createToggle('show_comments', 'ManaCore', 'dbpc', nil, {'BOTTOM', boosterFlyout, 'BOTTOM'}, nil, nil,
	'Interface\\AddOns\\ManaCore\\Media\\comments_on', 'Interface\\AddOns\\ManaCore\\Media\\comments_off',
	'Wowhead Comments', 'Automatically show Wowhead comments for each quest step, without needing to press the \'Show comments\' button.',
    nil,nil,nil,nil,nil,boosterFlyout)		
--[[createToggle('talentsAuto', 'Booster', 'db', nil, {'LEFT',m.show_comments,'RIGHT'}, nil, nil,
	'Interface\\AddOns\\ManaCore\\Media\\talents_on','Interface\\AddOns\\ManaCore\\Media\\talents_off',
	'Auto Assign Talent Points','Automatically spend talent points for the spec you have chosen each level.\n\n|cFFFF0000WARNING: This may result in respec fees if you wish to use a spec that is different from what Booster recommends.',
    nil,nil,nil,nil,nil,boosterFlyout)
--]]
createToggle('show_configpanel', 'ManaCore', 'vars', nil, {'LEFT',m.show_comments,'RIGHT'}, nil, nil,
	'Interface\\AddOns\\ManaCore\\Media\\config', 'Interface\\AddOns\\ManaCore\\Media\\config_desat',
	'Show More Options','Open the Interface Options window to adjust other settings.',
    'Open','Closed',nil,nil,nil,boosterFlyout)
createToggle('lock_frame', 'ManaCore', 'dbpc', nil, {'LEFT',m.show_configpanel,'RIGHT'}, nil, nil,
	'Interface\\AddOns\\ManaCore\\Media\\locked', 'Interface\\AddOns\\ManaCore\\Media\\unlocked',
	'Guide Lock','Lock the Guide status text to its current position.',
    'Locked','Unlocked',nil,nil,nil,boosterFlyout)
createToggle('show_muleshop', 'ManaCore', 'vars', nil, {'RIGHT',m.show_comments,'LEFT',-3,0}, nil, nil,
    'Interface\\AddOns\\ManaCore\\Media\\muleshopper_open','Interface\\AddOns\\ManaCore\\Media\\muleshopper_open_off',
    'Mule Shopper','Toggles the Mule Shopper window.\n\nMule Shopper allows you to look at the gear of your other characters without needing to log out.',
    'Shown','Hidden',nil,nil,nil,boosterFlyout)
createToggle('dungeon_mode', 'ManaCore', 'vars', 18, {'RIGHT',m.show_muleshop,'LEFT',-3,0}, nil, nil,
    'Interface\\AddOns\\ManaCore\\Media\\skull','Interface\\AddOns\\ManaCore\\Media\\skull_desat',
    'Dungeon Mode','Turns off the Guide, Ants, and TomTom arrow. This setting is temporary and will be reset on logout.',
    'Active','Disabled',nil,nil,nil,boosterFlyout)

--[[
createToggle('show_tycoon', 'ManaCore', 'vars', nil, {'BOTTOM',tycoonFlyout,'BOTTOM'}, nil, nil,
	'Interface\\AddOns\\ManaCore\\Media\\MC_logo_ty', 'Interface\\AddOns\\ManaCore\\Media\\MC_logo_ty_desat',
	'Show Tycoon','Open the Tycoon interface.',
    'Open','Closed',nil,nil,nil,tycoonFlyout)
--]]
	
local init
m:RegisterEvent('BOOSTER_CFG_READY','TYCOON_CFG_READY','IMPULSE_CFG_READY','EDGE_CFG_READY','GUIDE_FRAME_DRAG_START','GUIDE_FRAME_DRAG_STOP', 'CORE_REGISTERED')

m.CORE_REGISTERED = function (self,core)
    if m.pending[core] then
        for k,v in pairs(m.pending[core]) do v() end
    end
end

local setupButton = function (f,activate)
    local t = activate and f.tex_on or f.tex_off
    if not f.active_tex then f.active_tex = f.tex_off end
    f.active_tex:SetVertexColor(1,1,1,1)
    f.active_tex:Hide()
    f.active_tex = t
    f.active_tex:Show()
    f.active_tex:SetVertexColor(0.8,0.8,0.8,0.8)
    f.proxy = ns.vars
    f.ShowTooltip = ShowProductTooltip
    setProductTooltip(f)

    f.onEnter = onEnter
    f.onLeave = onLeave
    f.onClick = onClick
    f:SetScript('OnEnter',onEnter)
    f:SetScript('OnLeave',onLeave)
    f:SetScript('OnClick',onClick)
		
		if activate then
			f:Enable()
		else
			f:Disable()
		end
end

local isMO, isMNO, isMO2, isMNO2
m.GUIDE_FRAME_DRAG_START = function (self)
    m.Ring:EnableMouse(true)
    m.Ring:SetScript('OnEnter', function (self)
        print('TEST!')
        self.tex:SetVertexColor(0,1,0,0.9)
    end)
    m.Ring:SetScript('OnLeave', function (self)
        self.tex:SetVertexColor(1,1,1,0.9)
    end)
    m.Ring:SetScript('OnUpdate', function (self,elapsed)
        isMO = self:IsMouseOver()
        if isMO and not(isMO2) then
            isMO2 = true
            return self.tex:SetVertexColor(0,1,1,0.9)
        elseif not(isMO) and not(isMNO2) then
            isMNO2 = true
            return self.tex:SetVertexColor(1,1,1,0.9)
        end
        isMO2, isMNO2 = nil,nil
    end)
end

m.GUIDE_FRAME_DRAG_STOP = function (self)
    m.Ring:SetScript('OnEnter',function () end)
    m.Ring:SetScript('OnLeave',function () end)
    m.Ring:EnableMouse(false)
    m.Ring:SetScript('OnUpdate',nil)
    m.Ring.tex:SetVertexColor(1,1,1,0.9)
end

m.BOOSTER_CFG_READY = function (self)
    setupButton(m.BoosterButton,true)
end

m.TYCOON_CFG_READY = function (self)
    setupButton(m.TycoonButton,true)
end

m.EDGE_CFG_READY = function (self)
	setupButton(m.EdgeButton,true)
end

m.IMPULSE_CFG_READY = function (self)
	setupButton(m.ImpulseButton, true)
end

local init = true
m.MANACORE_CFG_READY = function (self)
    PositionManaCoreButton()
    ns.vars.ringState = (ns.dbpc.ringState == nil) or ns.dbpc.ringState
    ns.dbpc.ringLayout = ((ns.dbpc.ringLayout == nil or not(ringLayoutFuncs[ns.dbpc.ringLayout])) and ns.Layouts[1]) or ns.dbpc.ringLayout
    ringLayoutFuncs[ns.dbpc.ringLayout]()

    setupButton(m.BoosterButton)
    setupButton(m.TycoonButton)
    init = nil
		
		ns.vars.dungeon_mode = true; --Start with guide hidden until Booster is activated
end

m.MANACORE_CFG_CHANGED = onVarChanged
m.MANACORE_CFG_CHANGED_GLOBAL = onVarChanged
m.BOOSTER_CFG_CHANGED = onVarChanged

m.MANACORE_VARS_CHANGED = function (self,k,old,v)
    if k == 'ringState' then
        ns.dbpc[k] = ns.vars[k]
        SetRingState(ns.vars[k],init)
        if m.Button:IsMouseOver() then
            m.Button:HideTooltip()
            m.Button:ShowTooltip()
        end
    elseif k == 'ringLayout' then
        ns.dbpc[k] = ns.vars[k]
        SetRingLayout(ns.vars[k])
        if m.Button:IsMouseOver() then
            m.Button:HideTooltip()
            m.Button:ShowTooltip()
        end
    else return onVarChanged(self,k) end
end

slave.clicks = {
	['show_objectiveframe'] = function (v)
		if v then ShowUIPanel(Guide.objectiveframe) else HideUIPanel(Guide.objectiveframe) end
	end,
	['show_configpanel'] = function (v)
		if v then InterfaceOptionsFrame_OpenToCategory(Guide.configpanel) else HideUIPanel(InterfaceOptionsFrame) end
	end,
	['show_tycoon'] = function (v)
		if v then ShowUIPanel(TycoonPanel) else HideUIPanel(TycoonPanel) end
	end,
	['show_flyout_booster'] = function (v)
		if v then boosterFlyout:Show() else boosterFlyout:Hide() end
		GameTooltip:Hide()
		m.BoosterButton:onEnter()
	end,
	['show_flyout_tycoon'] = function (v)
		if v then tycoonFlyout:Show() else tycoonFlyout:Hide() end
		GameTooltip:Hide()
		m.TycoonButton:onEnter()
	end,
	['active_product'] = function (v, o)
		--clean up old products
		if o == "Booster" then
			ns.vars.dungeon_mode = true --Hide booster, bit of a cheat
			ns.vars.show_flyout_booster = false --Hide the booster flyout as well
			m.BoosterButton.lock_alpha = false
			m.BoosterButton.active_tex:SetVertexColor(0.8,0.8,0.8,0.8)
		end
		if o == "Tycoon" then
			m.TycoonButton.lock_alpha = false
			m.TycoonButton.active_tex:SetVertexColor(0.8,0.8,0.8,0.8)
		end
		--activate the new addon
		if v == "Booster" then
			ns.vars.dungeon_mode = false
			m.BoosterButton.lock_alpha = true
		end
		if v == "Tycoon" then
			m.TycoonButton.lock_alpha = true
		end
	end,
}
slave.MANACORE_VARS_CHANGED = function (self,k,old,v)
    return self.clicks[k] and self.clicks[k](v, old)
end

slave.MANACORE_CFG_READY = function (self)
    Guide.objectiveframe:HookScript('OnHide', function ()
        Guide.objectiveframe:HookScript('OnShow', function () ns.vars.show_objectiveframe = true end) -- The panel's OnShow script changes after the first Show call, so we need to rehook it. Done every time for safety.
        ns.vars.show_objectiveframe = false
    end)
		--This allowed for the button to update to enabled, but caused ALL interface panel opens to divert to our addon. Bad behavior.
    --InterfaceOptionsFrame:HookScript('OnShow', function () ns.vars.show_configpanel = true end)
    InterfaceOptionsFrame:HookScript('OnHide', function () ns.vars.show_configpanel = false end)
    Guide.objectiveframe:HookScript('OnShow', function () ns.vars.show_objectiveframe = true end)
end

slave.PLAYER_LOGIN = function(self)
	ns.vars.dungeon_mode = true; --Start with guide hidden until Booster is activated
end
slave:RegisterEvent("PLAYER_LOGIN");