﻿--/dump AucAdvanced.GetModule("Stat", "Simple").GetPriceColumns(itemlink)
--columns: Daily Avg, 3 day avg, 7 day avg, 14 day avg, min BO, avg MBO, false, daily total, daily count, seen days, seen count, mean, StdDev

--Auctioneer.lua
--This file provides a scanner interface for use with Tycoon
--when the Auctioneer addon is present.
if IsAddOnLoaded("Auc-Advanced") then
	local _,ns = ...
	local m,i,dbStatus = ns:RegisterModule(nil,"Auctioneer",'AUCTION_HOUSE_SHOW')
	local faction, adb
	
	local AucSimple = AucAdvanced.GetModule("Stat", "Simple");
	
	--create our own Auctioneer utilities module for private use
	local auclib, aucparent, aucprivate = AucAdvanced.NewModule("Util", "Tycoon")
	
	function auclib:GetName()
		return "Tycoon"
	end
	
	---day functions---
	local dayzero = time({year=2011, month=5, day=30, hour=0}) --do not change
	
	local function getday()
		return (math.floor ((time() - dayzero) / (86400)))
	end
	
	local today = getday()
	
	local round = function (v)
		math.floor (v + 0.5)
	end
	
	-- FromShortDate: Returns numeric shortdate, datestring, and date()-format for a given shortdate in string or number format
	local FromShortDate = function (t)
		local day = (type(t) == 'string' and tonumber(string.sub(t,2))) or t
		local when = dayzero + (day*86400)
		local datestring
		if day == today then
			datestring = 'Today'
		elseif day == today-1 then
			datestring = 'Yesterday'
		else
			datestring = date("%A, %B %d", when)
		end
		return day, datestring, when
	end
	
	-- ToShortDate: Returns numeric shortdate from time()
	local ToShortDate = function (t)
		return (math.floor ((t - dayzero) / (86400)))
	end
	
	--UpdateDay: Forces all day related variables to update after a scan
	--KNOWN BUG: If a scan starts before, but ends after, a day change this
	--will be incorrect. Low occurance race condition. Won't Fix.
	local UpdateDay = function ()
		today = getday()
	end
	
	m.TYCOON_VARIABLES_CHANGED = function (self,k,old,new)
	end
	
	m.TYCOON_VARIABLES_READY = function (self)
	end
	
	--NOTE: NOTHING SHOULD BE DONE WITH ADB AT CFG_READY_GLOBAL
	--API module is not garunteed to be done with it's setup yet
	m.TYCOON_CFG_READY_GLOBAL = function (self)
	end
	
	m.TYCOON_CFG_CHANGED_GLOBAL = function (self,k,old,new)
	end
	
	m.CleanDb = function ()
		local md = 14
		
		m:mdebug('cleaning up db entries')
		local removed = 0
		for day,_ in pairs(adb) do
			local d = tonumber(day) -- just in case.
			if d and (d < (today-md)) then
				removed = removed + 1
				adb[day] = nil;
			end
		end
		m:mdebug(removed..' entries removed')
	end
	
	m.PLAYER_ENTERING_WORLD = function (self)
		faction = UnitFactionGroup("player")
		ns.adb.auctioneer = ns.adb.auctioneer or {}
		ns.adb.auctioneer[faction] = ns.adb.auctioneer[faction] or {}
		adb = ns.adb.auctioneer[faction]
		adb.current = adb.current or {} 
		
		m.CleanDb()
		m.aucthrottle:Show()
	end
	m:RegisterEvent("PLAYER_ENTERING_WORLD")
	
	--[[
	local sessionScan,sessionScanStarted
	m.AUCTION_HOUSE_SHOW = function (self)
		if ((ns.db.autoscan == true) or (ns.dbpc.autoscan == true)) and ns.db.lastscanday ~= today then
			local canQuery,canQueryAll = CanSendAuctionQuery()
			if canQueryAll then
				sessionScanStarted = true
				Atr_ShowFullScanFrame()
				Atr_FullScanStart()
			end
		end
	end
	]]

	--
	
	-- ReloadUI safety calls, should also handle early VARIABLES_READY firing
	--if ns.vars and ns.vars._INITIALIZED then m:TYCOON_VARIABLES_READY() end
	--if ns.vars and ns.vars._INITIALIZED then m:TYCOON_CFG_READY_GLOBAL() end
	
	
	local average = function (t)
		if #t == 0 then return end
		local x = 0
		for i = 1,#t do
			x = x+t[i]
		end
		return #t>0 and (x/#t) or 0
	end
	
	local stddev = function (t)
		if #t == 0 then return end
		local avg = average(t)
		local temp = {}
		for i = 1,#t do
			temp[i] = (t[i]-avg)^2
		end
		local dev = math.sqrt(average(temp))
		local high = avg + (dev * 2) + 10
		local low = avg - (dev * 2) - 10
		return dev, avg, high, low
	end
	
	--[[
		Do-wrapped function declarations so we're dealing with locals everywhere
	]]--
	local outliers
	do
		local temp = {}
		outliers = function (t)
			if #t == 0 then return end
			local dev,avg,high,low = stddev(t)
			table.wipe(temp)
	
			local outlierfound
			for i = 1,#t do
				local outlier = (t[i] > high) or (t[i] < low)
				outlierfound = outlierfound or outlier
				if not(outlier) then table.insert(temp,t[i]) end
			end
	
			if outlierfound then
				t,dev,avg,high,low = outliers(temp)
				return temp,dev,avg,high,low
			end
			return temp,dev,avg,high,low
		end
	end
	
	-- Return a table with no non-number values
	local safetable = function (...)
		local it = type(select(1,...)) == 'table' and select(1,...)
		local t = {}
		if it then
			for k,v in pairs(it) do
				if tonumber(v) then table.insert(t,tonumber(v)) end
			end
		else
			for i = 1,select('#',...) do
				local v = select(i,...)
				if tonumber(v) then table.insert(t,tonumber(v)) end
			end
		end
		return t
	end
	
	---auctioneer scan handlers---
	auclib.ScanProcessors = {}
	local auctempbuyout = {}
	local auctempcount = {}
	local processcount
	
	function auclib.ScanProcessors.begin()
		m:mdebug("Auctioneer processor begin()")
		wipe(auctempbuyout)
		wipe(auctempcount)
		processcount = 0
	end
	
	local function processItem(itemData)
		--DevTools_Dump(itemData)
		local itemId = itemData.itemId
		local buyout = itemData.buyoutPrice
		--m:mdebug("Processing item: ", itemId, buyout)
		processcount = processcount + 1
		if (not buyout) or buyout == 0 then return end
		local buyoutper = ceil(buyout/itemData.stackSize)
		
		if (not auctempbuyout[itemId]) or buyoutper < auctempbuyout[itemId] then
			auctempbuyout[itemId] = buyoutper
		end
		
		auctempcount[itemId] = auctempcount[itemId] and auctempcount[itemId] + 1 or 1
		
		--m:mdebug("Auctioneer item processed", itemId, buyoutper, auctempcount[itemId], auctempbuyout[itemId])
	end
	
	function auclib.ScanProcessors.create(operation, itemData)
		processItem(itemData)
	end
	
	function auclib.ScanProcessors.update(operation, itemData, oldData)
		processItem(itemData)
	end
	
	function auclib.ScanProcessors.leave(operation, itemData)
		processItem(itemData)
	end
	
	function auclib.ScanProcessors.delete(operation, itemData)
		local itemType, itemId, property, factor = AucAdvanced.DecodeLink(itemData.link)
		
		--allows items reduced to 0-count to be updated properly
		auctempcount[itemId] = auctempcount[itemId] or 0
	end
	
	--use throttle to send data update after processor complete phase
	m.aucthrottle = CreateFrame('frame', nil, UIParent)
	m.aucthrottle:Hide()
	m.aucthrottle:SetScript('OnUpdate', function ()
		m.aucthrottle:Hide()
		ns:FireEvent('AUCTIONEER_SCAN_UPDATE')
	end)
	
	function auclib.ScanProcessors.complete()
		m:mdebug("Auctioneer processor complete()", processcount)
		UpdateDay()
		adb[today] = adb[today] or {}
		local day = adb[today]
		local current = adb.current
		
		for k, v in pairs(auctempcount) do
			b = auctempbuyout[k]
			--m:mdebug("Adding auctioneer item", k, v, b)
			if b or day[k] then --skip if no price data, count data is from a previous day disappearing
				day[k] = day[k] or {}
				if b then day[k].price = (not day[k].price) and b or b < day[k].price and b or day[k].price end
				day[k].count = v --currently just stores the last count, needs better logic if we start using daily counts for more
				current[k] = current[k] or {}
				current[k].price = b
				current[k].count = v
			end
		end
		
		--DevTools_Dump(auctempbuyout)
		--DevTools_Dump(auctempcount)
		m.aucthrottle:Show()
		
		wipe(auctempbuyout)
		wipe(auctempcount)
	end
	
	local GetPriceByDay, GetListingsByDay, GetLastListings, GetTrend, GetCompetition, GetItemTradeIndex, PopulateItemInfo
	do
		local temp = {}
		
		GetPriceByDay = function (item,day)
			if day and day ~= today then
				return adb[day] and adb[day][item] and adb[day][item].price
			else
				--local link = select(2,GetItemInfo(item))
				--local price  = select(5, AucSimple.GetPrice(link))
				local price = adb.current[item] and adb.current[item].price
				if price ~= 0 then return price end
			end
		end
	
		GetListingsByDay = function (item,day)
			local day = day or today
			if not (adb[day] and adb[day][item]) then return end
			return adb[day][item].count
		end
		
		GetLastListings = function (item)
			if adb.current[item] and adb.current[item].count ~= 0 then
					return adb.current[item].count
			end
		end
		
		GetAverage = function(item)
			local link = select(2,GetItemInfo(item))
			--local avg = select(4, AucAdvanced.GetModule("Stat", "Simple").GetPrice(link))
			local avg = AucAdvanced.API.GetMarketValue(link)
			if avg ~= 0 then return avg end
		end
	
		--This needs another look
		--currently returns the percentage difference between current and average
		GetTrend = function (item,period) -- Get trend over the past [period] scan days (default 14)
			local avg = GetAverage(item, period)
			local c = GetPriceByDay(item)
			--avg = (avg + c) / 2
			return avg and c and (c-avg)/avg or 0
		end
	
		--NOTE: Out of date
		GetCompetition = function (item,day,byDay)
			local day = tonumber(day) or today
			if not(ns.adb[item] and ns.adb[item][day]) then return end
			local cp,mp,xp,co,mo,xo = GetListingsByDay(item,day)
			local ap = byDay and average(safetable(cp,mp,xp)) or cp
			local ao = byDay and average(safetable(co,mo,xo)) or co
			return (ap * (ao/ap)^2)
		end
	
		--???
		GetItemTradeIndex = function (t)
			return (t.price/10000)*(1/t.competition)*(1+t.trend)
		end
	end
	
	m.IsDBReady = function()
		if adb then
			return true;
		end
	end
	m.GetItemPrice = function (item,daysago)
		daysago = daysago and today - daysago
		return GetPriceByDay(item, daysago);
	end
	m.GetAveragePrice = function(item)
		return GetAverage(item);
	end
	m.GetCurrentListings = function (item)
		return GetListingsByDay(item)
	end
	m.GetItemListings = function (item)
		return GetLastListings(item)
	end
	--Currently returns a percentage compared to average
	m.GetItemTrend = function (item)
		return GetTrend(item)
	end
end--End IsAddonLoaded("Auctionator")
