﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"LootTracker")
local PC = ns:GetModule("PriceCheck")
local deformat = LibStub("LibDeformat-3.0")

local items = {};
local starttime, gph;

--LootTracker:SetItems(items)
--itemtable: a numerically indexed table of tables that
--  include an id index that represents an itemid.
--	ex: { {id=52185} } --Elementium Ore
function m.SetItems(self, itemtable)
	wipe(items)
	for k, v in pairs(itemtable) do
		items[tostring(v.id)] = 0
		m:mdebug("LootTracker Item Added", v.id)
	end
	
	m:mdebug("LootTracker Started")
	
	starttime = nil;
	gph = nil;
end

function m.Reset(self)
	wipe(items)
	
	starttime = nil;
	gph = nil;
end

function m.GetGPH(self)
	return gph
end

----loot event handling----
local function linkToID(link)
	return select(2, strsplit(":", string.match(link, "item[%-?%d:]+")))
end

local function parseMoneyMessage(msg)
	local money = deformat(msg, LOOT_MONEY_SPLIT)
	if money then
		local gold = tonumber(deformat(money:match(GOLD_AMOUNT) or "", GOLD_AMOUNT)) or 0
		local silver = tonumber(deformat(money:match(SILVER_AMOUNT) or "", SILVER_AMOUNT)) or 0
		local copper = tonumber(deformat(money:match(COPPER_AMOUNT) or "", COPPER_AMOUNT)) or 0
		
		m:mdebug("Money Parsed", money, gold, silver, copper)
		
		return copper + silver * 100 + gold * 10000
	end
	money = deformat(msg, YOU_LOOT_MONEY)
	if money then
		local gold = tonumber(deformat(money:match(GOLD_AMOUNT) or "", GOLD_AMOUNT)) or 0
		local silver = tonumber(deformat(money:match(SILVER_AMOUNT) or "", SILVER_AMOUNT)) or 0
		local copper = tonumber(deformat(money:match(COPPER_AMOUNT) or "", COPPER_AMOUNT)) or 0
		
		m:mdebug("Money Parsed", money, gold, silver, copper)
		
		return copper + silver * 100 + gold * 10000
	end
end

local function parseLootMessage(msg)
  local item, quantity = deformat(msg, LOOT_ITEM_SELF_MULTIPLE)
  if item and quantity then
		m:mdebug("Loot Parsed", linkToID(item), quantity)
    return linkToID(item), tonumber(quantity)
  end
  quantity = 1
  item = deformat(msg, LOOT_ITEM_SELF)
  if item then
		m:mdebug("Loot Parsed", linkToID(item), quantity)
    return linkToID(item), tonumber(quantity)
  end
end

local gphlu = 0; --gph last update
local function updateGPH(force)
	if not starttime then return end
	
	local ctime = GetTime()
	
	if ctime - starttime > 1 and (force or ctime - gphlu > 5) then
		gphlu = ctime;
		local total = 0;
		for k, v in pairs(items) do
			if k == "coin" then
				total = total + v * 3600 / (ctime - starttime)
			else
				total = total + (PC.GetPrice(tonumber(k)) or 0) * (v * 3600 / (ctime - starttime))
			end
		end
		gph = total;
		
		ns:FireEvent("TYCOON_LOOTTRACKER_GPH_UPDATE")
	end
end

function m.CHAT_MSG_LOOT(self, msg)
	local id, quantity = parseLootMessage(msg)
	
	if id and quantity and items[id] then 
		items[id] = items[id] + quantity
		starttime = starttime or GetTime()
		m:mdebug("Item Tracked", id, quantity, starttime)
		updateGPH(true)
	else
		m:mdebug("Item Ignored", id, quantity)
	end
end
m:RegisterEvent("CHAT_MSG_LOOT");

function m.CHAT_MSG_MONEY(self, msg)
	local money = parseMoneyMessage(msg)
	
	if money and items["coin"] then
		items["coin"] = items["coin"] + money
		starttime = starttime or GetTime()
		m:mdebug("Money Tracked", money)
		updateGPH(true)
	else
		m:mdebug("Money Ignored", money)
	end
end
m:RegisterEvent("CHAT_MSG_MONEY")

local updater = CreateFrame("Frame", nil, UIParent)
updater:SetScript("OnUpdate", function() updateGPH() end)