﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"Auction")
local PC = ns:GetModule("PriceCheck")
local SL = ns:GetModule("ShoppingList")
local SF = ns:GetModule("StatusFrame")
local slave = ManaCore:RegisterModule();

local items = {}
TYCOON_ITEMS = items --for debug /dump

--List of values to be displayed in filtering drop down
local filterByTypeValues = {
	["Show All"] = true,
}

local function initFilters(...)
    for i=1, select("#", ...) do
        filterByTypeValues[select(i, ...)] = true;
    end
end

--[[
function m.RegisterCraft(Profession, ItemID, CraftID, Reagents, AmountCreated)
	tinsert(crafts, {skill=Profession, item=ItemID, recipie=CraftID, reagents=Reagents, amt=AmountCreated or 1})
	filterBySkillValues[Profession] = true;
end
--make this global for easy access from route register data files
RegisterTycoonCraft = m.RegisterCraft;
]]
local function PopulateItems()
	local table = PC:GetDataTable()
	for k, v in pairs(table) do
		tinsert(items, {item = tonumber(k)})
	end
	initFilters(GetAuctionItemClasses())
end

function m.AddSelected(index)
	SL:SetList("Tycoon Auctions")
	SL:AddToList(items[index].item, items[index].current)
	UIErrorsFrame:AddMessage("Item added to shopping list.", .99999780301005, 0.81960604246706, 0.0, 53, 5)
	ns.vars.active_module = "Auction";
	SF:SetText("Auction: Click for Shopping List")
end

function m.StatusOnClick()
	SL:Show()
end
	

----UI Accessors----
function m.GetNumItems()
	return #items;
end

function m.GetItemName(index)
	if items[index] then return (select(1,GetItemInfo(items[index].item)) or "") end
end

function m.GetItem(index)
	if items[index] then return items[index].item end
end

function m.GetAverage(index)
	if items[index] then return floor(items[index].avg) end
end

function m.GetCurrent(index)
	if items[index] then return items[index].current end
end

function m.GetPercent(index)
	if items[index] then return floor(items[index].percent) or 0 end
end

function m.GetProfit(index)
	if items[index] then return floor(items[index].profit) or 0 end
end

--Calling this function with no index will show the activeRoute tooltip
--(index for active route is not garunteed to remain static)
function m.ShowDailyTooltip(index, daysback, tooltip)
	local item = index and items[index]
	if item then
		local dayprice = PC.GetScannerPrice(item.item, daysback);
		if dayprice then
			tooltip:ClearLines()
			tooltip:AddLine(ns.BuildMoneyString(dayprice))
			tooltip:Show()
		end
	end
end

--Calling this function with no index will show the activeRoute tooltip
--(index for active route is not garunteed to remain static)
function m.ShowItemTooltip(index, tooltip)
	local item = index and items[index]
	if item then
		tooltip:ClearLines()
		tooltip:SetHyperlink("item:"..item.item)
		tooltip:Show()
	end
end

----Data manipulation functions----
--filters
ns.vars.auction_filter_byname = ""
ns.vars.auction_filter_minavgcost = 0;
ns.vars.auction_filter_bytype = "Show All";

function m.GetFilterByTypeValues()
	return filterByTypeValues;
end

--Professions.Filter
--Returns true if the item should be ignored, false otherwise
--Parameters
--  index - index to check
function m.Filter(index, ForceByType)
	local bytype = ForceByType or ns.vars.auction_filter_bytype
	local byname = ForceByName or ns.vars.auction_filter_byname
	local item = items[index]
	if item.avg == 0 or item.current == 0 then --require prices
		return true
	elseif (not PC.GetCurrentListings(item.item)) and byname == "" then --require current listings
		return true
	elseif not(byname == "" or strfind(strlower(select(1, GetItemInfo(item.item))), strlower(byname), 1, true)) then
		return true
	elseif bytype == "Show All" then
		--do nothing
	elseif not (select(6, GetItemInfo(item.item)) == bytype) then
		return true
	end
	return false
end

--Updates profit and percent diffrence from average
--Subtracts the 5% AH cut
--Does not take AH deposits into account
local function updateItems()
	local item, current, profit, percent, avg;
	for index, data in pairs(items) do
		item = data.item;
		current = PC.GetScannerPrice(item)
		avg = PC.GetAveragePrice(item)
		profit = avg and current and avg * .95 - current 
		percent = avg and current and ((current > avg and (current - avg) / avg * 100) or -((avg - current) / current * 100))
		data.current = current or 0
		data.avg = avg or 0
		data.profit = profit or 0
		data.percent = percent or 0
	end
end

local function sortbyPercent(a,b)
	return a.percent < b.percent
end

local function sortItems()
	table.sort(items, sortbyPercent)
end

--Updates and sorts route price data when new data is available
function m.TYCOON_PC_UPDATE()
	updateItems()
	sortItems()
	m.ClearSelection()
	m.FrameUpdate()
end
m:RegisterEvent("TYCOON_PC_UPDATE")

--Sets and sorts initial price data when the player logs in/reloads ui
function m.PLAYER_LOGIN()
	PopulateItems()
	if PC.IsDBReady() then
		updateItems()
		sortItems()
		m.FrameUpdate()
	end
end
m:RegisterEvent("PLAYER_LOGIN")

m.TYCOON_VARIABLES_CHANGED = function (self, var, old, new)
	if var == "auction_filter_byname" or
		 var == "auction_filter_minavgcost" or
		 var == "auction_filter_bytype" then
		m.ResetScrollFrame()
	end
end
m:RegisterEvent("TYCOON_VARIABLES_CHANGED")

m.TYCOON_CFG_READY = function (self)
	if ns.dbpc.auction_intro_never_again then
		m.HideIntroText()
	end
end
m:RegisterEvent("TYCOON_CFG_READY")