﻿local AddonName,ns = ...
local m = ns:GetModule("Auction");
local p = ns:GetModule("PanelAPI");
local PC = ns:GetModule("PriceCheck");

local index, c = p.AddPanelTab("Auctions");
local ROWS = 6;
local ROWHEIGHT = floor((c:GetHeight() - TYCOON_PANEL_CONTENT_TITLE_HEIGHT) / ROWS)

local selected;

function m.ClearSelection()
	selected = nil
	m.FrameUpdate()
end

local shortdays = {'Su', 'M', 'Tu', 'W', 'Th', 'F', 'Sa'}

--Adjust width to make room for scrollbar
c:SetWidth(c:GetWidth()-26)

--Create Scroll Bar, requires name
local f = CreateFrame("ScrollFrame", "TCPA_Scroll", c, "FauxScrollFrameTemplate")
f:SetWidth(c:GetWidth())
f:SetPoint("TOPLEFT", c, "TOPLEFT", 0, -4)
f:SetPoint("BOTTOMLEFT", c, "BOTTOMLEFT", 0 ,4)
local scrollbar = f;

local f = CreateFrame("Frame", nil, scrollbar)
f:SetSize(28, scrollbar:GetHeight()+10)
f:SetPoint("TOPLEFT", scrollbar, "TOPRIGHT", 0, 6)
f:SetBackdrop({
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true,
	edgeSize = 12, tileSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0},
})

--Create emptyText
local emptyText = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
emptyText:SetAllPoints(true)
emptyText:SetJustifyH("CENTER")
emptyText:SetJustifyV("CENTER")
emptyText:SetText("There is currently nothing to show in this tab. Please check your filters.\n\nThis module requires a recent scan by a supported auction house scanner such as Auctionator.")
emptyText:Hide()

local static = 42; --add static widths here
local columns = {
	{name = "Item", width = (c:GetWidth() - static)*.30 },
	{name = "Avg. Price", width = (c:GetWidth() - static)*.16 },
	{name = "Daily Breakdown", width = (c:GetWidth() - static)*.22},
	{name = "Cur. Price", width = (c:GetWidth() - static)*.16 },
	{name = "%", width = 42 },
	{name = "Profit@Avg", width = (c:GetWidth() - static)*.16 },
}

--Create howToText
local introtext = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
introtext:SetPoint("TOPLEFT", c, "TOPLEFT")
introtext:SetPoint("BOTTOMRIGHT", c, "BOTTOMRIGHT", 0, 20)
introtext:SetJustifyH("CENTER")
introtext:SetJustifyV("CENTER")

introtext:SetText("Welcome to the Tycoon Auctions module!\n\n"..
	"This is an advanced module that can make you lots of gold, but can also lose you lots of gold if used incorrectly. Please read the following tips before proceeding:\n\n"..
	"-You will need some seed money to invest to use this module. Try the other modules first to build up some seed money.\n\n"..
	"-This module can only give information based on the scan data available to it. Scan often over multiple days for the best results.\n\n"..
	"-These prices are based on auctions OBSERVED, not prices PAID. If something is listed with a much higher average price than expected, don't trust it to sell for that amount. Avoid things with percentages over 500.\n\n"..
	"-Hover over the days to view price information for that day during the previous week. Green means the price was above average. Yellow means it was below average, but above current. Red means it was below the current price.\n\n"..
	--"-Be careful when trying to corner the market on crafted items. Raise the price too much and the crafters will jump on your bandwagon.\n"..
	"-A wide variety of items are included in this module to allow for characters of all levels to try their luck on the AH market, but max level trade good items are your best bet when looking for steady income.\n\n"..
	--"-Be aware of recent or upcoming patch changes or holidays. These will affect the market on many items.\n"..
	"-Use the search feature to find items you have invested in and determine if it is a good time to sell those items.\n\n"..
	"Have fun becoming an Auction House Tycoon!")

ns.vars.show_auction_intro_text = true

local introaccept = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introaccept:SetSize(150, 20)
introaccept:SetPoint("BOTTOMRIGHT", c, "BOTTOM", -3, 0)
introaccept:SetText("OK! I'm Ready!")
introaccept:SetScript("OnClick", function()
	m.HideIntroText()
end)

local introdismiss = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introdismiss:SetSize(150, 20)
introdismiss:SetPoint("BOTTOMLEFT", c, "BOTTOM", 3, 0)
introdismiss:SetText("Don't Show Again")
introdismiss:SetScript("OnClick", function()
	ns.dbpc.auction_intro_never_again = true;
	m.HideIntroText()
end)

function m.HideIntroText()
	c.headers:Show()
	introtext:Hide()
	introaccept:Hide()
	introdismiss:Hide()
	
	ns.vars.show_auction_intro_text = false
	m.FrameUpdate();
end
	

--Create Header Container
local headers = CreateFrame("Frame", nil, c, nil);
headers:SetSize(c:GetWidth(), TYCOON_PANEL_CONTENT_TITLE_HEIGHT);
headers:SetPoint("TOP");
headers:Hide()
c.headers = headers

--Create Row Containers
local rows = {};

local function RowOnClick(self)
		--local offset = FauxScrollFrame_GetOffset(scrollbar);
		--if(selected and rows[selected-offset]) then rows[selected-offset]:UnlockHighlight() end
		selected = self:GetID();
		--self:LockHighlight();
		m.FrameUpdate()
end

local function RowOnEnter(self)
	self:LockHighlight();
end

local function RowOnLeave(self)
	local offset = FauxScrollFrame_GetOffset(scrollbar);
	if((not selected) or selected ~= self:GetID()) then
		self:UnlockHighlight()
	end
end

local previous = headers;
for i = 1, ROWS do
	local r = CreateFrame("BUTTON", nil, c, nil, i)
	r:SetSize(c:GetWidth(), ROWHEIGHT)
	r:SetPoint("TOP", previous, "BOTTOM")
	previous = r;
	
	local left = r:CreateTexture(nil, "BACKGROUND")
	local right = r:CreateTexture(nil, "BACKGROUND")
	local center = r:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, ROWHEIGHT)
	right:SetSize(10, ROWHEIGHT)
	center:SetSize(10, ROWHEIGHT)
	left:SetPoint("LEFT", r, "LEFT")
	right:SetPoint("RIGHT", r, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	r:SetHighlightTexture("Interface\\HelpFrame\\HelpFrameButton-Highlight")
	r:GetHighlightTexture():SetTexCoord(0, 1, 0, .578125)
	
	r:SetScript("OnClick", RowOnClick);
	r:SetScript("OnLeave", RowOnLeave);
	
	r:Hide()
	tinsert(rows, r)
end
	
--create headers
local previous = headers;
for k,v in pairs(columns) do
	local h = CreateFrame("BUTTON", nil, headers, nil, k)
	h:SetSize(v.width, headers:GetHeight())
	h:SetPoint("LEFT", previous, k==1 and "LEFT" or "RIGHT")
	previous = h;
	
	local left = h:CreateTexture(nil, "BACKGROUND")
	local right = h:CreateTexture(nil, "BACKGROUND")
	local center = h:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, h:GetHeight())
	right:SetSize(10, h:GetHeight())
	center:SetSize(10, h:GetHeight())
	left:SetPoint("LEFT", h, "LEFT")
	right:SetPoint("RIGHT", h, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	local htext = h:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	htext:SetAllPoints(true)
	htext:SetJustifyH("CENTER")
	htext:SetJustifyV("CENTER")
	htext:SetText(v.name)
end

--create rows
local function DaysOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowDailyTooltip(self:GetParent():GetParent():GetID(), self:GetID(), GameTooltip)
end

local function DaysOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

local function DaysOnClick(self)
	RowOnClick(self:GetParent():GetParent())
end

local function IconOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowItemTooltip(self:GetParent():GetParent():GetID(), GameTooltip)
end

local function IconOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

for i = 1, ROWS do
	local r = rows[i];
	local col = 0;
	local rc; --holds column container frames for individual rows
	local t; --holds temporary working widgets
	local previous;
	
	col = col + 1
	--Column 1
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", r, "LEFT")
	
	t = CreateFrame("BUTTON", nil, rc, nil)
	t:SetSize(32, 32)
	t:SetPoint("LEFT", rc, "LEFT", 5, 0)
	t.icontex = t:CreateTexture(nil, "BORDER")
	t.icontex:SetAllPoints(true)
	t.count = t:CreateFontString(nil, "BORDER", "NumberFontNormal")
	t.count:SetWidth(22)
	t.count:SetPoint("BOTTOM")
	t.count:Hide()
	t:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")	
	t:SetScript("OnEnter", IconOnEnter)
	t:SetScript("OnLeave", IconOnLeave)
	t:SetScript("OnClick", function(self) RowOnClick(self:GetParent():GetParent()) end)
	r.item = t;	
	
	t = r.item:CreateTexture(nil, "ARTWORK")
	t:SetSize(60,60)
	t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	t:SetPoint("CENTER")
	
	t = rc:CreateFontString(nil, "OVERLAY", "GameFontHighlight");
	t:SetHeight(ROWHEIGHT)
	t:SetPoint("LEFT", r.item, "RIGHT", 5, 0);
	t:SetPoint("RIGHT", rc, "RIGHT", 5, 0);
	t:SetJustifyH("LEFT");
	t:SetJustifyV("CENTER");
	t:SetText("[PH]Name Me!");
	r.itemname = t;

	previous = rc; col = col+1;
	--Column 2
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_A_AVG_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.avgprice = t;
	
	previous = rc; col = col+1;
	--Column 3
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	do
		local dprev = rc;
		for i = 1, 7 do
			t = CreateFrame("BUTTON", nil, rc, nil, 8-i)
			t:SetSize(rc:GetWidth()/7,20)
			t:SetPoint("LEFT", dprev, i == 1 and "LEFT" or "RIGHT")
			t:SetScript("OnEnter", DaysOnEnter)
			t:SetScript("OnLeave", DaysOnLeave)
			t:SetScript("OnClick", DaysOnClick)
			dprev = t;
			
			t = dprev:CreateFontString(nil, "BORDER", "GameFontNormal")
			t:SetWidth(dprev:GetWidth())
			t:SetPoint("CENTER")
			t:SetJustifyH("CENTER")
			t:SetJustifyV("CENTER")
			t:SetText(shortdays[i])
			r['d'..i] = t;
		end
	end
	
	previous = rc; col = col+1;
	--Column 4
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_A_CUR_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.curprice = t;
	
	previous = rc; col = col+1;
	--Column 5
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = rc:CreateFontString(nil, "OVERLAY", "GameFontNormal")
	t:SetWidth(rc:GetWidth())
	t:SetPoint("CENTER")
	t:SetJustifyH("CENTER")
	t:SetJustifyV("CENTER")
	t:SetText('0%')
	r.percent = t
	
	previous = rc; col = col+1;
	--Column 6
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_A_PROFIT_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.profit = t;
end

--Create GO button
--TODO: Replace GO button with ADD button
local function goOnClick(self)
	if(selected) then
		m.AddSelected(selected)
	end
end

local f = CreateFrame("BUTTON", nil, c, nil);
f:SetSize(154,43)
f:SetPoint("BOTTOMRIGHT", c:GetParent(), "BOTTOMRIGHT", -28, 7)
f:SetNormalTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetNormalTexture():SetTexCoord(0, 0.59765625, 0, 0.1640625);
f:SetHighlightTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetHighlightTexture():SetTexCoord(0, 0.59765625, 0.1640625, 0.33203125);
f:SetScript("OnClick", goOnClick);
c.go = f

--Create skill filter dropdown
local f = CreateFrame("Frame", "TCP_A_TypeDrop", c, "UIDropDownMenuTemplate")
--f:SetPoint("TOPRIGHT", c, "BOTTOMRIGHT", 0, -12) --DropDowns have the most stupid position systems
f:SetPoint("RIGHT", c.go, "LEFT", 15, 0)
c.TypeDrop = f
do--local space
	local function DropClicked(self, arg1, arg2, checked)
		ns.vars.auction_filter_bytype = self.value
		UIDropDownMenu_SetText(c.TypeDrop, arg1)
	end
	
	local info = {};
	local function DropInit(self, level)
		for k, _ in pairs(m.GetFilterByTypeValues())do
			wipe(info)
			info.text = k
			info.value = k
			info.arg1 = info.text
			info.func = DropClicked
			UIDropDownMenu_AddButton(info, level)
		end
	end
	
	UIDropDownMenu_Initialize(c.TypeDrop, DropInit)
	UIDropDownMenu_SetWidth(c.TypeDrop, 110)
	UIDropDownMenu_SetText(c.TypeDrop, ns.vars.auction_filter_bytype)
end

--Create search edit box
local f = CreateFrame("EditBox", "TCP_A_SearchBox", c, "InputBoxTemplate")
f:SetFontObject("ChatFontNormal")
f:SetAutoFocus(false)
f:SetHeight(24)
f:SetWidth(160)
f:SetPoint("TOPLEFT", c, "BOTTOMLEFT", 22, -15)
f:SetScript("OnEnterPressed", function ()
	c.SearchButton:Click()
end)
c.SearchBox = f

--[[alternate method, missed items in chat window
local old_HandleModifiedItemClick = HandleModifiedItemClick;
HandleModifiedItemClick = function(link, ...)
	if IsModifiedClick("CHATLINK") and c.SearchBox:IsVisible() then
		c.SearchBox:SetText(link)
		return true
	else
		return old_HandleModifiedItemClick(link, ...);
	end
end
]]--

--capture links when visible
local old_ChatEdit_InsertLink = ChatEdit_InsertLink
ChatEdit_InsertLink = function(text, ...)
	if IsModifiedClick("CHATLINK") and c.SearchBox:IsVisible() then
		c.SearchBox:SetText(strmatch(text, "%[(.-)%]"))
		return true
	else
		return old_ChatEdit_InsertLink(text, ...);
	end
end

local f = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
f:SetSize(65, 22)
f:SetPoint("LEFT", c.SearchBox, "RIGHT")
f:SetText("Search")
f:SetScript("OnClick", function ()
	ns.vars.auction_filter_byname = c.SearchBox:GetText()
end)
c.SearchButton = f

local f = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
f:SetSize(65, 22)
f:SetPoint("LEFT", c.SearchButton, "RIGHT")
f:SetText("Reset")
f:SetScript("OnClick", function ()
	ns.vars.auction_filter_byname = ""
	c.SearchBox:SetText("")
	ns.vars.auction_filter_bytype = "Show All"
	UIDropDownMenu_SetText(c.TypeDrop, ns.vars.auction_filter_bytype)
end)
c.ResetButton = f


--Update information displayed
local function FrameUpdate()
	if ns.vars.show_auction_intro_text then return end
	
	local offset = FauxScrollFrame_GetOffset(scrollbar);
	local row, item;
	local index = 0;
	local scrollcount = 0;
	
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
	
	--skip scrolled items
	while scrollcount < offset do
		index = index + 1
		if index <= m.GetNumItems() and not m.Filter(index) then
			scrollcount = scrollcount + 1
		else
			if index == selected then selected = nil end;
		end
	end
	
	if index > m.GetNumItems() then --We are scrolled past the end of the list, reset
		ResetScrollFrame()
		return --This function recalled by ResetScrollFrame
	end
	
	m:mdebug("Starting FrameUpdate rows", index, offset, scrollcount)
	
	for i = 1, ROWS do
		index = index + 1
		
		while index <= m.GetNumItems() and m.Filter(index) do
			if index == selected then selected = nil end;
			index = index + 1
		end
		
		row = rows[i];
		
		m:mdebug("Updating row", i, index)
		
		if(index > m.GetNumItems()) then
			row:Hide();
		else
			scrollcount = scrollcount + 1
			item = m.GetItem(index)
			
			--TODO: Item caching code, perhaps in panel API
			
			--Column 1
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(item);
			row.item.icontex:SetTexture(itemTexture)
			row.itemname:SetText(itemName)
			
			--Column 2
			local avgprice = m.GetAverage(index) --also used in column 3
			MoneyFrame_Update(row.avgprice, avgprice or 0)
			
			--Column 3
			local cday = select(1, CalendarGetDate())
			local curprice = m.GetCurrent(index) --also used in column 4
			
			local day
			for i = 1, 7 do
				local dayprice = PC.GetScannerPrice(item, 8-i)
				
				day = row['d'..i];
				day:SetFontObject(dayprice and ((dayprice < avgprice and dayprice > curprice and "GameFontNormal") or (dayprice < avgprice and "GameFontRed") or "GameFontGreen") or "GameFontDisable")
				day:SetText(shortdays[((cday - 2 + i)%7) + 1])
			end
			
			--Column 4
			MoneyFrame_Update(row.curprice, curprice or 0)
			
			--Column 5
			row.percent:SetFontObject(m.GetPercent(index) > 0 and "GameFontRed" or "GameFontGreen")
			row.percent:SetText((abs(m.GetPercent(index)) or 0)..'%');
			
			--Column 6
			MoneyFrame_Update(row.profit, max(m.GetProfit(index), 0))
			
			--Highlight
			if(selected and index == selected) then
				row:LockHighlight()
			else
				row:UnlockHighlight()
			end
			
			--Show
			row:Show()
			row:SetID(index)
		end
	end
	
	m:mdebug("Ending FrameUpdate rows", index, offset, scrollcount)
	
	while index <  m.GetNumItems() do --we only want to advance to the final index, not past it
		index = index + 1
		if not m.Filter(index) then
			scrollcount = scrollcount + 1;
		end
	end
	
	if scrollcount == 0 then 
		emptyText:Show()
	else
		emptyText:Hide()
	end
	
	m:mdebug("Ending FrameUpdate", index, offset, scrollcount)
	
	--Update scrollframe
	FauxScrollFrame_Update(scrollbar, scrollcount, 6, ROWHEIGHT, nil, nil, nil, nil, nil, nil, true)
end
m.FrameUpdate = FrameUpdate;
scrollbar:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, ROWHEIGHT, FrameUpdate) end);
c:SetScript("OnShow", FrameUpdate);

function ResetScrollFrame()
	FauxScrollFrame_SetOffset(scrollbar, 0)
	FrameUpdate()
end
m.ResetScrollFrame = ResetScrollFrame
		