﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"Gatherer")
local PC = ns:GetModule("PriceCheck")
local SF = ns:GetModule("StatusFrame")
local LT = ns:GetModule("LootTracker")
local PI = ns:GetModule("ProfInfo")
local slave = ManaCore:RegisterModule();

local routes = {}
local activeRoute

local filterBySkillValues = {
	["Show All"] = true,
	["Current Character"] = true,
}

function m.RegisterRoute(name, desc, items, zoneid, profs, route)
	--TODO: Remove newprofs code when we know all old routes have been converted
	local newprofs;
	if (profs[1]) then --old system, convert
		newprofs = {}
		for _, prof in pairs(profs) do 
			newprofs[prof] = 1;
		end
		profs = newprofs
	end
	
	tinsert(routes, {name = name, desc=desc, items = items, zoneid=zoneid, profs=profs, route = route})
	
	for prof, _ in pairs(profs) do
		filterBySkillValues[prof] = true;
	end
end
--make this global for easy access from route register data files
RegisterTycoonGatherRoute = m.RegisterRoute;

----Route Engine----
function m.StartRoute(index)
	m:mdebug("StartRoute", index)
	activeRoute = routes[index];
	ns.vars.active_module = "Gatherer";
	ManaCore:GetModule("MapMarker").SetRoute(m.GetZoneMapID(index), m.GetRoute(index));
	LT:SetItems(m.GetItems(index));
	SF:SetText(m.GetName(index))
end

function m.StatusOnHover(frame)
	GameTooltip:SetOwner(frame)
	m.ShowPrimaryTooltip(nil, GameTooltip)
	GameTooltip:AddLine("\nLeft-click to open world map to zone.")
	GameTooltip:Show()
end

function m.StatusOnClick(frame)
	ToggleFrame(WorldMapFrame) --WorldMapFrame:Show() causes errors if not previously opened manually
	SetMapByID(activeRoute.zoneid)
end

----UI Accessors----
function m.GetNumRoutes()
	return #routes;
end

function m.GetName(index)
	if routes[index] then return routes[index].name end
end

function m.GetDesc(index)
	if routes[index] then return routes[index].desc end
end

function m.GetItems(index)
	if routes[index] then return routes[index].items end
end

function m.GetZoneMapID(index)
	if routes[index] then return routes[index].zoneid end
end

function m.GetZoneName(index)
	if routes[index] then
		SetMapByID(routes[index].zoneid)
		return GetMapZones(GetCurrentMapContinent())[select(1,GetCurrentMapZone())];
	end
end

function m.GetRoute(index)
	if routes[index] then return routes[index].route end
end

function m.GetGPH(index)
	if routes[index] then return routes[index].gph or 0 end
end

local goltex		= "|TInterface\\MoneyFrame\\UI-GoldIcon:12:12:4:0|t"
local siltex	= "|TInterface\\MoneyFrame\\UI-SilverIcon:12:12:4:0|t"
local coptex	= "|TInterface\\MoneyFrame\\UI-CopperIcon:12:12:4:0|t"

local function buildMoneyString(amt)
	local gol, sil, cop;
	local out = ""
	
	gol = floor(amt / 10000)
	sil = floor((amt%10000)/100)
	cop = amt%100
	
	if gol ~= 0 then out = gol..goltex.." "	end
	if sil ~= 0 or gol ~= 0 then out = out..sil..siltex.." " end
	out = out..cop..coptex
	
	return out
end

function m.ShowAddTooltip(index, tooltip)
	if routes[index] then
		items = routes[index].items
		if #items > 1 then
			tooltip:ClearLines()
			tooltip:AddLine("Additional Items")
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
			local r, b, g
			for k, v in pairs(items) do
				if(k ~= 1) then
					if v.id == "coin" then
						tooltip:AddLine("Coin",1,1,1)
						tooltip:AddDoubleLine("Est/hr",  buildMoneyString(v.rate), 1, 1, 1, 1, 1, 1)
					else
						itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(v.id);		
						r, g, b = GetItemQualityColor(itemRarity or 0)
						tooltip:AddLine(itemName or "Unknown", r, g, b)
						tooltip:AddDoubleLine("|cffffffffEst #/hr:|cff00ff00"..v.rate, buildMoneyString(PC.GetPrice(v.id) or 0), 0, 1, 0, 1, 1, 1)
					end
				end
			end
			tooltip:Show()
		end
	end
end

--Calling this function with no index will show the activeRoute tooltip
--(index for active route is not garunteed to remain static)
function m.ShowPrimaryTooltip(index, tooltip)
	local route = (index and routes[index]) or ((not index) and activeRoute)
	if route then
		tooltip:ClearLines()
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(route.items[1].id);
		tooltip:AddLine("Primary Item: |c"..select(4, GetItemQualityColor(itemRarity or 0))..itemName);
		tooltip:AddLine("");
		tooltip:AddLine(route.desc, 1, 1, 1, 1)
		tooltip:Show()
	end
end

----Data manipulation functions----
--filters
ns.vars.gatherer_filter_byskill = "Current Character"

function m.GetFilterBySkillValues()
	return filterBySkillValues;
end

--Professions.Filter
--Returns true if the item should be ignored, false otherwise
--Parameters
--  index - index to check
--	ForceBySkill - force the by skill value to
function m.Filter(index, ForceBySkill)
	local byskill = ForceBySkill or ns.vars.gatherer_filter_byskill
	if byskill == "Show All" then
		--do nothing
	elseif byskill == "Current Character" then
		for prof, skill in pairs(routes[index].profs) do
			if not PI:PlayerHasProfession(prof, skill) then
				return true
			end
		end
	else
		for prof, _ in pairs(routes[index].profs) do
			if not (prof == byskill) then
				return true
			end
		end
	end
	return false;
end

local function updateGPH()
	local gph;
	for index, route in pairs(routes) do
		gph = 0;
		for key, item in pairs(route.items) do
			if item.id == "coin" then
				gph = gph + item.rate
			else
				gph = gph + ((PC.GetPrice(item.id) or 0) * item.rate)
			end
		end
		route.gph = gph
	end
end

local function sortbyGPH(a,b)
	return a.gph > b.gph
end

local function sortRoutes()
	table.sort(routes, sortbyGPH)
end
	
--Updates and sorts route price data when new data is available
function m.TYCOON_PC_UPDATE()
	updateGPH()
	sortRoutes()
	m.ClearSelection()
	m.FrameUpdate()
end
m:RegisterEvent("TYCOON_PC_UPDATE")

--Sets and sorts initial price data when the player logs in/reloads ui
function m.PLAYER_LOGIN()
	if PC.IsDBReady() then
		updateGPH()
		sortRoutes()
		m.FrameUpdate()
	end
end
m:RegisterEvent("PLAYER_LOGIN")

m.TYCOON_LOOTTRACKER_GPH_UPDATE = function(self)
	if ns.vars.active_module == "Gatherer" and not ns.dbpc.statusframe_gph_hidden then
		SF:ShowRate()
		SF:SetRateText(Tycoon.BuildMoneyString(LT:GetGPH(), true)..'/hr')
		m:mdebug("GPH text updated", LT:GetGPH(), Tycoon.BuildMoneyString(LT:GetGPH(), true))
	end
end
m:RegisterEvent("TYCOON_LOOTTRACKER_GPH_UPDATE")

m.TYCOON_VARIABLES_CHANGED = function (self, var, old, new)
	if var == "gatherer_filter_byskill" then
		m.ResetScrollFrame()
	end
end
m:RegisterEvent("TYCOON_VARIABLES_CHANGED")

m.TYCOON_CFG_READY = function (self)
	if ns.dbpc.gatherer_intro_never_again then
		m.HideIntroText()
	end
end
m:RegisterEvent("TYCOON_CFG_READY")

slave.MANACORE_VARS_CHANGED = function(self, var, old, new)
	--TODO: this should techncly only fire if Gatherer is the active module
	if var == "active_product" then
		if new ~= "Tycoon" then
			LT:Reset()
		end
	end
end