﻿local AddonName,ns = ...
local m = ns:GetModule("Gatherer");
local p = ns:GetModule("PanelAPI");
local PC = ns:GetModule("PriceCheck");
local PI = ns:GetModule("ProfInfo");

local index, c = p.AddPanelTab("Gathering");
local ROWS = 6;
local ROWHEIGHT = floor((c:GetHeight() - TYCOON_PANEL_CONTENT_TITLE_HEIGHT) / ROWS)

local selected;

function m.ClearSelection()
	selected = nil
	m.FrameUpdate()
end

--Adjust width to make room for scrollbar
c:SetWidth(c:GetWidth()-26)

--Create Scroll Bar, requires name
local f = CreateFrame("ScrollFrame", "TCPG_Scroll", c, "FauxScrollFrameTemplate")
f:SetWidth(c:GetWidth())
f:SetPoint("TOPLEFT", c, "TOPLEFT", 0, -4)
f:SetPoint("BOTTOMLEFT", c, "BOTTOMLEFT", 0 ,4)
local scrollbar = f;

local f = CreateFrame("Frame", nil, scrollbar)
f:SetSize(28, scrollbar:GetHeight()+10)
f:SetPoint("TOPLEFT", scrollbar, "TOPRIGHT", 0, 6)
f:SetBackdrop({
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true,
	edgeSize = 12, tileSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0},
})

--Create emptyText
local emptyText = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
emptyText:SetAllPoints(true)
emptyText:SetJustifyH("CENTER")
emptyText:SetJustifyV("CENTER")
emptyText:SetText("There is currently nothing to show in this tab. Please check your filters and make sure TycoonContent is enabled.")
emptyText:Hide()

--Create introtext
local introtext = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
introtext:SetPoint("TOPLEFT", c, "TOPLEFT")
introtext:SetPoint("BOTTOMRIGHT", c, "BOTTOMRIGHT", 0, 20)
introtext:SetJustifyH("CENTER")
introtext:SetJustifyV("CENTER")

introtext:SetText("Welcome to the Tycoon Gathering module!\n\n"..
	"This module shows you which gathering routes will earn you the most gold from your gathering professions. Here are some basic tips to get you started:\n\n"..
	"-Select a route you would like to run, and hit GO! Clicking on the status bar will open the map to the zone for your route.\n\n"..
	"-By default, only routes for your current character's professions are shown. Use the drop down to view other routes.\n\n"..
	"-Follow the line as closely as you can when gathering. It is designed to take you into tracking range of the nodes.\n\n"..
	"-Routes and estimates are tuned for level apropriate travel speeds. Your mileage may vary if you have a very slow or very fast mount for the content you are gathering.\n\n"..
	"-Unless otherwise noted, ignore underground nodes when running routes. Underground nodes appear darker on the minimap when outdoors.\n\n"..
	"Have fun bustin' your mailbox with Tycoon!")

ns.vars.gatherer_show_intro_text = true

local introaccept = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introaccept:SetSize(150, 20)
introaccept:SetPoint("BOTTOMRIGHT", c, "BOTTOM", -3, 0)
introaccept:SetText("OK! I'm Ready!")
introaccept:SetScript("OnClick", function()
	m.HideIntroText()
end)

local introdismiss = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introdismiss:SetSize(150, 20)
introdismiss:SetPoint("BOTTOMLEFT", c, "BOTTOM", 3, 0)
introdismiss:SetText("Don't Show Again")
introdismiss:SetScript("OnClick", function()
	ns.dbpc.gatherer_intro_never_again = true;
	m.HideIntroText()
end)

function m.HideIntroText()
	c.headers:Show()
	introtext:Hide()
	introaccept:Hide()
	introdismiss:Hide()
	
	ns.vars.gatherer_show_intro_text = false
	m.FrameUpdate();
end

local static = 42; --add static widths here
local columns = {
	{name = "Route", width = (c:GetWidth() - static)*.30 },
	{name = "Cur. Price", width = (c:GetWidth() - static)*.18 },
	{name = "Est #/Hr", width = (c:GetWidth() - static)*.12 },
	{name = "Competition", width = (c:GetWidth() - static)*.12 },
	{name = "Trend", width = (c:GetWidth() - static)*.10 },
	{name = "Add.", width = 42 },
	{name = "Total Est. G/Hr", width = (c:GetWidth() - static)*.18 },
}

--Create Header Container
local headers = CreateFrame("Frame", nil, c, nil);
headers:SetSize(c:GetWidth(), TYCOON_PANEL_CONTENT_TITLE_HEIGHT);
headers:SetPoint("TOP");
headers:Hide()
c.headers = headers;

--Create Row Containers
local rows = {};

local function RowOnClick(self)
		selected = self:GetID();
		m.FrameUpdate()
end

local function RowOnEnter(self)
	self:LockHighlight();
end

local function RowOnLeave(self)
	if((not selected) or selected ~= self:GetID()) then
		self:UnlockHighlight()
	end
end

local previous = headers;
for i = 1, ROWS do
	local r = CreateFrame("BUTTON", nil, c, nil, i)
	r:SetSize(c:GetWidth(), ROWHEIGHT)
	r:SetPoint("TOP", previous, "BOTTOM")
	r:Hide()
	previous = r;
	
	local left = r:CreateTexture(nil, "BACKGROUND")
	local right = r:CreateTexture(nil, "BACKGROUND")
	local center = r:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, ROWHEIGHT)
	right:SetSize(10, ROWHEIGHT)
	center:SetSize(10, ROWHEIGHT)
	left:SetPoint("LEFT", r, "LEFT")
	right:SetPoint("RIGHT", r, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	r:SetHighlightTexture("Interface\\HelpFrame\\HelpFrameButton-Highlight")
	r:GetHighlightTexture():SetTexCoord(0, 1, 0, .578125)
	
	r:SetScript("OnClick", RowOnClick);
	r:SetScript("OnLeave", RowOnLeave);
	
	tinsert(rows, r)
end
	
--create headers
local previous = headers;
for k,v in pairs(columns) do
	local h = CreateFrame("BUTTON", nil, headers, nil, k)
	h:SetSize(v.width, headers:GetHeight())
	h:SetPoint("LEFT", previous, k==1 and "LEFT" or "RIGHT")
	previous = h;
	
	local left = h:CreateTexture(nil, "BACKGROUND")
	local right = h:CreateTexture(nil, "BACKGROUND")
	local center = h:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, h:GetHeight())
	right:SetSize(10, h:GetHeight())
	center:SetSize(10, h:GetHeight())
	left:SetPoint("LEFT", h, "LEFT")
	right:SetPoint("RIGHT", h, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	local htext = h:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	htext:SetAllPoints(true)
	htext:SetJustifyH("CENTER")
	htext:SetJustifyV("CENTER")
	htext:SetText(v.name)
end

--create rows
local function AddOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowAddTooltip(self:GetParent():GetParent():GetID(), GameTooltip)
end

local function AddOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

local function IconOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowPrimaryTooltip(self:GetParent():GetParent():GetID(), GameTooltip)
end

local function IconOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

for i = 1, ROWS do
	local r = rows[i];
	local col = 0;
	local rc; --holds column container frames for individual rows
	local t; --holds temporary working widgets
	local previous;
	
	col = col + 1
	--Column 1
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", r, "LEFT")
	
	t = CreateFrame("BUTTON", nil, rc, nil)
	t:SetSize(32, 32)
	t:SetPoint("LEFT", rc, "LEFT", 5, 0)
	t.icontex = t:CreateTexture(nil, "BORDER")
	t.icontex:SetAllPoints(true)
	t:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")	
	t:SetScript("OnEnter", IconOnEnter)
	t:SetScript("OnLeave", IconOnLeave)
	t:SetScript("OnClick", function(self) RowOnClick(self:GetParent():GetParent()) end)
	r.item = t;
	
	
	t = r.item:CreateTexture(nil, "ARTWORK")
	t:SetSize(60,60)
	t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	t:SetPoint("CENTER")
	
	t = rc:CreateFontString(nil, "OVERLAY", "GameFontHighlight");
	t:SetHeight(ROWHEIGHT)
	t:SetPoint("LEFT", r.item, "RIGHT", 5, 0);
	t:SetPoint("RIGHT", rc, "RIGHT", 5, 0);
	t:SetJustifyH("LEFT");
	t:SetJustifyV("CENTER");
	t:SetText("[PH]Name Me!");
	r.route = t;

	previous = rc; col = col+1;
	--Column 2
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_G_AVG_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.avgprice = t;
	
	previous = rc; col = col+1;
	--Column 3
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = rc:CreateFontString(nil, "OVERLAY", "GameFontGreen");
	t:SetPoint("LEFT", rc, "LEFT", 5, 0);
	t:SetPoint("RIGHT", rc, "RIGHT", 5, 0);
	t:SetJustifyH("CENTER");
	t:SetJustifyV("CENTER");
	t:SetText("[PH]150");
	r.perhour = t;
	
	previous = rc; col = col+1;
	--Column 4
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = p.CreateSignalBars(nil, rc, col)
	t:SetPoint("CENTER", rc, "CENTER")
	r.competition = t;
	
	previous = rc; col = col+1;
	--Column 5
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = p.CreateSignalBars(nil, rc, col)
	t:SetPoint("CENTER", rc, "CENTER")
	r.trend = t;
	
	previous = rc; col = col+1;
	--Column 6
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = CreateFrame("BUTTON", nil, rc, nil)
	t:SetSize(32, 32)
	t:SetPoint("LEFT", rc, "LEFT", 5, 0)
	t.icontex = t:CreateTexture(nil, "BORDER")
	t.icontex:SetAllPoints(true)
	t:SetScript("OnEnter", AddOnEnter)
	t:SetScript("OnLeave", AddOnLeave)
	t:SetScript("OnClick", function(self) RowOnClick(self:GetParent():GetParent()) end)
	--r.add:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	r.add = t
	
	t = r.add:CreateTexture(nil, "ARTWORK")
	t:SetSize(60,60)
	t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	t:SetPoint("CENTER")
	
	previous = rc; col = col+1;
	--Column 7
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_G_EST_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.estimate = t;
end

--Create GO button
local function goOnClick(self)
	if(selected) then
		m.StartRoute(selected)
		HideUIPanel(c:GetParent())
	end
end

local f = CreateFrame("BUTTON", nil, c, nil);
f:SetSize(154,43)
f:SetPoint("BOTTOMRIGHT", c:GetParent(), "BOTTOMRIGHT", -28, 7)
f:SetNormalTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetNormalTexture():SetTexCoord(0, 0.59765625, 0, 0.1640625);
f:SetHighlightTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetHighlightTexture():SetTexCoord(0, 0.59765625, 0.1640625, 0.33203125);
f:SetScript("OnClick", goOnClick);
c.go = f

--Create skill filter dropdown
local f = CreateFrame("Frame", "TCP_G_SkillDrop", c, "UIDropDownMenuTemplate")
f:SetPoint("RIGHT", c.go, "LEFT", 10, 0)
c.SkillDrop = f
do--local space for easy copy/paste in case we do more dropdowns 
	local function DropClicked(self, arg1, arg2, checked)
		ns.vars.gatherer_filter_byskill = self.value
		UIDropDownMenu_SetText(c.SkillDrop, arg1)
	end
	
	local info = {};
	local function DropInit(self, level)
		for k, _ in pairs(m.GetFilterBySkillValues())do
			wipe(info)
			info.text = PI:GetLocaleProfessionName(k)
			info.value = k
			info.arg1 = info.text
			info.func = DropClicked
			UIDropDownMenu_AddButton(info, level)
		end
	end
	
	UIDropDownMenu_Initialize(c.SkillDrop, DropInit)
	UIDropDownMenu_SetWidth(c.SkillDrop, 110)
	UIDropDownMenu_SetText(c.SkillDrop, ns.vars.gatherer_filter_byskill)
end

--Update information displayed
local function FrameUpdate()
	if ns.vars.gatherer_show_intro_text then return end
	
	local offset = FauxScrollFrame_GetOffset(scrollbar);
	local row, itemtable;
	local index = 0
	local scrollcount = 0
	
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
	
		--skip scrolled items
	while scrollcount < offset do
		index = index + 1
		if not m.Filter(index) then
			scrollcount = scrollcount + 1
		else
			if index == selected then selected = nil end;
		end
	end
	
	m:mdebug("Starting FrameUpdate rows", index, offset, scrollcount)
	
	for i = 1, ROWS do
		index = index + 1
		
		while index <= m.GetNumRoutes() and m.Filter(index) do
			if index == selected then selected = nil end;
			index = index + 1
		end
		
		row = rows[i];
		
		m:mdebug("Updating row", i, index)
		
		if(index > m.GetNumRoutes()) then
			row:Hide();
		else
			scrollcount = scrollcount + 1
			itemtable = m.GetItems(index);
			
			--TODO: Item caching code, perhaps in panel API
			
			--Column 1
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(itemtable[1].id);
			row.item.icontex:SetTexture(itemTexture)
			row.route:SetText(m.GetName(index))
			
			--Column 2
			MoneyFrame_Update(row.avgprice, PC.GetPrice(itemtable[1].id) or 0)
			
			--Column 3
			row.perhour:SetText(itemtable[1].rate)
			
			--Column 4
			row.competition:SetStrength(PC.GetCompetition(itemtable[1].id) or 0);
			
			--Column 5
			row.trend:SetStrength(PC.GetTrend(itemtable[1].id) or 0);
			
			--Column 6
			if(#itemtable > 1) then
				row.add.icontex:SetTexture("Interface\\Icons\\INV_Misc_Bag_10")
			else
				row.add.icontex:SetTexture("Interface\\PaperDoll\\UI-PaperDoll-Slot-Bag")
			end
			
			--Column 7
			MoneyFrame_Update(row.estimate, m.GetGPH(index))
			
			--Highlight
			if(selected and index == selected) then
				row:LockHighlight()
			else
				row:UnlockHighlight()
			end
			
			--Show
			row:Show()
			row:SetID(index)
		end
	end
	
	m:mdebug("Ending FrameUpdate rows", index, offset, scrollcount)
	
	while index <  m.GetNumRoutes() do --we only want to advance to the final index, not past it
		index = index + 1
		if not m.Filter(index) then
			scrollcount = scrollcount + 1;
		end
	end
	
	if scrollcount == 0 then 
		emptyText:Show()
	else
		emptyText:Hide()
	end
	
	m:mdebug("Ending FrameUpdate", index, offset, scrollcount)
	
	--Update scrollframe
	FauxScrollFrame_Update(scrollbar, scrollcount, 6, ROWHEIGHT, nil, nil, nil, nil, nil, nil, true)
end
m.FrameUpdate = FrameUpdate;
scrollbar:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, ROWHEIGHT, FrameUpdate) end);
c:SetScript("OnShow", FrameUpdate);

function ResetScrollFrame()
	FauxScrollFrame_SetOffset(scrollbar, 0)
	FrameUpdate()
end
m.ResetScrollFrame = ResetScrollFrame