﻿local AddonName,ns = ...
local m = ns:GetModule("Professions");
local p = ns:GetModule("PanelAPI");
local PC = ns:GetModule("PriceCheck");
local PI = ns:GetModule("ProfInfo");

local index, c = p.AddPanelTab("Crafting");
local ROWS = 6;
local ROWHEIGHT = floor((c:GetHeight() - TYCOON_PANEL_CONTENT_TITLE_HEIGHT) / ROWS)

local selected;

function m.ClearSelection()
	selected = nil
	m.FrameUpdate()
end

--Adjust width to make room for scrollbar
c:SetWidth(c:GetWidth()-26)

--Create Scroll Bar, requires name
local f = CreateFrame("ScrollFrame", "TCPP_Scroll", c, "FauxScrollFrameTemplate")
f:SetWidth(c:GetWidth())
f:SetPoint("TOPLEFT", c, "TOPLEFT", 0, -4)
f:SetPoint("BOTTOMLEFT", c, "BOTTOMLEFT", 0 ,4)
local scrollbar = f;

local f = CreateFrame("Frame", nil, scrollbar)
f:SetSize(28, scrollbar:GetHeight()+10)
f:SetPoint("TOPLEFT", scrollbar, "TOPRIGHT", 0, 6)
f:SetBackdrop({
	edgeFile = "Interface/Tooltips/UI-Tooltip-Border", tile = true,
	edgeSize = 12, tileSize = 16, insets = {left = 0, right = 0, top = 0, bottom = 0},
})

--Create emptyText
local emptyText = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
emptyText:SetAllPoints(true)
emptyText:SetJustifyH("CENTER")
emptyText:SetJustifyV("CENTER")
emptyText:SetText("There is currently nothing to show in this tab. Please check your filters and make sure TycoonContent is enabled.")
emptyText:Hide()

--Create introtext
local introtext = c:CreateFontString(nil, "OVERLAY", "GameFontWhite")
introtext:SetPoint("TOPLEFT", c, "TOPLEFT")
introtext:SetPoint("BOTTOMRIGHT", c, "BOTTOMRIGHT", 0, 20)
introtext:SetJustifyH("CENTER")
introtext:SetJustifyV("CENTER")

introtext:SetText("Welcome to the Tycoon Crafting module!\n\n"..
	"This module shows you which items you can craft for profit. Here are some basic tips to get you started:\n\n"..
	"-Select an item you would like to craft, and hit GO! Clicking on the status bar will open a profession window to the selected recipie if known. Shift-clicking on the status bar will open a shopping list.\n\n"..
	"-The module will substitute raw materials when they are cheaper than a processed material (ex. 2x[Elementium Ore] can replace an [Elementium Bar]).\n\n"..
	"-By default, only recipies for your current character's professions are shown. Use the drop down to view other professions.\n\n"..
	"-The module DOES NOT filter out recipies you don't know. This is intentional so you can determine which recipies will be profitable to acquire.\n\n"..
	"-The best items to make for a steady profit are consumable item enhancers, such as belt buckles and leg enchants.\n\n"..
	"Have fun bustin' your mailbox with Tycoon!")

ns.vars.crafting_show_intro_text = true

local introaccept = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introaccept:SetSize(150, 20)
introaccept:SetPoint("BOTTOMRIGHT", c, "BOTTOM", -3, 0)
introaccept:SetText("OK! I'm Ready!")
introaccept:SetScript("OnClick", function()
	m.HideIntroText()
end)

local introdismiss = CreateFrame("Button", nil, c, "UIPanelButtonTemplate")
introdismiss:SetSize(150, 20)
introdismiss:SetPoint("BOTTOMLEFT", c, "BOTTOM", 3, 0)
introdismiss:SetText("Don't Show Again")
introdismiss:SetScript("OnClick", function()
	ns.dbpc.crafting_intro_never_again = true;
	m.HideIntroText()
end)

function m.HideIntroText()
	c.headers:Show()
	introtext:Hide()
	introaccept:Hide()
	introdismiss:Hide()
	
	ns.vars.crafting_show_intro_text = false
	m.FrameUpdate();
end

local static = 42; --add static widths here
local columns = {
	{name = "Recipie", width = (c:GetWidth() - static)*.30 },
	{name = "Sell Price", width = (c:GetWidth() - static)*.16 },
	{name = "Mats", width = 42 },
	{name = "Cost", width = (c:GetWidth() - static)*.16 },
	{name = "Competition", width = (c:GetWidth() - static)*.12 },
	{name = "Trend", width = (c:GetWidth() - static)*.10 },
	{name = "Profit", width = (c:GetWidth() - static)*.16 },
}

--Create Header Container
local headers = CreateFrame("Frame", nil, c, nil);
headers:SetSize(c:GetWidth(), TYCOON_PANEL_CONTENT_TITLE_HEIGHT);
headers:SetPoint("TOP");
headers:Hide()
c.headers = headers

--Create Row Containers
local rows = {};

local function RowOnClick(self)
		--local offset = FauxScrollFrame_GetOffset(scrollbar);
		--if(selected and rows[selected-offset]) then rows[selected-offset]:UnlockHighlight() end
		selected = self:GetID();
		--self:LockHighlight();
		m.FrameUpdate()
end

local function RowOnEnter(self)
	self:LockHighlight();
end

local function RowOnLeave(self)
	local offset = FauxScrollFrame_GetOffset(scrollbar);
	if((not selected) or selected ~= self:GetID()) then
		self:UnlockHighlight()
	end
end

local previous = headers;
for i = 1, ROWS do
	local r = CreateFrame("BUTTON", nil, c, nil, i)
	r:SetSize(c:GetWidth(), ROWHEIGHT)
	r:SetPoint("TOP", previous, "BOTTOM")
	r:Hide()
	previous = r;
	
	local left = r:CreateTexture(nil, "BACKGROUND")
	local right = r:CreateTexture(nil, "BACKGROUND")
	local center = r:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, ROWHEIGHT)
	right:SetSize(10, ROWHEIGHT)
	center:SetSize(10, ROWHEIGHT)
	left:SetPoint("LEFT", r, "LEFT")
	right:SetPoint("RIGHT", r, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	r:SetHighlightTexture("Interface\\HelpFrame\\HelpFrameButton-Highlight")
	r:GetHighlightTexture():SetTexCoord(0, 1, 0, .578125)
	
	r:SetScript("OnClick", RowOnClick);
	r:SetScript("OnLeave", RowOnLeave);
	
	tinsert(rows, r)
end
	
--create headers
local previous = headers;
for k,v in pairs(columns) do
	local h = CreateFrame("BUTTON", nil, headers, nil, k)
	h:SetSize(v.width, headers:GetHeight())
	h:SetPoint("LEFT", previous, k==1 and "LEFT" or "RIGHT")
	previous = h;
	
	local left = h:CreateTexture(nil, "BACKGROUND")
	local right = h:CreateTexture(nil, "BACKGROUND")
	local center = h:CreateTexture(nil, "BACKGROUND")
	left:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	right:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	center:SetTexture("Interface\\AuctionFrame\\UI-AuctionItemNameFrame")
	left:SetSize(10, h:GetHeight())
	right:SetSize(10, h:GetHeight())
	center:SetSize(10, h:GetHeight())
	left:SetPoint("LEFT", h, "LEFT")
	right:SetPoint("RIGHT", h, "RIGHT")
	center:SetPoint("LEFT", left, "RIGHT")
	center:SetPoint("RIGHT", right, "LEFT")
	left:SetTexCoord(0, 0.078125, 0, 1)
	right:SetTexCoord(0.75, 0.828125, 0, 1)
	center:SetTexCoord(0.078125, .75, 0, 1)
	
	local htext = h:CreateFontString(nil, "OVERLAY", "GameFontHighlight")
	htext:SetAllPoints(true)
	htext:SetJustifyH("CENTER")
	htext:SetJustifyV("CENTER")
	htext:SetText(v.name)
end

--create rows
local function MatsOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowMatsTooltip(self:GetParent():GetParent():GetID(), GameTooltip)
end

local function MatsOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

local function IconOnEnter(self)
	RowOnEnter(self:GetParent():GetParent())
	GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
	m.ShowRecipieTooltip(self:GetParent():GetParent():GetID(), GameTooltip)
end

local function IconOnLeave(self)
	RowOnLeave(self:GetParent():GetParent())
	GameTooltip:Hide()
end

for i = 1, ROWS do
	local r = rows[i];
	local col = 0;
	local rc; --holds column container frames for individual rows
	local t; --holds temporary working widgets
	local previous;
	
	col = col + 1
	--Column 1
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", r, "LEFT")
	
	t = CreateFrame("BUTTON", nil, rc, nil)
	t:SetSize(32, 32)
	t:SetPoint("LEFT", rc, "LEFT", 5, 0)
	t.icontex = t:CreateTexture(nil, "BORDER")
	t.icontex:SetAllPoints(true)
	t.count = t:CreateFontString(nil, "BORDER", "NumberFontNormal")
	t.count:SetWidth(22)
	t.count:SetPoint("BOTTOM")
	t.count:Hide()
	t:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")	
	t:SetScript("OnEnter", IconOnEnter)
	t:SetScript("OnLeave", IconOnLeave)
	t:SetScript("OnClick", function(self) RowOnClick(self:GetParent():GetParent()) end)
	r.item = t;
	
	
	t = r.item:CreateTexture(nil, "ARTWORK")
	t:SetSize(60,60)
	t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	t:SetPoint("CENTER")
	
	t = rc:CreateFontString(nil, "OVERLAY", "GameFontHighlight");
	t:SetHeight(ROWHEIGHT)
	t:SetPoint("LEFT", r.item, "RIGHT", 5, 0);
	t:SetPoint("RIGHT", rc, "RIGHT", 5, 0);
	t:SetJustifyH("LEFT");
	t:SetJustifyV("CENTER");
	t:SetText("[PH]Name Me!");
	r.recipie = t;

	previous = rc; col = col+1;
	--Column 2
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_P_PRICE_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.price = t;
	
	previous = rc; col = col+1;
	--Column 3
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = CreateFrame("BUTTON", nil, rc, nil)
	t:SetSize(32, 32)
	t:SetPoint("LEFT", rc, "LEFT", 5, 0)
	t.icontex = t:CreateTexture(nil, "BORDER")
	t.icontex:SetAllPoints(true)
	t.icontex:SetTexture("Interface\\Icons\\INV_Misc_Bag_10")
	t:SetScript("OnEnter", MatsOnEnter)
	t:SetScript("OnLeave", MatsOnLeave)
	t:SetScript("OnClick", function(self) RowOnClick(self:GetParent():GetParent()) end)
	--r.add:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square","ADD")
	r.mats = t
	
	t = r.mats:CreateTexture(nil, "ARTWORK")
	t:SetSize(60,60)
	t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
	t:SetPoint("CENTER")
	
	previous = rc; col = col+1;
	--Column 4
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_P_COST_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.cost = t;
	
	previous = rc; col = col+1;
	--Column 5
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = p.CreateSignalBars(nil, rc, col)
	t:SetPoint("CENTER", rc, "CENTER")
	r.competition = t;
	
	previous = rc; col = col+1;
	--Column 6
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	t = p.CreateSignalBars(nil, rc, col)
	t:SetPoint("CENTER", rc, "CENTER")
	r.trend = t;
	
	previous = rc; col = col+1;
	--Column 7
	rc = CreateFrame("FRAME", nil, r, nil, col)
	rc:SetSize(columns[col].width, r:GetHeight())
	rc:SetPoint("LEFT", previous, "RIGHT")
	
	--SmallMoneyFrameTemplate requires a frame name
	t = CreateFrame("Frame", "TCP_P_PROFIT_"..i, rc, "SmallMoneyFrameTemplate", col);
	t:SetPoint("CENTER", rc, "CENTER");
	MoneyFrame_SetType(t, "AUCTION");
	r.profit = t;
end

--Create GO button
local function goOnClick(self)
	if(selected) then
		m.StartCraft(selected)
		HideUIPanel(c:GetParent());
	end
end

local f = CreateFrame("BUTTON", nil, c, nil);
f:SetSize(154,43)
f:SetPoint("BOTTOMRIGHT", c:GetParent(), "BOTTOMRIGHT", -28, 7)
f:SetNormalTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetNormalTexture():SetTexCoord(0, 0.59765625, 0, 0.1640625);
f:SetHighlightTexture('Interface\\AddOns\\Tycoon\\Media\\Tycoon_Panel_Go')
f:GetHighlightTexture():SetTexCoord(0, 0.59765625, 0.1640625, 0.33203125);
f:SetScript("OnClick", goOnClick);
c.go = f

--Create skill filter dropdown
local f = CreateFrame("Frame", "TCP_P_SkillDrop", c, "UIDropDownMenuTemplate")
f:SetPoint("RIGHT", c.go, "LEFT", 10, 0) --DropDowns have the most stupid position systems
c.SkillDrop = f
do--local space
	local function DropClicked(self, arg1, arg2, checked)
		ns.vars.professions_filter_byskill = self.value
		UIDropDownMenu_SetText(c.SkillDrop, arg1)
	end
	
	local info = {};
	local function DropInit(self, level)
		for k, _ in pairs(m.GetFilterBySkillValues())do
			wipe(info)
			info.text = PI:GetLocaleProfessionName(k)
			info.value = k
			info.arg1 = info.text
			info.func = DropClicked
			UIDropDownMenu_AddButton(info, level)
		end
	end
	
	UIDropDownMenu_Initialize(c.SkillDrop, DropInit)
	UIDropDownMenu_SetWidth(c.SkillDrop, 110)
	UIDropDownMenu_SetText(c.SkillDrop, ns.vars.professions_filter_byskill)
end

--Update information displayed
local function FrameUpdate()
	if ns.vars.crafting_show_intro_text then return end
	
	local offset = FauxScrollFrame_GetOffset(scrollbar);
	local row, item;
	local index = 0;
	local scrollcount = 0;
	
	local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
	
	--skip scrolled items
	while scrollcount < offset do
		index = index + 1
		if not m.Filter(index) then
			scrollcount = scrollcount + 1
		else
			if index == selected then selected = nil end;
		end
	end
	
	m:mdebug("Starting FrameUpdate rows", index, offset, scrollcount)
	
	for i = 1, ROWS do
		index = index + 1
		
		while index <= m.GetNumCrafts() and m.Filter(index) do
			if index == selected then selected = nil end;
			index = index + 1
		end
		
		row = rows[i];
		
		m:mdebug("Updating row", i, index)
		
		if(index > m.GetNumCrafts()) then
			row:Hide();
		else
			scrollcount = scrollcount + 1
			item = m.GetItem(index)
			
			--TODO: Item caching code, perhaps in panel API
			
			--Column 1
			itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(item);
			row.item.icontex:SetTexture(itemTexture)
			row.recipie:SetText(m.GetRecipieName(index))
			if m.GetAmount(index) > 1 then row.item.count:SetText(m.GetAmount(index)) else row.item.count:Hide() end
			
			--Column 2
			MoneyFrame_Update(row.price, PC.GetPrice(item) or 0)
			
			--Column 3
			--Do Nothing, only shows tooltip on enter
			
			--Column 4
			MoneyFrame_Update(row.cost, m.GetCost(index))
			
			--Column 5
			row.competition:SetStrength(PC.GetCompetition(item, 2) or 0);
			
			--Column 6
			row.trend:SetStrength(PC.GetTrend(item) or 0);
			
			--Column 7
			MoneyFrame_Update(row.profit, max(m.GetProfit(index), 0))
			
			--Highlight
			if(selected and index == selected) then
				row:LockHighlight()
			else
				row:UnlockHighlight()
			end
			
			--Show
			row:Show()
			row:SetID(index)
		end
	end
	
	m:mdebug("Ending FrameUpdate rows", index, offset, scrollcount)
	
	while index <  m.GetNumCrafts() do --we only want to advance to the final index, not past it
		index = index + 1
		if not m.Filter(index) then
			scrollcount = scrollcount + 1;
		end
	end
	
	if scrollcount == 0 then 
		emptyText:Show()
	else
		emptyText:Hide()
	end
	
	m:mdebug("Ending FrameUpdate", index, offset, scrollcount)
	
	--Update scrollframe
	FauxScrollFrame_Update(scrollbar, scrollcount, 6, ROWHEIGHT, nil, nil, nil, nil, nil, nil, true)
end
m.FrameUpdate = FrameUpdate;
scrollbar:SetScript("OnVerticalScroll", function(self, offset) FauxScrollFrame_OnVerticalScroll(self, offset, ROWHEIGHT, FrameUpdate) end);
c:SetScript("OnShow", FrameUpdate);

function ResetScrollFrame()
	FauxScrollFrame_SetOffset(scrollbar, 0)
	FrameUpdate()
end
m.ResetScrollFrame = ResetScrollFrame
		