﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"Professions")
local PC = ns:GetModule("PriceCheck")
local SF = ns:GetModule("StatusFrame")
local PI = ns:GetModule("ProfInfo")
local SL = ns:GetModule("ShoppingList")
--local LT = ns:GetModule("LootTracker")
local slave = ManaCore:RegisterModule();

local crafts = {}
local activeCraft --used to open proper recipie when status frame is clicked
local subs = {}
local temp = {} --generic temp table to prevent garbage table generation

--List of values to be displayed in filtering drop down
local filterBySkillValues = {
	["Show All"] = true,
	["Current Character"] = true,
}

function m.RegisterCraft(Profession, ItemID, CraftID, Reagents, AmountCreated)
	tinsert(crafts, {skill=Profession, item=ItemID, recipie=CraftID, reagents=Reagents, amt=AmountCreated or 1})
	filterBySkillValues[Profession] = true;
end
--make this global for easy access from route register data files
RegisterTycoonCraft = m.RegisterCraft;

function m.RegisterSub(item, ...)
	subs[item] = {}
	local k, v
	for i=1, select('#', ...), 2 do
		k, v = select(i, ...), select(i+1, ...)
		subs[item][k] = v
	end
end
--make this global for easy access from route register data files
RegisterTycoonSubstitute = m.RegisterSub;


----Craft Engine----
function m.StartCraft(index)
	m:mdebug("StartCraft", index)
	ns.vars.active_module = "Professions";
	activeCraft = crafts[index]
	SF:SetText(m.GetRecipieName(index))
	SF:HideRate()
	
	SL:SetList("Tycoon Crafting")
	SL:ClearList()
	
	local items = crafts[index].reagents
	local subbed
	for item, rate in pairs(items) do
		subbed = m.GetSubs(item, temp)
		for subitem, subprice in pairs(temp) do
		  SL:AddToList(subitem, PC.GetPrice(subitem), (subbed and m.GetSubRate(item, subitem)*rate or rate))
		end
	end
end

function m.StatusOnHover(frame)
	GameTooltip:SetOwner(frame)
	m:mdebug("Professions.StatusOnHover")
	m.ShowMatsTooltip(nil, GameTooltip)
	GameTooltip:AddLine("\nLeft-click to open tradeskill panel.")
	GameTooltip:AddLine("Shift-click to open the shopping list.")
	GameTooltip:Show()
end

function m.StatusOnClick(frame)
	if IsShiftKeyDown() then
		SL:Show()
	elseif not PI:GoToTradeskill(activeCraft.skill, activeCraft.recipie) then
		UIErrorsFrame:AddMessage("You don't know that recipie B.", 1.0, 0.0, 0.0, 53, 5)
		m:mdebug("Tradeskill", activeCraft.skill, "Recipe", activeCraft.recipie)
	end
end

----UI Accessors----
function m.GetNumCrafts()
	return #crafts;
end

function m.GetRecipieName(index)
	if crafts[index] then return (PI:GetLocaleProfessionName(crafts[index].skill) or "")..': '..(select(1,GetItemInfo(crafts[index].item)) or "") end
end

function m.GetItem(index)
	if crafts[index] then return crafts[index].item end
end

function m.GetAmount(index)
	if crafts[index] then return crafts[index].amt end
end

function m.GetCost(index)
	if crafts[index] then return crafts[index].cost or 0 end
end

function m.GetProfit(index)
	if crafts[index] then return floor(crafts[index].profit) or 0 end
end

--Calling this function with no index will show the activeRoute tooltip
--(index for active route is not garunteed to remain static)
function m.ShowMatsTooltip(index, tooltip)
	--TODO: Make this work with subs
	local craft = (index and crafts[index]) or ((not index) and activeCraft)
	m:mdebug("ShowMatsTooltip", index, tooltip, craft and craft.item)
	if craft then
		local items = craft.reagents
		tooltip:ClearLines()
		tooltip:AddLine("Materials")
		local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice;
		local hex
		local subbed
		for item, rate in pairs(items) do
			subbed = m.GetSubs(item, temp)
			for subitem, subprice in pairs(temp) do
				itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(subitem);		
				_, _, _, hex = GetItemQualityColor(itemRarity or 0)
				tooltip:AddDoubleLine((subbed and m.GetSubRate(item, subitem)*rate or rate)..'x|c'..hex..(itemName or "Unknown")..'|r', ns.BuildMoneyString(PC.GetPrice(subitem) or 0) .. " each", 1, 1, 1, 1, 1, 1)
			end
		end
		tooltip:Show()
	end
end

--Calling this function with no index will show the activeRoute tooltip
--(index for active route is not garunteed to remain static)
function m.ShowRecipieTooltip(index, tooltip)
	local craft = (index and crafts[index]) or ((not index) and activeCraft)
	if craft then
		tooltip:ClearLines()
		tooltip:SetHyperlink("enchant:"..craft.recipie)
		tooltip:Show()
	end
end

----Data manipulation functions----
--filters
ns.vars.professions_filter_byskill = "Current Character"

function m.GetFilterBySkillValues()
	return filterBySkillValues;
end

--Professions.Filter
--Returns true if the item should be ignored, false otherwise
--Parameters
--  index - index to check
--	ForceBySkill - force the by skill value to
function m.Filter(index, ForceBySkill)
	local byskill = ForceBySkill or ns.vars.professions_filter_byskill
	if byskill == "Show All" then
		--do nothing
	elseif byskill == "Current Character" then
		if not PI:PlayerHasProfession(crafts[index].skill) then
			return true
		end
	else
		if not (crafts[index].skill == byskill) then
			return true
		end
	end
	return false;
end

--Professions.CheckSubs
--Returns the cheapest option availble to obtain an item through simple crafting.
--Parameters:
--	item - the item to check for subs
--	tbl [optional] - a table to store results (to prevent garbage table creation)
--returns: SubCheaper(true/false), tbl{[itemid] = price}
--	tbl is always returned. If SubCheaper is false, it will contain the passed item.
-- 	"price" is the price to obtain a single of "item"	
function m.GetSubs(item, tbl)
	if tbl then wipe(tbl) else tbl = {} end
	
	if not subs[item] then
		tbl[item] = PC.GetPrice(item) or 0
		return false, tbl
	else
		local subtotal, price = 0, 0
		for subid, ratio in pairs(subs[item]) do
			price = (PC.GetPrice(subid) or 0) * ratio
			tbl[subid] = price
			subtotal = subtotal + price
		end
		local price = PC.GetPrice(item) or 0
		if price > subtotal then
			return true, tbl
		else
			wipe(tbl)
			tbl[item] = PC.GetPrice(item) or 0
			return false, tbl
		end
	end
end

--a workaround due to not returning rate with GetSubs
function m.GetSubRate(item, subitem)
	return subs[item] and subs[item][subitem]
end

local function updateProfit()
	local profit;
	local cost;
	for index, craft in pairs(crafts) do
		local cost = 0
		for item, rate in pairs(craft.reagents) do
			m.GetSubs(item, temp)
			for _, price in pairs(temp) do
				cost = cost + (price * rate)
			end
		end
		craft.cost = cost
		craft.profit = (PC.GetPrice(craft.item) or 0) * craft.amt * .95 - cost;
	end
end

local function sortbyProfit(a,b)
	return a.profit > b.profit
end

local function sortCrafts()
	table.sort(crafts, sortbyProfit)
end

--Updates and sorts route price data when new data is available
function m.TYCOON_PC_UPDATE()
	updateProfit()
	sortCrafts()
	m.ClearSelection()
	m.FrameUpdate()
end
m:RegisterEvent("TYCOON_PC_UPDATE")

--Sets and sorts initial price data when the player logs in/reloads ui
function m.PLAYER_LOGIN()
	if PC.IsDBReady() then
		updateProfit()
		sortCrafts()
		m.FrameUpdate()
	end
end
m:RegisterEvent("PLAYER_LOGIN")

m.TYCOON_VARIABLES_CHANGED = function (self, var, old, new)
	if var == "professions_filter_byskill" then
		m.ResetScrollFrame()
	end
end
m:RegisterEvent("TYCOON_VARIABLES_CHANGED")

m.TYCOON_CFG_READY = function (self)
	if ns.dbpc.crafting_intro_never_again then
		m.HideIntroText()
	end
end
m:RegisterEvent("TYCOON_CFG_READY")