﻿local AddonName,ns = ...
local m,i = ns:RegisterModule(nil,"PriceCheck")
local slave = ManaCore:RegisterModule() --used to listen for neccecary ManaCore events

--Stored as scanner so that in the future we can add
--additional scanner modules easily so long as they
--all share an interface.
local Scanner = ns:GetModule("Auctioneer")
if not Scanner then Scanner = ns:GetModule("Auctionator") end
local ScannerName = Scanner and Scanner.ModuleName
TycoonPC = m; --global reference for use by data files

--[[ PriceCheck.lua
	This code wraps up price/competition/trend data lookups
	to be used with Tycoon. If a supported auction addon is
	installed the addon will use that data. Otherwise, the
	default data provided in the contend directory will be
	used instead. The default data is only provided to allow
	the addon to function with no other data available and a
	disclaimer will be displayed at startup if no other data
	source is found.
--]]

local data = {} --all the data, this may be changed to a module table with appropriate events/metatable
local staticdata = {}

function m.RegisterItem(ID, Price, Competition, Trend)
	data[ID] = {price=Price, comp=Competition, trend=Trend}
end

function m.RegisterStaticItem(ID, Price)
	staticdata[ID] = Price
end

function m.RegisterItems(table)
	for k, v in pairs(table) do
		m.RegisterItem(unpack(v))
	end
end

function m.RegisterStaticItems(table)
	for k, v in pairs(table) do
		m.RegisterStaticItem(unpack(v))
	end
end

--used to provide auction module with an interesting items list
function m.IsDBReady()
	if Scanner then return Scanner.IsDBReady() end
	return true
end

function m.GetDataTable()
	return data;
end

function m.GetPrice(id)
	--return vendor price when set
	if staticdata[id] then return staticdata[id] end
	
	--force vendor sell value for grey items
	local itemRarity, itemSellPrice
	_, _, itemRarity, _, _, _, _, _, _, _, itemSellPrice = GetItemInfo(id)
	if itemRarity == 0 then return itemSellPrice end
	
	local price
	--5/13/2011, no longer returns default price if scanner available
	if Scanner then
		price = Scanner.GetItemPrice(id)
	else
		price = data[id] and data[id].price
	end
	return price;
end

--Returns only the scanner price
function m.GetScannerPrice(id, daysago)
	local price = Scanner and Scanner.GetItemPrice(id, daysago);
	return price;
end

--Requires scanner to function
function m.GetAveragePrice(id)
	local price = Scanner and Scanner.GetAveragePrice(id);
	return price;
end

--Requires scanner to function
function m.GetCurrentListings(id)
	return Scanner and Scanner.GetCurrentListings(id)
end

function m.GetCompetition(id, base)
	local comp = data[id] and data[id].comp;
	local temp;
	if Scanner then
		temp = Scanner.GetItemListings(id)
		comp = temp and floor(max(min(temp/(base or 10),5)), 0) or comp;
	end
	return comp;
end

function m.GetTrend(id)
	local trend = data[id] and data[id].trend;
	local temp
	if Scanner then
		temp = Scanner.GetItemTrend(id)
		if temp then
			if temp > .10 then trend = 5
			elseif temp > .05 then trend = 4
			elseif temp < -.05 then trend = 2
			elseif temp < -.10 then trend = 1
			elseif temp < -.15 then trend = 0
			else trend = 3 end-- -.5 < temp < .5
		end
	end
	return trend;
end

StaticPopupDialogs["TYCOON_NOSCANNER"] = {
	text = "No supported auction house scanner was found. Tycoon will use a set of default prices. To get accurate data for your server, please download and install a supported scanner. (Auctionator or Auctioneer)",
	button1 = OKAY,
	timeout = 0,
	whileDead = true,
}

StaticPopupDialogs["TYCOON_MULTISCANNER"] = {
	text = "Both Auctioneer and Auctionator were found. Because Auctioneer interferes with Auctionator's scan ability, only Auctioneer data will be used. If you wish to use Auctionator's data, please disable Auctioneer.",
	button1 = OKAY,
	timeout = 0,
	whileDead = true,
}

local warnonce = true
slave.MANACORE_VARS_CHANGED = function(self, var, old, new)
	if warnonce and var == "active_product" then
		if new == "Tycoon" then
			if not Scanner then
				StaticPopup_Show("TYCOON_NOSCANNER")
			end
			if ns:GetModule("Auctioneer") and ns:GetModule("Auctionator") then
				StaticPopup_Show("TYCOON_MULTISCANNER")
			end
			warnonce = false
		end
	end
end
slave:RegisterEvent("MANACORE_VARS_CHANGED")

--Update Event Handlers/Refire
--Note: will see more use when additional scanners are implemented
function m.AUCTIONATOR_SCAN_PARTIAL()
	if ScannerName == "Auctionator" then
		ns:FireEvent("TYCOON_PC_UPDATE")
	end
end
m:RegisterEvent("AUCTIONATOR_SCAN_PARTIAL")

function m.AUCTIONATOR_SCAN_FULL()
	if ScannerName == "Auctionator" then
		ns:FireEvent("TYCOON_PC_UPDATE")
	end
end
m:RegisterEvent("AUCTIONATOR_SCAN_FULL")

function m.AUCTIONATOR_DB_READY()
	if ScannerName == "Auctionator" then
		ns:FireEvent("TYCOON_PC_UPDATE")
	end
end
m:RegisterEvent("AUCTIONATOR_DB_READY")

function m.AUCTIONEER_SCAN_UPDATE()
	if ScannerName == "Auctioneer" then
		ns:FireEvent("TYCOON_PC_UPDATE")
	end
end
m:RegisterEvent("AUCTIONEER_SCAN_UPDATE")
