﻿--Professions.lua
--Contains the ProfInfo module which provides character tradeskill data to Tycoon.
--Note: Unlocalized names are used in this file for table indexing only.
--	Printed data should use localized names from m.GetLocaleProfessionName
local _,ns = ...
local m,i = ns:RegisterModule(nil,'ProfInfo')

local deformat = LibStub("LibDeformat-3.0")

local primary = {
	--['English Name'] = true;
	['Alchemy'] = true,
	['Blacksmithing'] = true,
	['Enchanting'] = true,
	['Engineering'] = true,
	['Inscription'] = true,
	['Jewelcrafting'] = true,
	['Leatherworking'] = true,
	['Tailoring'] = true,
	
	['Herbalism'] = true,
	['Mining'] = true,
	['Skinning'] = true,
}

local secondary = {
	['Archaeology'] = true,
	['Cooking'] = true,
	['First Aid'] = true,
	['Fishing'] = true,
}

local profIDs = {
	--['English Name'] = {75, 150, 225, 300, 375, 450, 525, },
	['Alchemy'] = {2259, 3101, 3464, 11611, 28596, 51304, 80731, },
	['Blacksmithing'] = {2018, 3100, 3538, 9785, 29844, 51300, 76666, },
	['Enchanting'] = {7411, 7412, 7413, 13920, 28029, 51313, 74258, },
	['Engineering'] = {4036, 4037, 4038, 12656, 30350, 51306, 82774, },
	['Inscription'] = {45357, 45358, 45359, 45360, 45361, 45363, 86008, },
	['Jewelcrafting'] = {25229, 25230, 28894, 28895, 28897, 51311, 73318, },
	['Leatherworking'] = {2108, 3104, 3811, 10662, 32549, 51302, 81199, },
	['Tailoring'] = {3908, 3909, 3910, 12180, 26790, 51309, 75156, },
	
	--These skills are problematic. Their actual skills are no longer known by the player
	['Herbalism'] = {2366, 2368, 3570, 11993, 28695, 50300, 74519, }, --These are actually for the Herb Gathering spell
	--['Mining'] = {2575, 2576, 3564, 10248, 29354, 50310, 74517, },
	['Mining'] = {2656}, --This is for Smelting
	['Skinning'] = {8613, 8617, 8618, 10768, 32678, 50305, 74522, },
	
	['Archaeology'] = {78670, 88961, 89718, 89719, 89720, 89721, 89722, },
	['Cooking'] = {2550, 3102, 3413, 18260, 33359, 51296, 88053, },
	['First Aid'] = {3273, 3274, 7924, 10846, 27028, 45542, 74559, },
	['Fishing'] = {7620, 7731, 7732, 18248, 33095, 51294, 88868, },
}

local profByID = {}
for prof, data in pairs(profIDs) do
	for _, id in pairs(data) do
		profByID[id] = prof;
	end
end

-- /script for i,v in pairs{GetProfessions()} do local name,tex = GetProfessionInfo(v); print(name,tex) end
local profByIcon = {
	['Interface\\Icons\\Trade_Alchemy'] = 'Alchemy',
	['Interface\\Icons\\Trade_BlackSmithing'] = 'Blacksmithing',
	['Interface\\Icons\\Trade_Engraving'] = 'Enchanting',
	['Interface\\Icons\\Trade_Engineering'] = 'Engineering',
	['Interface\\Icons\\INV_Inscription_Tradeskill01'] = 'Inscription',
	['Interface\\Icons\\INV_Misc_Gem_01'] = 'Jewelcrafting', --Jewelcrafting 1 uses a different icon then 2-7
	['Interface\\Icons\\INV_Misc_Gem_02'] = 'Jewelcrafting', 
	['Interface\\Icons\\Trade_LeatherWorking'] = 'Leatherworking',
	['Interface\\Icons\\Trade_Tailoring'] = 'Tailoring',
	
	['Interface\\Icons\\Trade_Herbalism'] = 'Herbalism',
	['Interface\\Icons\\INV_Pick_02'] = 'Mining',
	['Interface\\Icons\\INV_Misc_Pelt_Wolf_01'] = 'Skinning',
	
	['INTERFACE\\ICONS\\trade_archaeology'] = 'Archaeology',
	['Interface\\Icons\\INV_Misc_Food_15'] = 'Cooking',
	['Interface\\Icons\\Spell_Holy_SealOfSacrifice'] = 'First Aid',	
	['Interface\\Icons\\Trade_Fishing'] = 'Fishing',
}

--[[Cannot be used due to Jewelcrafting's double icon
local profByName = {}
for k,v in pairs(profByIcon) do
	profByName[v] = k
end
m.ProfessionsByName = profByName
]]
m.ProfessionsByIcon = profByIcon
m.ProfessionsByID = profByID
m.ProfessionIDLists = profIDs

-- Since only localized names are returned by tradeskill funcs, we need to delocalize. Icons seem to be the easiest way to do so.
m.TexToProfession = function (self,tex)
	if tex and profByIcon[tex] then return profByIcon[tex] end
end


m.GetLocaleProfessionName = function (self, engName)
	--STUB: Will eventually return the localized profession name
	return engName
end

--[[Depreciated
m.GetProfessionInfo = function (self,i)
	local pname, texture, rank, maxRank = GetProfessionInfo(i)
	local actual = m:GetProfessionName(texture)
	if actual then
		return actual, {name = pname,
			texture = texture,
			rank = rank,
			maxRank = maxRank,
			index = i
		}
	end
end
]]

--[[Depreciated
m.GetProfessions = function (self,name)
	local uname = UnitName('player')

	if not(name) or (name == uname) then
		local t = {GetProfessions()}
		local t2 = {}
		for k,v in pairs(t) do
			m:mdebug("Fetching profession index " .. tostring(v), k)
			local n,info = m:GetProfessionInfo(v)
			t2[n] = info
		end
		return t2
	elseif ns.dbs[name] then
		return ns.dbs[name].profs
	end
end
]]

--[[Depreciated
m.GetProfessionList = function (self,name)
	local profs = m:GetProfessions(name)
	local t = {}
	local i = 0
	for k,v in pairs(profs) do
		i = i+1
		t[i] = k
	end
	return t
end
]]

function m.PlayerHasProfession(self, profession, skill)
	if skill then 
		if ns.dbc.profs[profession] and ns.dbc.profs[profession].skill >= skill then
			return true 
		else 
			return false 
		end
	end
	
	if ns.dbc.profs[profession] then return true else return false end
end

local function ValidateSkill(tradeskill)
	for _,v in pairs(profIDs[tradeskill]) do
		if IsSpellKnown(v) then
			return true
		end
	end
	ns.dbc.profs[tradeskill] = nil;
	return false;
end

m.UpdateProfessions = function (self)
	--first, make sure saved skills are still present
	for tradeskill, recipies in pairs(ns.dbc.profs) do
		ValidateSkill(tradeskill)
	end
	
	--check for skills and update levels
	local profindexs = {GetProfessions()}
	for _,v in pairs(profindexs) do
		local name, icon, skillLevel, maxSkillLevel, numAbilities, spelloffset, skillLine, skillModifier = GetProfessionInfo(v)
		
		--convert to english name for storage
		local engname = m:TexToProfession(icon);
		assert(engname ~= nil, "No known profession for icon:" .. icon)
		--create table if not present
		if not ns.dbc.profs[engname] then ns.dbc.profs[engname] = {} end
		--save skill level
		ns.dbc.profs[engname].skill = skillLevel;
	end
	--TODO: Store recipies
end
		
-- "|cffffd000|Htrade:51306:355:450:580000006E62A84:4/uP71yVYR33kFAAAQ+SQAgBAAAAOAI6PAAAcDgAAcEIAAAAAEAAAAAAA|h[Engineering]|h|r"
--[[ Encoding:
	|cffffd000 = color
	|H = hyperlink signifier
	trade: = link type (trade == profession link)
	:51306 = spellID
	:355 = Current skill
	:450 = Max skill
	:580000006E62A84 = UnitGUID

	... = Encoded data
	
	|h[Engineering]|h = Link title
	|r = end-of-color signifier
]]--
--[[depreciated
local GSIFromProfessionLink = function (self,link)
	local spellID = tonumber(string.match(link,'trade:(%d+)'))
	return GetSpellInfo(spellID)
end
--]]

--[[depreciated
m.GetProfessionLink = function (self)
	if not(TradeSkillFrame and GetTradeSkillTexture()) then return end
	
	local current = not(IsTradeSkillLinked()) and CURRENT_TRADESKILL == GetTradeSkillLine() and CURRENT_TRADESKILL
	if not(current) then return end
	
	local link = GetTradeSkillListLink()
	if link then
		local n = m:GetProfessionName(GetTradeSkillTexture())
		if n and ns.dbc.profs and ns.dbc.profs[n] then
			ns.dbc.profs[n].link = link
			ns:debug('UpdateProfessionLink('..n..')',link)
		end
	end
end
--]]

local function clearTradeskillFilters()
	--get rid of all filters and open all subcategories
	--May need next line, from blizzard UI
	--TradeSkillFrame.filterTbl = {hasMaterials = false, hasSkillUp = false, subClassValue = -1, slotValue = -1 };
	TradeSkillOnlyShowSkillUps(false);
  TradeSkillOnlyShowMakeable(false);
  --SetTradeSkillSubClassFilter(-1, 1, 1);
  SetTradeSkillInvSlotFilter(-1, 1, 1);
  TradeSkillUpdateFilterBar();
  CloseDropDownMenus();
	ExpandTradeSkillSubClass(0) --0 = expand all
end

m.OpenTradeskill = function (self, tradeskill)
	if not profIDs[tradeskill] then return false end
	for _,v in pairs(profIDs[tradeskill]) do
		if IsSpellKnown(v) then
			m:mdebug("Open Trade Skill:", v, select(1,GetSpellInfo(v)))
			CastSpellByName(select(1,GetSpellInfo(v)))
			return true
		end
	end
	return false
end

m.GoToTradeskill = function (self, tradeskill, recipie)
	if self:OpenTradeskill(tradeskill) then 
		clearTradeskillFilters()
	else
		m:mdebug("Tradeskill not known a")
		return false;
	end
	local link, skill, i;
	m:mdebug("Check for skill:", tradeskill, recipie, "Number of Tradeskill", GetNumTradeSkills())
	for i = 1, GetNumTradeSkills() do
		link = GetTradeSkillRecipeLink(i)
		skill = tonumber(select(3,string.find(link or "", "^|%x+|Henchant:(.+)|h%[.+%]")) or 0)
		m:mdebug("Loop skill: ", skill, "Link:", link, "NumSkills", i)
		if skill == recipie then
			SelectTradeSkill(i)
			TradeSkillFrame_Update() --Must be called after SelectTradeSkill because the API function forgot to
			m:mdebug("Skill found", i)
			return true
		end
	end
	m:mdebug("Skill not found, lines checked:", i, "tradeSkill", tradeskill, " Recipe", recipie)
	CloseTradeSkill()
	return false
end

local function cacheProfNames()
	--STUB: this function will make sure localized profession names are cached
end

m.TYCOON_CFG_READY = function (self)
	ns.dbs = ManaCore.dbs	-- Server db
	ns.dbc = ManaCore.dbc	-- dbs[name] shortcut
	
	if ns.dbc and	not ns.dbc.profs then ns.dbc.profs = {} end

	--local prof1, prof2, arch, fishing, cooking = GetProfessions()
	--if (prof1 or prof2 or arch or fishing or cooking) then self:TRADE_SKILL_UPDATE() else m:RegisterEvent('PLAYER_ENTERING_WORLD') end
end

--[[fires far too often for our purposes
m.TRADE_SKILL_UPDATE = function (self)
	m:mdebug("Trade Skills Updated")
	m:UpdateProfessions()
	--m:GetProfessionLink()
end
--]]

m.SKILL_LINES_CHANGED = function (self)
	m:UpdateProfessions()
	m:UnregisterEvent("SKILL_LINES_CHANGED")
end
m:RegisterEvent("SKILL_LINES_CHANGED")

m.CHAT_MSG_SYSTEM = function (self, msg)
	--ERR_LEARN_ABILITY_S: on proffession max increase or new skill
	--ERR_LEARN_RECIPE_S: on new recipies
	--TODO: add recipe check when we start using it
	local match = deformat(msg, ERR_LEARN_ABILITY_S)
	
	if match then m:UpdateProfessions() end	
end

--m.PLAYER_ENTERING_WORLD = m.TRADE_SKILL_UPDATE
--m.TRADE_SKILL_SHOW = m.TRADE_SKILL_UPDATE
