﻿local AddonName,ns = ...
Tycoon = ns

ns.variables = {}
ns.variables.active_module = false
ManaCore:RegisterCoreModule(ns,'Tycoon',
    {proxy='db', lookup='TycoonCFG', event='TYCOON_CFG_CHANGED_GLOBAL', ready = 'TYCOON_CFG_READY_GLOBAL',
    	default = {
      	autoscan = true,
    	}
    },
    {proxy='dbpc', lookup='TycoonCFGPC', event='TYCOON_CFG_CHANGED', ready = 'TYCOON_CFG_READY',
			default = {
				statusframe_detached = false,
				lock_frame = false,
				statusframe_gph_hidden = false,
			}
		},
    {proxy='vars', event='TYCOON_VARIABLES_CHANGED',ready='TYCOON_VARIABLES_READY'}
)

local function deepcopy(object)
  local lookup_table = {}
  local function _copy(object)
    if type(object) ~= "table" then
        return object
    elseif lookup_table[object] then
        return lookup_table[object]
    end
    local new_table = {}
    lookup_table[object] = new_table
    for index, value in pairs(object) do
        new_table[_copy(index)] = _copy(value)
    end
    return setmetatable(new_table, getmetatable(object))
  end
  return _copy(object)
end

local API,i = Tycoon:RegisterModule(nil,'API')

local function CheckDbVersion()
	if Tycoon_AuctionDB.version ~= 2 then
		API:mdebug("Updating AuctionDB to version 2")
		
		local realm = GetRealmName()
		local temp = Tycoon_AuctionDB
		Tycoon_AuctionDB = {}
		Tycoon_AuctionDB[realm] = {}
		Tycoon_AuctionDB[realm].auctionator = {}
		--can't be sure player entering world has fired, so we'll just copy the
		--data to both factions. Not a perfect solution, but it will have to do
		--Startup cleanup may want to check both factions
		Tycoon_AuctionDB[realm].auctionator["Alliance"] = deepcopy(temp)
		Tycoon_AuctionDB[realm].auctionator["Horde"] = deepcopy(temp)
		
		Tycoon_AuctionDB.version = 2
	end
end

do
    API.TYCOON_CFG_READY_GLOBAL = function (self)
				Tycoon_AuctionDB = Tycoon_AuctionDB or {version = 2} --cannot be a module lookup table because of how often it changes
				CheckDbVersion()
        local realm = GetRealmName()
        Tycoon_AuctionDB[realm] = Tycoon_AuctionDB[realm] or {}
    		--setmetatable(Tycoon_AuctionDB,{__mode = 'v'}) -- maybe later...
        ns.dbs = ManaCore.dbs
        ns.dbc = ManaCore.dbc
        ns.adb = Tycoon_AuctionDB[realm]
				
				--Depreciated
				if not ns.dbs.scandays then ns.dbs.scandays = {} end
    end
end

----Utility Functions----
local goltex	= "|TInterface\\MoneyFrame\\UI-GoldIcon:12:12:4:0|t"
local siltex	= "|TInterface\\MoneyFrame\\UI-SilverIcon:12:12:4:0|t"
local coptex	= "|TInterface\\MoneyFrame\\UI-CopperIcon:12:12:4:0|t"

--returns a formatted string including coin icons
--amt: the value to convert, in copper
--gcd: greatest currency denomination, only shows gold, or silver, or coppper
function Tycoon.BuildMoneyString(amt, gcd)
	if not amt then return 0 end
	local gol, sil, cop;
	local out = ""
	
	gol = floor(amt / 10000)
	sil = floor((amt%10000)/100)
	cop = amt%100
	
	if gol ~= 0 then out = gol..goltex.." "	end
	if (sil ~= 0 or gol ~= 0) and ((not gcd) or gol == 0 ) then out = out..sil..siltex.." " end
	if (not gcd) or (gol == 0 and sil == 0) then out = out..cop..coptex end
	
	return out
end